@@ -50,7 +50,6 @@ class AsyncFile:
         self.sock = sock
         self.mode = mode
         self.name = name
-        self.softspace = 0
         self.nWriteErrors = 0
 
         self.wpending = u''
@@ -94,11 +93,13 @@ class AsyncFile:
         """
         return self.wpending.rfind('\n') + 1
 
-    def close(self):
+    def close(self, closeit=0):
         """
         Public method to close the file.
+        
+        @param closeit flag to indicate a close ordered by the debugger code (boolean)
         """
-        if not self.closed:
+        if closeit and not self.closed:
             self.flush()
             self.sock.close()
             self.closed = 1
@@ -201,7 +202,7 @@ class AsyncFile:
         """
         Public method to move the filepointer.
         
-        This method is not supported and always raises an
+        @exception IOError This method is not supported and always raises an
         IOError.
         """
         raise IOError, '[Errno 29] Illegal seek'
