@@ -23,8 +23,8 @@ try:
             QT_TRANSLATE_NOOP('Variable Types', 'String'), \
             QT_TRANSLATE_NOOP('Variable Types', 'Unicode String'), \
             QT_TRANSLATE_NOOP('Variable Types', 'Tuple'), \
-            QT_TRANSLATE_NOOP('Variable Types', 'List'), \
-            QT_TRANSLATE_NOOP('Variable Types', 'Dictionary'), \
+            QT_TRANSLATE_NOOP('Variable Types', 'List/Array'), \
+            QT_TRANSLATE_NOOP('Variable Types', 'Dictionary/Hash/Map'), \
             QT_TRANSLATE_NOOP('Variable Types', 'Dictionary Proxy'), \
             QT_TRANSLATE_NOOP('Variable Types', 'File'), \
             QT_TRANSLATE_NOOP('Variable Types', 'X Range'), \
@@ -48,7 +48,7 @@ except ImportError:
     ConfigVarTypeDispStrings = [
             'Hidden Attributes', 'None', 'Type', 'Boolean', 'Integer', \
             'Long Integer', 'Float', 'Complex', 'String', 'Unicode String', \
-            'Tuple', 'List', 'Dictionary', 'Dictionary Proxy', 'File', \
+            'Tuple', 'List/Array', 'Dictionary/Hash/Map', 'Dictionary Proxy', 'File', \
             'X Range', 'Slice', 'Buffer', 'Class', 'Class Instance', \
             'Class Method', 'Class Property', 'Generator', 'Function', \
             'Builtin Function', 'Code', 'Module', 'Ellipsis', \
@@ -62,3 +62,22 @@ ConfigVarTypeStrings = ['__', 'NoneType', 'type',\
         'instance method', 'property', 'generator',\
         'function', 'builtin_function_or_method', 'code', 'module',\
         'ellipsis', 'traceback', 'frame', 'other']
+
+#######################################################
+## Configuration settings for the debug clients
+#######################################################
+
+import DebugClientCapabilities
+
+ClientDefaultCapabilities = {
+    "Python" : DebugClientCapabilities.HasAll,
+    "Ruby"   : DebugClientCapabilities.HasDebugger | \
+               DebugClientCapabilities.HasShell | \
+               DebugClientCapabilities.HasInterpreter | \
+               DebugClientCapabilities.HasCompleter,
+}
+
+ClientTypeAssociations = {
+    ".py" : "Python",
+    ".rb" : "Ruby",
+}
