@@ -56,6 +56,9 @@ class DebugBase(bdb.Bdb):
         # Use it like this:
         # if hasattr(sys, 'breakpoint): sys.breakpoint()
         sys.breakpoint = self.set_trace
+        
+        # initialize parent
+        bdb.Bdb.reset(self)
     
     def getCurrentFrame(self):
         """
@@ -189,6 +192,80 @@ class DebugBase(bdb.Bdb):
 
         return frame.f_code.co_filename
 
+    def set_watch(self, cond, temporary=0):
+        """
+        Public method to set a watchpoint.
+        
+        @param cond condition of the watchpoint (string)
+        @param temporary flag indicating a temporary watchpoint (boolean)
+        """
+        bp = bdb.Breakpoint("Watch", 0, temporary, cond)
+        if not self.breaks.has_key("Watch"):
+            self.breaks["Watch"] = 1
+        else:
+            self.breaks["Watch"] += 1
+    
+    def clear_watch(self, cond):
+        """
+        Public method to clear a watchpoint.
+        
+        @param cond condition of the watchpoint to be cleared (string)
+        """
+        possibles = bdb.Breakpoint.bplist["Watch", 0]
+        for i in range(0, len(possibles)):
+            b = possibles[i]
+            if b.cond == cond:
+                b.deleteMe()
+                self.breaks["Watch"] -= 1
+                if self.breaks["Watch"] == 0:
+                    del self.breaks["Watch"]
+                break
+    
+    def get_watch(self, cond):
+        """
+        Public method to clear a watchpoint.
+        
+        @param cond condition of the watchpoint to be cleared (string)
+        """
+        possibles = bdb.Breakpoint.bplist["Watch", 0]
+        for i in range(0, len(possibles)):
+            b = possibles[i]
+            if b.cond == cond:
+                return b
+    
+    def do_clearWatch(self, cond):
+        """
+        Private method called to clear a temporary watchpoint.
+        
+        @param cond condition of the watchpoint to be cleared (string)
+        """
+        self.clear_watch(cond)
+        self._dbgClient.write('%s%s\n' % (ResponseClearWatch, cond))
+
+    def effective(self, frame):
+        """
+        Private method to determine, if a watchpoint is effective.
+        
+        @param frame the current execution frame
+        @return tuple of watchpoint and a flag to indicate, that a temporary
+            watchpoint may be deleted (bdb.Breakpoin, boolean)
+        """
+        possibles = bdb.Breakpoint.bplist["Watch", 0]
+        for i in range(0, len(possibles)):
+            b = possibles[i]
+            if b.enabled == 0:
+                continue
+            if not b.cond:
+                # watchpoint without condition shouldn't occur, just ignore it
+                continue
+            try:
+                val = eval(b.cond, frame.f_globals, frame.f_locals)
+                if val:
+                    return (b, 1)
+            except:
+                continue
+        return (None, None)
+    
     def break_here(self, frame):
         """
         Reimplemented from bdb.py to fix the filename from the frame. 
@@ -199,20 +276,30 @@ class DebugBase(bdb.Bdb):
         @return flag indicating the break status (boolean)
         """
         filename = self.canonic(self.fix_frame_filename(frame))
-        if not self.breaks.has_key(filename):
-            return 0
-        lineno = frame.f_lineno
-        if not lineno in self.breaks[filename]:
-            return 0
-        # flag says ok to delete temp. bp
-        (bp, flag) = bdb.effective(filename, lineno, frame)
-        if bp:
-            self.currentbp = bp.number
-            if (flag and bp.temporary):
-                self.do_clear(filename, lineno)
-            return 1
-        else:
+        if not self.breaks.has_key(filename) and not self.breaks.has_key("Watch"):
             return 0
+        
+        if self.breaks.has_key(filename):
+            lineno = frame.f_lineno
+            if lineno in self.breaks[filename]:
+                # flag says ok to delete temp. bp
+                (bp, flag) = bdb.effective(filename, lineno, frame)
+                if bp:
+                    self.currentbp = bp.number
+                    if (flag and bp.temporary):
+                        self.do_clear(filename, lineno)
+                    return 1
+        
+        if self.breaks.has_key("Watch"):
+            # flag says ok to delete temp. bp
+            (bp, flag) = self.effective(frame)
+            if bp:
+                self.currentbp = bp.number
+                if (flag and bp.temporary):
+                    self.do_clearWatch(bp.cond)
+                return 1
+        
+        return 0
 
     def break_anywhere(self, frame):
         """
@@ -225,7 +312,8 @@ class DebugBase(bdb.Bdb):
         @return flag indicating the break status (boolean)
         """
         return self.breaks.has_key(
-            self.canonic(self.fix_frame_filename(frame)))
+            self.canonic(self.fix_frame_filename(frame))) or \
+            (self.breaks.has_key("Watch") and self.breaks["Watch"])
 
     def get_break(self, filename, lineno):
         """
@@ -261,7 +349,7 @@ class DebugBase(bdb.Bdb):
         """
         line = frame.f_lineno
 
-        # We never stop an line 0.
+        # We never stop on line 0.
         if line == 0:
             return
 
@@ -431,3 +519,5 @@ class DebugBase(bdb.Bdb):
 
         if self._dbgClient.shouldSkip(fn):
             return 1
+        
+        return 0
