@@ -21,6 +21,7 @@ import re
 
 
 from DebugProtocol import *
+import DebugClientCapabilities
 from DebugBase import printerr
 from AsyncFile import *
 from Config import ConfigVarTypeStrings
@@ -110,6 +111,8 @@ class DebugClientBase:
     <b>Note</b>: This class is meant to be subclassed by individual DebugClient classes.
     Do not instantiate it directly.
     """
+    clientCapabilities = DebugClientCapabilities.HasAll
+    
     def __init__(self):
         """
         Constructor
@@ -171,6 +174,7 @@ class DebugClientBase:
             self.compile_command = codeop.CommandCompiler()
         
         self.coding_re = re.compile(r"coding[:=]\s*([-\w_.]+)")
+        self.defaultCoding = 'utf-8'
 
     def get_coding(self, filename):
         """
@@ -259,9 +263,9 @@ class DebugClientBase:
         # make sure we close down our end of the socket
         # might be overkill as normally stdin, stdout and stderr
         # SHOULD be closed on exit, but it does not hurt to do it here
-        self.readstream.close()
-        self.writestream.close()
-        self.errorstream.close()
+        self.readstream.close(1)
+        self.writestream.close(1)
+        self.errorstream.close(1)
 
         # Ok, go away.
         sys.exit()
@@ -401,6 +405,14 @@ class DebugClientBase:
                 else:
                     os.chdir(wd)
 
+                self.running = sys.argv[0]
+                self.mainFrame = None
+                self.botframe = None
+                self.inRawMode = 0
+                
+                self.threads.clear()
+                self.attachThread(mainThread = 1)
+                
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.unhandled_exception
@@ -541,7 +553,7 @@ class DebugClientBase:
                         bp.enable()
                     else:
                         bp.disable()
-                        
+                    
                 return
                 
             if cmd == RequestBreakIgnore:
@@ -555,6 +567,41 @@ class DebugClientBase:
                     
                 return
                 
+            if cmd == RequestWatch:
+                cond, temporary, set = arg.split(',')
+                set = int(set)
+                temporary = int(temporary)
+
+                if set:
+                    self.mainThread.set_watch(cond, temporary)
+                else:
+                    self.mainThread.clear_watch(cond)
+
+                return
+                
+            if cmd == RequestWatchEnable:
+                cond, enable = arg.split(',')
+                enable = int(enable)
+                
+                bp = self.mainThread.get_watch(cond)
+                if bp is not None:
+                    if enable:
+                        bp.enable()
+                    else:
+                        bp.disable()
+                    
+                return
+                
+            if cmd == RequestWatchIgnore:
+                cond, count = arg.split(',')
+                count = int(count)
+                
+                bp = self.mainThread.get_watch(cond)
+                if bp is not None:
+                    bp.ignore = count
+                    
+                return
+                
             if cmd == RequestEval:
                 try:
                     value = eval(arg, self.currentThread.getCurrentFrame().f_globals,
@@ -615,9 +662,14 @@ class DebugClientBase:
             
             if cmd == RequestBanner:
                 self.write('%s%s\n' % (ResponseBanner, 
-                    str((sys.version, socket.gethostname(), self.variant))))
+                    str(("Python %s" % sys.version, socket.gethostname(), self.variant))))
                 return
             
+            if cmd == RequestCapabilities:
+                self.write('%s%d, "Python"\n' % (ResponseCapabilities, 
+                    self.clientCapabilities))
+                return
+                
             if cmd == RequestCompletion:
                 self.completionList(arg)
                 return
@@ -738,8 +790,8 @@ class DebugClientBase:
                             _locals["sys"].stdout = __stdout
                         else:
                             exec code in _globals, _locals
-                except SystemExit:
-                    self.sessionClose()
+                except SystemExit, exc:
+                    self.progTerminated(exc.code)
                 except:
                     # Report the exception and the traceback
                     try:
@@ -842,7 +894,7 @@ class DebugClientBase:
         if self.errorstream in wrdy:
             self.writeReady(self.errorstream.fileno())
         
-    def connectDebugger(self,port,remoteAddress=None):
+    def connectDebugger(self,port,remoteAddress=None,redirect=1):
         """
         Public method to establish a session with the debugger. 
         
@@ -852,6 +904,7 @@ class DebugClientBase:
         
         @param port the port number to connect to (int)
         @param remoteAddress the network address of the debug server host (string)
+        @param redirect flag indicating redirection of stdin, stdout and stderr (boolean)
         """
         sock = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
         if remoteAddress is None:
@@ -860,14 +913,14 @@ class DebugClientBase:
             sock.connect((remoteAddress,port))
         sock.setblocking(0)
 
-        sys.stdin = AsyncFile(sock,sys.stdin.mode,sys.stdin.name)
-        sys.stdout = AsyncFile(sock,sys.stdout.mode,sys.stdout.name)
-        sys.stderr = AsyncFile(sock,sys.stderr.mode,sys.stderr.name)
+        self.readstream = AsyncFile(sock,sys.stdin.mode,sys.stdin.name)
+        self.writestream = AsyncFile(sock,sys.stdout.mode,sys.stdout.name)
+        self.errorstream = AsyncFile(sock,sys.stderr.mode,sys.stderr.name)
         
-        # save the stream in case a program redirects them
-        self.readstream = sys.stdin
-        self.writestream = sys.stdout
-        self.errorstream = sys.stderr
+        if redirect:
+            sys.stdin = self.readstream
+            sys.stdout = self.writestream
+            sys.stderr = self.errorstream
         
         # attach to the main thread here
         self.attachThread(mainThread=1)
@@ -958,9 +1011,16 @@ class DebugClientBase:
             except:
                 status = 1
 
-        self.set_quit()
-        self.running = None
+        if self.running:
+            self.set_quit()
+            self.running = None
         self.write('%s%d\n' % (ResponseExit,status))
+        
+        # reset coding
+        try:
+            sys.setappdefaultencoding(self.defaultCoding)
+        except AttributeError:
+            pass
 
     def dumpVariables(self, frmnr, scope, filter):
         """
@@ -1029,6 +1089,13 @@ class DebugClientBase:
             # search the correct dictionary
             i = 0
             rvar = var[:]
+            dictkeys = None
+            obj = None
+            isDict = 0
+            formatSequences = 0
+            access = ""
+            oaccess = ""
+            odict = dict
             
             while i < len(var):
                 if len(dict):
@@ -1036,51 +1103,92 @@ class DebugClientBase:
                 ndict = {}
                 # this has to be in line with VariablesViewer.indicators
                 if var[i][-2:] in ["[]", "()", "{}"]:
-                    try:
-                        exec 'mdict = dict["%s"][%s].__dict__' % (var[i][:-2], var[i+1])
-                        ndict.update(mdict)
-                        if mdict and not "sipThis" in mdict.keys():
-                            del rvar[0:2]
-                    except:
-                        pass
-                    try:
-                        cdict = {}
-                        exec 'slv = dict["%s"][%s].__slots__' % (var[i][:-2], var[i+1])
-                        for v in slv:
-                            exec 'cdict[v] = dict["%s"][%s].%s' % (var[i][:-2], var[i+1], v)
-                        ndict.update(cdict)
-                    except:
-                        pass
-                    i += 1
+                    if i+1 == len(var):
+                        if var[i][:-2] == '...':
+                            dictkeys = [var[i-1]]
+                        else:
+                            dictkeys = [var[i][:-2]]
+                        formatSequences = 1
+                        if not access and not oaccess:
+                            if var[i][:-2] == '...':
+                                access = '["%s"]' % var[i-1]
+                                dict = odict
+                            else:
+                                access = '["%s"]' % var[i][:-2]
+                        else:
+                            if var[i][:-2] == '...':
+                                if oaccess:
+                                    access = oaccess
+                                else:
+                                    access = '%s[%s]' % (access, var[i-1])
+                                dict = odict
+                            else:
+                                access = '%s[%s]' % (access, var[i][:-2])
+                        if var[i][-2:] == "{}":
+                            isDict = 1
+                        break
+                    else:
+                        if not access:
+                            if var[i][:-2] == '...':
+                                access = '["%s"]' % var[i-1]
+                                dict = odict
+                            else:
+                                access = '["%s"]' % var[i][:-2]
+                        else:
+                            if var[i][:-2] == '...':
+                                access = '%s[%s]' % (access, var[i-1])
+                                dict = odict
+                            else:
+                                access = '%s[%s]' % (access, var[i][:-2])
                 else:
-                    try:
-                        ndict.update(dict[var[i]].__dict__)
-                        del rvar[0]
-                    except:
-                        pass
-                    try:
-                        cdict = {}
-                        slv = dict[var[i]].__slots__
-                        for v in slv:
-                            exec 'cdict[v] = dict[var[i]].%s' % v
-                        ndict.update(cdict)
-                    except:
-                        pass
+                    if access:
+                        if oaccess:
+                            access = '%s[%s]' % (oaccess, var[i])
+                        else:
+                            access = '%s[%s]' % (access, var[i])
+                        if var[i-1][:-2] == '...':
+                            oaccess = access
+                        else:
+                            oaccess = ''
+                        try:
+                            exec 'mdict = dict%s.__dict__' % access
+                            ndict.update(mdict)
+                            exec 'obj = dict%s' % access
+                            if mdict and not "sipThis" in mdict.keys():
+                                del rvar[0:2]
+                                access = ""
+                        except:
+                            pass
+                        try:
+                            cdict = {}
+                            exec 'slv = dict%s.__slots__' % access
+                            for v in slv:
+                                exec 'cdict[v] = dict%s.%s' % (access, v)
+                            exec 'obj = dict%s' % access
+                            ndict.update(cdict)
+                            access = ""
+                        except:
+                            pass
+                    else:
+                        try:
+                            ndict.update(dict[var[i]].__dict__)
+                            del rvar[0]
+                            obj = dict[var[i]]
+                        except:
+                            pass
+                        try:
+                            cdict = {}
+                            slv = dict[var[i]].__slots__
+                            for v in slv:
+                                exec 'cdict[v] = dict[var[i]].%s' % v
+                            ndict.update(cdict)
+                            obj = dict[var[i]]
+                        except:
+                            pass
+                    odict = dict
+                    dict = ndict
                 i += 1
-                dict = ndict
-                
-            if len(dict) == 0 and len(rvar) > 0:
-                access = '["%s"]' % rvar[0][:-2]
-                i = 1
-                while i < len(rvar) - 1:
-                    access = '%s[%s]' % (access, rvar[i][:-2])
-                    i += 1
-                if i < len(rvar):
-                    access = '%s[%s]' % (access, rvar[-1])
-                exec 'dict = udict%s.__dict__' % access
-            else:
-                access = ""
-                
+            
             if ("sipThis" in dict.keys() and len(dict) == 1) or \
                (len(dict) == 0 and len(udict) > 0):
                 if access:
@@ -1093,9 +1201,29 @@ class DebugClientBase:
                 vlist = self.formatQtVariable(qvar)
             else:
                 # format the dictionary found
-                vlist = self.formatVariablesList(dict.keys(), dict, scope, filter)
+                if dictkeys is None:
+                    dictkeys = dict.keys()
+                else:
+                    # treatment for sequences and dictionaries
+                    if access:
+                        exec "dict = dict%s" % access
+                    else:
+                        dict = dict[dictkeys[0]]
+                    if isDict:
+                        dictkeys = dict.keys()
+                    else:
+                        dictkeys = range(len(dict))
+                vlist = self.formatVariablesList(dictkeys, dict, scope, filter, formatSequences)
             varlist.extend(vlist)
-            
+        
+            if obj is not None and not formatSequences:
+                if unicode(obj).startswith('{'):
+                    varlist.append(('...', 'dict', "%d" % len(obj.keys())))
+                elif unicode(obj).startswith('['):
+                    varlist.append(('...', 'list', "%d" % len(obj)))
+                elif unicode(obj).startswith('('):
+                    varlist.append(('...', 'tuple', "%d" % len(obj)))
+        
         self.write('%s%s\n' % (ResponseVariable, str(varlist)))
         
     def formatQtVariable(self, value):
@@ -1200,7 +1328,7 @@ class DebugClientBase:
             
         return varlist
         
-    def formatVariablesList(self, keylist, dict, scope, filter = []):
+    def formatVariablesList(self, keylist, dict, scope, filter = [], formatSequences = 0):
         """
         Private method to produce a formated variables list.
         
@@ -1217,6 +1345,9 @@ class DebugClientBase:
         @param filter the indices of variable types to be filtered. Variables are
             only added to the list, if their type is not contained in the filter 
             list.
+        @param formatSequences flag indicating, that sequence or dictionary variables
+            should be formatted. If it is 0 (or false), just the number of items contained
+            in these variables is returned. (boolean)
         @return A tuple consisting of a list of formatted variables. Each variable
             entry is a tuple of three elements, the variable name, its type and 
             value.
@@ -1231,7 +1362,7 @@ class DebugClientBase:
             # filter based on the filter pattern
             matched = 0
             for pat in patternFilterObjects:
-                if pat.match(key):
+                if pat.match(str(key)):
                     matched = 1
                     break
             if matched:
@@ -1253,21 +1384,38 @@ class DebugClientBase:
                     # handle new class type of python 2.2+
                     if ConfigVarTypeStrings.index('instance') in filter:
                         continue
-                    valtype = valtypestr[7:-1]
+                    valtype = valtypestr
                 else:
                     valtype = valtypestr[6:-1]
                     try:
                         if ConfigVarTypeStrings.index(valtype) in filter:
                             continue
                     except ValueError:
-                        if ConfigVarTypeStrings.index('other') in filter:
+                        if valtype == "classobj":
+                            if ConfigVarTypeStrings.index('instance') in filter:
+                                continue
+                        elif ConfigVarTypeStrings.index('other') in filter:
                             continue
                     
                 try:
-                    rvalue = repr(value)
+                    if valtype not in ['list', 'tuple', 'dict']:
+                        rvalue = repr(value)
+                        if valtype.startswith('class') and \
+                           rvalue[0] in ['{', '(', '[']:
+                            rvalue = ""
+                    else:
+                        if valtype == 'dict':
+                            rvalue = "%d" % len(value.keys())
+                        else:
+                            rvalue = "%d" % len(value)
                 except:
                     rvalue = ''
-                    
+                
+            if formatSequences:
+                if str(key) == key:
+                    key = "'%s'" % key
+                else:
+                    key = str(key)
             varlist.append((key, valtype, rvalue))
         
         return varlist
@@ -1346,7 +1494,7 @@ class DebugClientBase:
             try:
                 self.running = os.path.abspath(sys.argv[0])
             except:
-                pass
+                self.running = None
         if self.running:
             coding = self.get_coding(self.running)
             try:
@@ -1377,7 +1525,7 @@ class DebugClientBase:
             debugging = 0
         
     def startProgInDebugger(self, progargs, wd = '', host=None, 
-            port=None, exceptions=1, tracePython=0):
+            port=None, exceptions=1, tracePython=0, redirect=1):
         """
         Method used to start the remote debugger.
         
@@ -1387,7 +1535,8 @@ class DebugClientBase:
         @param host hostname of the debug server (string)
         @param port portnumber of the debug server (int)
         @param exceptions flag to enable exception reporting of the IDE (boolean)
-        @param tracePython flag to enable tracing into the Python library
+        @param tracePython flag to enable tracing into the Python library (boolean)
+        @param redirect flag indicating redirection of stdin, stdout and stderr (boolean)
         """
         global debugging
         debugging = 1
@@ -1397,7 +1546,7 @@ class DebugClientBase:
         if port is None:
             port = os.getenv('ERICPORT', 42424)
             
-        self.connectDebugger(port, socket.gethostbyname(host))
+        self.connectDebugger(port, socket.gethostbyname(host), redirect)
         
         self.fncache = {}
         self.dircache = []
@@ -1456,12 +1605,19 @@ class DebugClientBase:
         """
         Public method implementing the main method.
         """
+        try:
+            sys.setappdefaultencoding(self.defaultCoding)
+        except AttributeError:
+            pass
+        
         if '--' in sys.argv:
             args = sys.argv[1:]
             host = None
             port = None
             wd = ''
             tracePython = 0
+            exceptions = 1
+            redirect = 1
             while args[0]:
                 if args[0] == '-h':
                     host = args[1]
@@ -1478,24 +1634,38 @@ class DebugClientBase:
                 elif args[0] == '-t':
                     tracePython = 1
                     del args[0]
+                elif args[0] == '-e':
+                    exceptions = 0
+                    del args[0]
+                elif args[0] == '-n':
+                    redirect = 0
+                    del args[0]
                 elif args[0] == '--':
                     del args[0]
                     break
+                else:   # unknown option
+                    del args[0]
             if not args:
                 print "No program given. Aborting!"
             else:
-                self.startProgInDebugger(args, wd, host, port)
+                self.startProgInDebugger(args, wd, host, port, 
+                                         exceptions=exceptions, tracePython=tracePython,
+                                         redirect=redirect)
         else:
             try:
                 port = int(sys.argv[1])
             except:
                 port = -1
             try:
-                remoteAddress = sys.argv[2]
+                redirect = int(sys.argv[2])
+            except:
+                redirect = 1
+            try:
+                remoteAddress = sys.argv[3]
             except:
                 remoteAddress = None
             sys.argv = ['']
             sys.path.insert(0, '')
             if port >= 0:
-                self.connectDebugger(port, remoteAddress)
+                self.connectDebugger(port, remoteAddress, redirect)
                 self.interact()
