@@ -18,9 +18,12 @@ from qt import SIGNAL, PYSIGNAL, qApp, QTextCodec
 from qtnetwork import QServerSocket, QSocket
 
 from DebugProtocol import *
+from Config import ClientDefaultCapabilities, ClientTypeAssociations
 import Preferences
 import Utilities
 
+from eric3config import getConfig
+
 
 class DebugServer(QServerSocket):
     """
@@ -45,8 +48,12 @@ class DebugServer(QServerSocket):
     @signal clientExit(exitcode) emitted after the client has exited
     @signal clientClearBreak(filename, lineno) emitted after the debug client
             has decided to clear a temporary breakpoint
+    @signal clientClearWatch(condition) emitted after the debug client
+            has decided to clear a temporary watchpoint
     @signal clientRawInput(prompt, echo) emitted after a raw input request was received
     @signal clientBanner(banner) emitted after the client banner was received
+    @signal clientCapabilities(capabilities, cltype) emitted after the clients capabilities were
+            received
     @signal clientCompletionList(completionList, text) emitted after the client
             the commandline completion list and the reworked searchstring was
             received from the client
@@ -74,43 +81,35 @@ class DebugServer(QServerSocket):
             socket = Preferences.getDebugger("PassiveDbgPort") # default: 42424
             QServerSocket.__init__(self,socket)
             self.passive = 1
+            self.passiveClientExited = 0
         else:
             QServerSocket.__init__(self,0)
             self.passive = 0
 
         self.qsock = None
-        self.progLoaded = 0
         self.debugging = 1
         self.queue = []
         self.clientPID = 0
+        self.clientType, ok = Preferences.Prefs.settings.readEntry(\
+            '/eric3/DebugClient/Type', 'Python')
+        self.lastClientType = ''
+        # set default values for capabilities of clients
+        self.clientCapabilities = {
+            "Python" : ClientDefaultCapabilities["Python"],
+            "Ruby"   : ClientDefaultCapabilities["Ruby"],
+        }
+        
+        self.connect(self, PYSIGNAL("clientCapabilities"), self.handleClientCapabilities)
 
         self.codec = QTextCodec.codecForName(sys.getdefaultencoding())
 
-        if not self.passive:
-            self.startRemote()
-
     def startRemote(self):
         """
-        Private method to start a remote interpreter.
+        Public method to start a remote interpreter.
         """
-        if Preferences.getDebugger("CustomPythonInterpreter"):
-            interpreter = str(Preferences.getDebugger("PythonInterpreter"))
-            if interpreter == "":
-                interpreter = sys.executable
-        else:
-            interpreter = sys.executable
-            
-        debugClientType = Preferences.getDebugger("DebugClientType")
-        if debugClientType == 1:
-            debugClient = os.path.join(sys.path[0],'Debugger','DebugClient.py')
-        elif debugClientType == 2:
-            debugClient = os.path.join(sys.path[0],'Debugger','DebugClientThreads.py')
-        elif debugClientType == 3:
-            debugClient = os.path.join(sys.path[0],'Debugger','DebugClientNoQt.py')
-        else:
-            debugClient = str(Preferences.getDebugger("DebugClient"))
-            if debugClient == "":
-                debugClient = os.path.join(sys.path[0],'Debugger','DebugClient.py')
+        # only start the client, if we are not in passive mode
+        if self.passive:
+            return
             
         if self.clientPID:
             try:
@@ -126,6 +125,36 @@ class DebugServer(QServerSocket):
             except:
                 pass
             
+        if self.clientType == "Ruby":
+            self.startRemoteRuby()
+        else:
+            self.startRemotePython()
+        
+    def startRemotePython(self):
+        """
+        Private method to start a remote Python interpreter.
+        """
+        if Preferences.getDebugger("CustomPythonInterpreter"):
+            interpreter = str(Preferences.getDebugger("PythonInterpreter"))
+            if interpreter == "":
+                interpreter = sys.executable
+        else:
+            interpreter = sys.executable
+            
+        debugClientType = Preferences.getDebugger("DebugClientType")
+        if debugClientType == 1:
+            debugClient = os.path.join(getConfig('ericDir'),'Debugger','DebugClient.py')
+        elif debugClientType == 2:
+            debugClient = os.path.join(getConfig('ericDir'),'Debugger','DebugClientThreads.py')
+        elif debugClientType == 3:
+            debugClient = os.path.join(getConfig('ericDir'),'Debugger','DebugClientNoQt.py')
+        else:
+            debugClient = str(Preferences.getDebugger("DebugClient"))
+            if debugClient == "":
+                debugClient = os.path.join(sys.path[0],'Debugger','DebugClient.py')
+            
+        redirect = str(Preferences.getDebugger("PythonRedirect"))
+        
         if Preferences.getDebugger("RemoteDbgEnabled"):
             ipaddr = socket.gethostbyname(socket.gethostname())
             rexec = str(Preferences.getDebugger("RemoteExecution"))
@@ -135,18 +164,124 @@ class DebugServer(QServerSocket):
             if rexec:
                 args = Utilities.parseOptionString(rexec) + \
                        [rhost, interpreter, os.path.abspath(debugClient),
-                        `self.port()`, ipaddr]
+                        `self.port()`, redirect, ipaddr]
                 args[0] = Utilities.getExecutablePath(args[0])
                 self.clientPID = os.spawnv(os.P_NOWAIT, args[0], args)
                 return
         
-        if sys.platform == "win32":
-            self.clientPID = os.spawnv(os.P_NOWAIT,interpreter,
-                [interpreter, '"%s"' % debugClient, `self.port()`])
+        # setup the environment for the debugger
+        if Preferences.getDebugger("DebugEnvironmentReplace"):
+            clientEnv = {}
         else:
-            self.clientPID = os.spawnv(os.P_NOWAIT,interpreter,
-                [interpreter, debugClient, `self.port()`])
+            clientEnv = os.environ.copy()
+        envlist = Utilities.parseEnvironmentString(\
+            Preferences.getDebugger("DebugEnvironment"))
+        for el in envlist:
+            try:
+                key, value = el.split('=', 1)
+                if value.startswith('"') or value.startswith("'"):
+                    value = value[1:-1]
+                clientEnv[unicode(key)] = unicode(value)
+            except UnpackError:
+                pass
+        
+        if Preferences.getDebugger("ConsoleDbgEnabled"):
+            ccmd = unicode(Preferences.getDebugger("ConsoleDbgCommand"))
+            if ccmd:
+                args = Utilities.parseOptionString(ccmd) + \
+                       [interpreter, os.path.abspath(debugClient),
+                        `self.port()`, '0']
+                args[0] = Utilities.getExecutablePath(args[0])
+                self.clientPID = os.spawnve(os.P_NOWAIT, args[0], args, clientEnv)
+                return
+        
+        if sys.platform == "win32":
+            self.clientPID = os.spawnve(os.P_NOWAIT,interpreter,
+                ['"%s"' % interpreter, '"%s"' % debugClient, `self.port()`, redirect], clientEnv)
+        else:
+            self.clientPID = os.spawnve(os.P_NOWAIT,interpreter,
+                [interpreter, debugClient, `self.port()`, redirect], clientEnv)
 
+    def startRemoteRuby(self):
+        """
+        Private method to start a remote Ruby interpreter.
+        """
+        interpreter = str(Preferences.getDebugger("RubyInterpreter"))
+        if interpreter == "":
+            interpreter = "/usr/bin/ruby"
+        debugClient = os.path.join(getConfig('ericDir'), 
+            "DebugClients", "Ruby", "DebugClient.rb")
+        
+        redirect = str(Preferences.getDebugger("RubyRedirect"))
+        
+        if Preferences.getDebugger("RemoteDbgEnabled"):
+            ipaddr = socket.gethostbyname(socket.gethostname())
+            rexec = str(Preferences.getDebugger("RemoteExecution"))
+            rhost = str(Preferences.getDebugger("RemoteHost"))
+            if rhost == "":
+                rhost = "localhost"
+            if rexec:
+                args = Utilities.parseOptionString(rexec) + \
+                       [rhost, interpreter, os.path.abspath(debugClient),
+                        `self.port()`, redirect, ipaddr]
+                args[0] = Utilities.getExecutablePath(args[0])
+                self.clientPID = os.spawnv(os.P_NOWAIT, args[0], args)
+                return
+        
+        # setup the environment for the debugger
+        if Preferences.getDebugger("DebugEnvironmentReplace"):
+            clientEnv = {}
+        else:
+            clientEnv = os.environ.copy()
+        envlist = Utilities.parseEnvironmentString(\
+            Preferences.getDebugger("DebugEnvironment"))
+        for el in envlist:
+            try:
+                key, value = el.split('=', 1)
+                if value.startswith('"') or value.startswith("'"):
+                    value = value[1:-1]
+                clientEnv[unicode(key)] = unicode(value)
+            except UnpackError:
+                pass
+        
+        if Preferences.getDebugger("ConsoleDbgEnabled"):
+            ccmd = unicode(Preferences.getDebugger("ConsoleDbgCommand"))
+            if ccmd:
+                args = Utilities.parseOptionString(ccmd) + \
+                       [interpreter, os.path.abspath(debugClient),
+                        `self.port()`, '0']
+                args[0] = Utilities.getExecutablePath(args[0])
+                self.clientPID = os.spawnve(os.P_NOWAIT, args[0], args, clientEnv)
+                return
+        
+        if sys.platform == "win32":
+            self.clientPID = os.spawnve(os.P_NOWAIT,interpreter,
+                ['"%s"' % interpreter, '"%s"' % debugClient, `self.port()`, redirect], clientEnv)
+        else:
+            self.clientPID = os.spawnve(os.P_NOWAIT,interpreter,
+                [interpreter, debugClient, `self.port()`, redirect], clientEnv)
+
+    def handleClientCapabilities(self, cap, clType):
+        """
+        Private slot to handle the reporting of the clients capabilities.
+        
+        @param cap client capabilities (integer)
+        @param clType type of the debug client (string)
+        """
+        self.clientCapabilities[clType] = cap
+        
+    def getClientCapabilities(self, type):
+        """
+        Public method to retrieve the debug clients capabilities.
+        
+        @param type debug client type (string)
+        @return debug client capabilities (integer)
+        """
+        try:
+            return self.clientCapabilities[type]
+        except KeyError:
+            return 0    # no capabilities
+        
     def newConnection(self,sockfd):
         """
         Reimplemented to handle a new connection.
@@ -166,14 +301,19 @@ class DebugServer(QServerSocket):
 
         self.qsock = sock
 
+        # Get the remote clients capabilities
+        self.remoteCapabilities()
+        
+        # Perform actions neccessary, if client type has changed
+        if self.lastClientType != self.clientType:
+            self.lastClientType = self.clientType
+            self.remoteBanner()
+        
         # Send commands that were waiting for the connection.
         for cmd in self.queue:
             self.qsock.writeBlock(str(cmd))
 
         self.queue = []
-        
-        if self.passive:
-            self.progLoaded = 1
 
     def shutdownServer(self):
         """
@@ -197,7 +337,6 @@ class DebugServer(QServerSocket):
         
         # reinitialize
         self.qsock = None
-        self.progLoaded = 1  # fake server into starting new client later...
         self.queue = []
 
     def remoteEnvironment(self, env):
@@ -230,15 +369,17 @@ class DebugServer(QServerSocket):
             as well
         """
         # Restart the client if there is already a program loaded.
-        if self.progLoaded:
-            self.startClient(0)
+        try:
+            self.setClientType(ClientTypeAssociations[os.path.splitext(fn)[1]])
+        except KeyError:
+            self.setClientType('Python')    # assume it is a Python file
+        self.startClient(0)
         
         self.remoteEnvironment(env)
         
         self.sendCommand('%s%s|%s|%s|%d\n' % \
             (RequestLoad, unicode(wd),
              unicode(os.path.abspath(unicode(fn))),str(argv),tracePython))
-        self.progLoaded = 1
         self.debugging = 1
         self.restoreBreakpoints()
 
@@ -252,15 +393,17 @@ class DebugServer(QServerSocket):
         @param env environment settings (string)
         """
         # Restart the client if there is already a program loaded.
-        if self.progLoaded:
-            self.startClient(0)
+        try:
+            self.setClientType(ClientTypeAssociations[os.path.splitext(fn)[1]])
+        except KeyError:
+            self.setClientType('Python')    # assume it is a Python file
+        self.startClient(0)
         
         self.remoteEnvironment(env)
         
         self.sendCommand('%s%s|%s|%s\n' % \
             (RequestRun, unicode(wd),
              unicode(os.path.abspath(unicode(fn))),str(argv)))
-        self.progLoaded = 1
         self.debugging = 0
 
     def remoteCoverage(self,fn,argv,wd,env,erase,exclude=""):
@@ -275,15 +418,17 @@ class DebugServer(QServerSocket):
         @param exclude searchpattern marking line to exclude from coverage
         """
         # Restart the client if there is already a program loaded.
-        if self.progLoaded:
-            self.startClient(0)
+        try:
+            self.setClientType(ClientTypeAssociations[os.path.splitext(fn)[1]])
+        except KeyError:
+            self.setClientType('Python')    # assume it is a Python file
+        self.startClient(0)
         
         self.remoteEnvironment(env)
         
         self.sendCommand('%s%s@@%s@@%s@@%d@@%s\n' % \
             (RequestCoverage, unicode(wd),
              unicode(os.path.abspath(unicode(fn))),str(argv),erase,unicode(exclude)))
-        self.progLoaded = 1
         self.debugging = 0
 
     def remoteProfile(self,fn,argv,wd,env,erase):
@@ -297,15 +442,17 @@ class DebugServer(QServerSocket):
         @param erase flag indicating that timing info should be cleared first (boolean)
         """
         # Restart the client if there is already a program loaded.
-        if self.progLoaded:
-            self.startClient(0)
+        try:
+            self.setClientType(ClientTypeAssociations[os.path.splitext(fn)[1]])
+        except KeyError:
+            self.setClientType('Python')    # assume it is a Python file
+        self.startClient(0)
         
         self.remoteEnvironment(env)
         
         self.sendCommand('%s%s|%s|%s|%d\n' % \
             (RequestProfile, unicode(wd),
              unicode(os.path.abspath(unicode(fn))),str(argv),erase))
-        self.progLoaded = 1
         self.debugging = 0
 
     def remoteCyclops(self,fn,argv,wd,env,modfunc,reports):
@@ -322,15 +469,17 @@ class DebugServer(QServerSocket):
         @param reports bit mask specifying the reports wanted (integer)
         """
         # Restart the client if there is already a program loaded.
-        if self.progLoaded:
-            self.startClient(0)
+        try:
+            self.setClientType(ClientTypeAssociations[os.path.splitext(fn)[1]])
+        except KeyError:
+            self.setClientType('Python')    # assume it is a Python file
+        self.startClient(0)
         
         self.remoteEnvironment(env)
         
         self.sendCommand('%s%s|%s|%s|%s|%d\n' % \
             (RequestCyclops, unicode(wd),
              unicode(os.path.abspath(unicode(fn))),str(argv),str(modfunc),reports))
-        self.progLoaded = 1
         self.debugging = 0
 
     def remoteStatement(self,stmt):
@@ -405,6 +554,34 @@ class DebugServer(QServerSocket):
         """
         self.sendCommand('%s%s,%d,%d\n' % (RequestBreakIgnore,fn,line,count))
         
+    def remoteWatchpoint(self,cond,set,temp=0):
+        """
+        Public method to set or clear a watchpoint.
+        
+        @param cond condition of the watchpoint (string)
+        @param set flag indicating setting or resetting a watchpoint (boolean)
+        @param temp flag indicating a temporary watchpoint (boolean)
+        """
+        self.sendCommand('%s%s,%d,%d\n' % (RequestWatch,cond,temp,set))
+    
+    def remoteWatchpointEnable(self,cond,enable):
+        """
+        Public method to enable or disable a watchpoint.
+        
+        @param cond condition of the watchpoint (string)
+        @param enable flag indicating enabling or disabling a watchpoint (boolean)
+        """
+        self.sendCommand('%s%s,%d\n' % (RequestWatchEnable,cond,enable))
+    
+    def remoteWatchpointIgnore(self,cond,count):
+        """
+        Public method to ignore a watchpoint the next couple of occurences.
+        
+        @param cond condition of the watchpoint (string)
+        @param count number of occurences to ignore (int)
+        """
+        self.sendCommand('%s%s,%d\n' % (RequestWatchIgnore,cond,count))
+    
     def remoteRawInput(self,s):
         """
         Public method to send the raw input to the debugged program.
@@ -466,6 +643,12 @@ class DebugServer(QServerSocket):
         """
         self.sendCommand(RequestBanner + '\n')
         
+    def remoteCapabilities(self):
+        """
+        Public slot to get the debug clients capabilities.
+        """
+        self.sendCommand(RequestCapabilities + '\n')
+        
     def remoteCompletion(self, text):
         """
         Public slot to get the a list of possible commandline completions
@@ -487,13 +670,15 @@ class DebugServer(QServerSocket):
         @param coverase flag indicating erasure of coverage data is requested
         """
         # Restart the client if there is already a program loaded.
-        if self.progLoaded:
-            self.startClient(0)
+        try:
+            self.setClientType(ClientTypeAssociations[os.path.splitext(fn)[1]])
+        except KeyError:
+            self.setClientType('Python')    # assume it is a Python file
+        self.startClient(0)
         
         self.sendCommand('%s%s|%s|%d|%s|%d\n' % \
             (RequestUTPrepare,
              unicode(os.path.abspath(unicode(fn))), str(tn), cov, str(covname), coverase))
-        self.progLoaded = 1
         self.debugging = 0
         
     def remoteUTRun(self):
@@ -524,7 +709,7 @@ class DebugServer(QServerSocket):
                 if not line:
                     continue
 
-##            print line          ##debug
+##            print "Server: ", line          ##debug
             
             eoc = line.find('<') + 1
             
@@ -535,8 +720,10 @@ class DebugServer(QServerSocket):
             boc = line.find('>')
             if boc > 0 and eoc > boc:
                 self.emit(PYSIGNAL('clientOutput'),(line[:boc],))
+                line = line[boc:]
+                eoc = line.find('<') + 1
+                boc = line.find('>')
                 
-                    
             if boc >= 0 and eoc > boc:
                 # Emit the signal corresponding to the response.
                 resp = line[boc:eoc]
@@ -586,6 +773,11 @@ class DebugServer(QServerSocket):
                     self.emit(PYSIGNAL('clientClearBreak'),(fn, lineno))
                     continue
                     
+                if resp == ResponseClearWatch:
+                    cond = line[eoc:-1]
+                    self.emit(PYSIGNAL('clientClearWatch'),(cond,))
+                    continue
+                    
                 if resp == ResponseRaw:
                     prompt, echo = eval(line[eoc:-1])
                     self.emit(PYSIGNAL('clientRawInput'),(prompt, echo))
@@ -596,6 +788,11 @@ class DebugServer(QServerSocket):
                     self.emit(PYSIGNAL('clientBanner'),bi)
                     continue
                     
+                if resp == ResponseCapabilities:
+                    cap = eval(line[eoc:-1])
+                    self.emit(PYSIGNAL('clientCapabilities'),cap)
+                    continue
+                    
                 if resp == ResponseCompletion:
                     clstring, text = line[eoc:-1].split('||')
                     cl = eval(clstring)
@@ -663,19 +860,31 @@ class DebugServer(QServerSocket):
         self.shutdownServer()
         print unicode(self.trUtf8("Passive debug connection closed"))
         
-    def startClient(self,unplanned=1):
+    def setClientType(self, clType):
         """
-        Private method to start a debug client.
+        Private method to set the client type.
+        
+        @param clType type of client to be started (string, Python or Ruby)
+        """
+        if clType is not None and clType in ["Python", "Ruby"]:
+            self.clientType = clType
+            ok = Preferences.Prefs.settings.writeEntry('/eric3/DebugClient/Type', 
+                self.clientType)
+        
+    def startClient(self,unplanned=1,clType=None):
+        """
+        Public method to start a debug client.
         
         @param unplanned flag indicating that the client has died
+        @param clType type of client to be started (string, Python or Ruby)
         """
         if not self.passive or not self.passiveClientExited: 
             if self.qsock is not None:
                 self.shutdownServer()
                 self.emit(PYSIGNAL('clientGone'),(unplanned & self.debugging,))
             
-        if not self.passive:
-            self.startRemote()
+        self.setClientType(clType)
+        self.startRemote()
 
     def sendCommand(self,cmd):
         """
@@ -690,7 +899,7 @@ class DebugServer(QServerSocket):
 
     def restoreBreakpoints(self):
         """
-        Private method to restore the break points after a restart.
+        Private method to restore the break points and watch points after a restart.
         """
         vm = qApp.mainWidget().getViewManager()
         for fn in vm.getOpenFilenames():
@@ -702,3 +911,19 @@ class DebugServer(QServerSocket):
                     self.remoteBreakpointEnable(fn, line, 0)
                 if ignorecount:
                     self.remoteBreakpointIgnore(fn, line, ignorecount)
+        
+        breaks = self.watchpointViewer.getWatchpoints()
+        for cond, temp, enabled, ignorecount in breaks:
+            self.remoteWatchpoint(cond, 1, temp)
+            if not enabled:
+                self.remoteWatchpointEnable(cond, 0)
+            if ignorecount:
+                self.remoteWatchpointIgnore(cond, ignorecount)
+    
+    def setWatchpointViewer(self, watchpointViewer):
+        """
+        Public method to set the watchpoint viewer.
+        
+        @param reference to the watchpoint viewer widget
+        """
+        self.watchpointViewer = watchpointViewer
