@@ -16,8 +16,8 @@ from KdeQt import KQMessageBox, KQInputDialog
 from UI.Info import *
 from VariablesFilterDialog import *
 from ExceptionsFilterDialog import *
-from BreakpointDialog import *
 from StartDialog import *
+from DebugClientCapabilities import *
 import Preferences
 import Utilities
 import UI.PixmapCache
@@ -30,8 +30,10 @@ class DebugUI(QObject):
     
     @signal clientStack(stack) emitted at breaking after a reported exception
     @signal compileForms() emitted if changed project forms should be compiled
+    @signal debuggingStarted(filename) emitted when a debugging session was started
+    @signal resetUI()   emitted to reset the UI
     """
-    def __init__(self,ui,vm,dbs,sbv):
+    def __init__(self,ui,vm,dbs,sbv,pro):
         """
         Constructor
         
@@ -39,6 +41,7 @@ class DebugUI(QObject):
         @param vm reference to the viewmanager
         @param dbs reference to the debug server
         @param sbv reference to the shell/browser/variables/exception widget
+        @param pro reference to the project object
         """
         QObject.__init__(self, ui)
         
@@ -46,6 +49,7 @@ class DebugUI(QObject):
         self.viewmanager = vm
         self.dbs = dbs
         self.sbv = sbv
+        self.project = pro
         
         # Clear some variables
         self.projectOpen = 0
@@ -456,49 +460,9 @@ class DebugUI(QObject):
         act.connectIt(SIGNAL('activated()'), self.handleClearBreakpoints)
         self.actions.append(act)
 
-        self.dbgAllBpActGrp = E3ActionGroup(self)
-        
-        self.dbgFileBpAct = E3Action(self.trUtf8('File Breakpoints'),
-                self.trUtf8('&File Breakpoints...'), 0, 0, self.dbgAllBpActGrp,
-                'dbg_file_breakpoints')
-        self.dbgFileBpAct.setStatusTip(self.trUtf8('Show breakpoints of the current file'))
-        self.dbgFileBpAct.setWhatsThis(self.trUtf8(
-            """<b>File Breakpoints</b>"""
-            """<p>Show all breakpoints of the current file. The dialog allows to"""
-            """ delete or change these breakpoints.</p>"""
-        ))
-        self.dbgFileBpAct.connectIt(SIGNAL('activated()'), self.handleAllFileBp)
-        self.actions.append(self.dbgFileBpAct)
-        
-        act = E3Action(self.trUtf8('All Breakpoints'),
-                self.trUtf8('&All Breakpoints...'), 0, 0, self.dbgAllBpActGrp,
-                'dbg_all_breakpoints')
-        act.setStatusTip(self.trUtf8('Show all breakpoints'))
-        act.setWhatsThis(self.trUtf8(
-            """<b>All Breakpoints</b>"""
-            """<p>Show all breakpoints. The dialog allows to"""
-            """ delete or change these breakpoints.</p>"""
-        ))
-        act.connectIt(SIGNAL('activated()'), self.handleAllBp)
-        self.actions.append(act)
-        
-        self.dbgProjectBpAct = E3Action(self.trUtf8('Project Breakpoints'),
-                self.trUtf8('Pro&ject Breakpoints...'), 0, 0, self,
-                'dbg_project_breakpoints')
-        self.dbgProjectBpAct.setStatusTip(self.trUtf8('Show the projects breakpoints'))
-        self.dbgProjectBpAct.setWhatsThis(self.trUtf8(
-            """<b>Project Breakpoints</b>"""
-            """<p>Show all breakpoints of the current project. The dialog allows to"""
-            """ delete or change these breakpoints.</p>"""
-        ))
-        self.dbgProjectBpAct.connectIt(SIGNAL('activated()'), self.handleProjectBp)
-        self.actions.append(self.dbgProjectBpAct)
-        
         self.debugActGrp.setEnabled(0)
         self.debugActGrp2.setEnabled(0)
         self.dbgSetBpActGrp.setEnabled(0)
-        self.dbgAllBpActGrp.setEnabled(0)
-        self.dbgProjectBpAct.setEnabled(0)
         self.runAct.setEnabled(0)
         self.runProjectAct.setEnabled(0)
         self.profileAct.setEnabled(0)
@@ -546,9 +510,6 @@ class DebugUI(QObject):
         self.connect(self.breakpointsMenu,SIGNAL('aboutToShow()'),self.handleShowBreakpointsMenu)
         self.connect(self.breakpointsMenu,SIGNAL('activated(int)'),self.handleBreakpointSelected)
         menu.insertSeparator()
-        self.dbgProjectBpAct.addTo(menu)
-        self.dbgAllBpActGrp.addTo(menu)
-        menu.insertSeparator()
         self.dbgFilterAct.addTo(menu)
         self.excFilterAct.addTo(menu)
         
@@ -650,7 +611,6 @@ class DebugUI(QObject):
         
         @param fn filename of the opened editor
         """
-        self.dbgAllBpActGrp.setEnabled(1)
         self.editorOpen = 1
         
         if fn:
@@ -671,7 +631,6 @@ class DebugUI(QObject):
         self.cyclopsAct.setEnabled(0)
         self.debugActGrp.setEnabled(0)
         self.debugActGrp2.setEnabled(0)
-        self.dbgAllBpActGrp.setEnabled(0)
         self.dbgSetBpActGrp.setEnabled(0)
         self.lastAction = -1
         if not self.projectOpen:
@@ -690,14 +649,22 @@ class DebugUI(QObject):
         else:
             fn = None
             
-        if fn and (fn.endswith('.py') or editor.isPyFile()):
+        if fn and ((os.path.splitext(fn)[1] in ['.py', '.rb']) or editor.isPyFile()):
+            ext = os.path.splitext(fn)[1]
+            if ext == '.py' or editor.isPyFile():
+                cap = self.dbs.getClientCapabilities('Python')
+            elif ext == '.rb':
+                cap = self.dbs.getClientCapabilities('Ruby')
+            else:
+                cap = 0
+                
             if not self.passive:
-                self.runAct.setEnabled(1)
-                self.coverageAct.setEnabled(1)
-                self.cyclopsAct.setEnabled(1)
-                self.profileAct.setEnabled(1)
-                self.debugAct.setEnabled(1)
-            self.dbgSetBpActGrp.setEnabled(1)
+                self.runAct.setEnabled(cap & HasInterpreter)
+                self.coverageAct.setEnabled(cap & HasCoverage)
+                self.cyclopsAct.setEnabled(cap & HasCyclesFinder)
+                self.profileAct.setEnabled(cap & HasProfiler)
+                self.debugAct.setEnabled(cap & HasDebugger)
+            self.dbgSetBpActGrp.setEnabled(cap & HasDebugger)
             if editor.curLineHasBreakpoint():
                 self.dbgEditBpAct.setEnabled(1)
             else:
@@ -705,11 +672,9 @@ class DebugUI(QObject):
             if editor.hasBreakpoints():
                 self.dbgNextBpAct.setEnabled(1)
                 self.dbgPrevBpAct.setEnabled(1)
-                self.dbgFileBpAct.setEnabled(1)
             else:
                 self.dbgNextBpAct.setEnabled(0)
                 self.dbgPrevBpAct.setEnabled(0)
-                self.dbgFileBpAct.setEnabled(0)
         else:
             self.runAct.setEnabled(0)
             self.coverageAct.setEnabled(0)
@@ -717,7 +682,6 @@ class DebugUI(QObject):
             self.profileAct.setEnabled(0)
             self.debugAct.setEnabled(0)
             self.dbgSetBpActGrp.setEnabled(0)
-            self.dbgFileBpAct.setEnabled(0)
         
     def handleCursorChanged(self, editor):
         """
@@ -738,24 +702,22 @@ class DebugUI(QObject):
             if editor.hasBreakpoints():
                 self.dbgNextBpAct.setEnabled(1)
                 self.dbgPrevBpAct.setEnabled(1)
-                self.dbgFileBpAct.setEnabled(1)
             else:
                 self.dbgNextBpAct.setEnabled(0)
                 self.dbgPrevBpAct.setEnabled(0)
-                self.dbgFileBpAct.setEnabled(0)
         
     def handleProjectOpened(self):
         """
         Private slot to handle the projectOpened signal.
         """
         self.projectOpen = 1
+        cap = self.dbs.getClientCapabilities(self.project.pdata["PROGLANGUAGE"][0])
         if not self.passive:
-            self.debugProjectAct.setEnabled(1)
-            self.runProjectAct.setEnabled(1)
-            self.profileProjectAct.setEnabled(1)
-            self.coverageProjectAct.setEnabled(1)
-            self.cyclopsProjectAct.setEnabled(1)
-        self.dbgProjectBpAct.setEnabled(1)
+            self.debugProjectAct.setEnabled(cap & HasDebugger)
+            self.runProjectAct.setEnabled(cap & HasInterpreter)
+            self.profileProjectAct.setEnabled(cap & HasProfiler)
+            self.coverageProjectAct.setEnabled(cap & HasCoverage)
+            self.cyclopsProjectAct.setEnabled(cap & HasCyclesFinder)
         
     def handleProjectClosed(self):
         """
@@ -767,7 +729,6 @@ class DebugUI(QObject):
         self.coverageProjectAct.setEnabled(0)
         self.cyclopsProjectAct.setEnabled(0)
         self.debugProjectAct.setEnabled(0)
-        self.dbgProjectBpAct.setEnabled(0)
         
         if not self.editorOpen:
             self.restartAct.setEnabled(0)
@@ -802,11 +763,12 @@ class DebugUI(QObject):
         self.debugActGrp.setEnabled(0)
         self.debugActGrp2.setEnabled(0)
         if not self.passive:
-            self.debugAct.setEnabled(self.editorOpen)
-            self.runAct.setEnabled(self.editorOpen)
-            self.profileAct.setEnabled(self.editorOpen)
-            self.coverageAct.setEnabled(self.editorOpen)
-            self.cyclopsAct.setEnabled(self.editorOpen)
+            if self.editorOpen:
+                editor = self.viewmanager.activeWindow()
+            else:
+                editor = None
+            self.checkActions(editor)
+            
             self.debugProjectAct.setEnabled(self.projectOpen)
             self.runProjectAct.setEnabled(self.projectOpen)
             self.profileProjectAct.setEnabled(self.projectOpen)
@@ -828,6 +790,8 @@ class DebugUI(QObject):
         """
         self.ui.raiseW()
         self.ui.setActiveWindow()
+        if self.ui.getViewProfile() != "debug":
+            self.ui.handleSetDebugProfile()
         self.viewmanager.setFileLine(fn,line)
         self.getClientVariables()
 
@@ -841,10 +805,15 @@ class DebugUI(QObject):
 
         self.resetUI()
 
-        KQMessageBox.information(self.ui,Program,
-            self.trUtf8('<p><b>%1</b> has terminated with an exit status of %2.</p>')
-                .arg(Utilities.normabspath(self.ui.currentProg))
-                .arg(status))
+        if self.ui.currentProg is None:
+            KQMessageBox.information(self.ui,Program,
+                self.trUtf8('<p>The program has terminated with an exit status of %1.</p>')
+                    .arg(status))
+        else:
+            KQMessageBox.information(self.ui,Program,
+                self.trUtf8('<p><b>%1</b> has terminated with an exit status of %2.</p>')
+                    .arg(Utilities.normabspath(self.ui.currentProg))
+                    .arg(status))
 
     def handleSyntaxError(self,status):
         """
@@ -1094,31 +1063,15 @@ class DebugUI(QObject):
         self.viewmanager.displayPythonFile(\
             self.breakpoints[id][0], self.breakpoints[id][1])
         
-    def handleAllFileBp(self):
+    def handleGotoBreakpoint(self, filename, lineno):
         """
-        Private slot for displaying the breakpoints of the current file.
-        """
-        fn = self.viewmanager.getActiveName()
-        if fn is not None:
-            BreakpointDialog(self.ui, 'BPDialog', 0, Qt.WDestructiveClose,
-                fn).show()
-        else:
-            KQMessageBox.information(self.ui, Program, 
-                self.trUtf8('No source window active'), self.trUtf8('&OK'))
+        Public method to show the source related to a breakpoint.
         
-    def handleAllBp(self):
+        @param filename name of the related file (string or QString)
+        @param line number of the breakpoint (integer)
         """
-        Private slot for displaying all breakpoints.
-        """
-        BreakpointDialog(self.ui, 'BPDialog', 0, Qt.WDestructiveClose).show()
-
-    def handleProjectBp(self):
-        """
-        Private slot to display all breakpoints of the current project
-        """
-        BreakpointDialog(self.ui, 'BPDialog', 0, Qt.WDestructiveClose,
-            'project').show()
-            
+        self.viewmanager.displayPythonFile(unicode(filename), lineno)
+        
     def compileChangedProjectForms(self):
         """
         Private method to signal compilation of changed forms is wanted.
@@ -1147,6 +1100,8 @@ class DebugUI(QObject):
         @param runProject flag indicating coverage of the current project (1)
                 or script (0)
         """
+        self.resetUI()
+        
         # Get the command line arguments, the working directory and the
         # exception reporting flag.
         if runProject:
@@ -1237,6 +1192,8 @@ class DebugUI(QObject):
         @param runProject flag indicating profiling of the current project (1)
                 or script (0)
         """
+        self.resetUI()
+        
         # Get the command line arguments, the working directory and the
         # exception reporting flag.
         if runProject:
@@ -1432,6 +1389,8 @@ class DebugUI(QObject):
         @param unProject flag indicating running the current project (1)
                 or script (0)
         """
+        self.resetUI()
+        
         # Get the command line arguments, the working directory and the
         # exception reporting flag.
         if runProject:
@@ -1515,6 +1474,8 @@ class DebugUI(QObject):
         @param debugProject flag indicating debugging the current project (1)
                 or script (0)
         """
+        self.resetUI()
+        
         # Get the command line arguments, the working directory and the
         # exception reporting flag.
         if debugProject:
@@ -1597,6 +1558,8 @@ class DebugUI(QObject):
         """
         Private slot to handle the restart action to restart the last debugged file.
         """
+        self.resetUI()
+        
         # first save any changes
         if self.lastStartAction in [1, 3, 5, 7, 9]:
             dummy, editor = self.viewmanager.getEditor(self.lastDebuggedFile)
@@ -1665,6 +1628,9 @@ class DebugUI(QObject):
         # Set exception reporting
         self.setExceptionReporting(exc)
         
+        # Signal that we have started a debugging session
+        self.emit(PYSIGNAL('debuggingStarted'), (fn,))
+        
     def handleContinue(self):
         """
         Private method to handle the Continue action.
