@@ -18,11 +18,13 @@ from VariableDetailDialog import VariableDetailDialog
 from Utilities import toUnicode
 
 
-class ClassNode(QListViewItem):
+class SpecialVarNode(QListViewItem):
     """
-    Class implementing a QListViewItem that represents a class.  
+    Class implementing a QListViewItem that represents a special variable node.
+    
+    These special variable nodes are generated for classes, lists, tuples and dictionaries.
     """
-    def __init__(self,parent, dvar, dvalue, dtype, frmnr, scope):
+    def __init__(self, parent, dvar, dvalue, dtype, frmnr, scope):
         """
         Constructor
         
@@ -31,7 +33,7 @@ class ClassNode(QListViewItem):
         @param dvalue value string (string or QString)
         @param dtype type string (string or QString)
         @param frmnr frame number (0 is the current frame) (int)
-        @param scope flag indicating global (0) or local (1) variables
+        @param scope flag indicating global (1) or local (0) variables
         """
         QListViewItem.__init__(self,parent,dvar,dvalue,dtype)
         
@@ -56,7 +58,7 @@ class ClassNode(QListViewItem):
                 
             dbg = qApp.mainWidget().getDebugger()
             dbs = qApp.mainWidget().getDebugServer()
-            if self.scope == 0:
+            if self.scope == 1:
                 filter = dbg.globalsVarFilter
             else:
                 filter = dbg.localsVarFilter
@@ -93,12 +95,17 @@ class VariablesViewer(QListView):
         Constructor
         
         @param parent the parent (QWidget)
-        @param scope flag indicating global (0) or local (1) variables
+        @param scope flag indicating global (1) or local (0) variables
         """
         QListView.__init__(self,parent)
         
-        self.indicators = {'list' : '[]', 'tuple' : '()', 'dict' : '{}'}
+        self.indicators = {'list' : '[]', 'tuple' : '()', 'dict' : '{}', # Python types
+                           'Array' : '[]', 'Hash' : '{}'}                # Ruby types
         self.rx_class = QRegExp('<.*(instance|object) at 0x.*>')
+        self.rx_class2 = QRegExp('class .*')
+        self.rx_class3 = QRegExp('<class .* at 0x.*>')
+        self.dvar_rx_class1 = QRegExp(r'<.*(instance|object) at 0x.*>(\[\]|\{\}|\(\))')
+        self.dvar_rx_class2 = QRegExp(r'<class .* at 0x.*>(\[\]|\{\}|\(\))')
         self.framenr = 0
 
         self.loc = sys.getdefaultencoding()
@@ -170,6 +177,9 @@ class VariablesViewer(QListView):
                 </ul>
         @param frmnr frame number (0 is the current frame) (int)
         """
+        self.current = self.selectedItem()
+        if self.current:
+            self.curpathlist = self.__buildTreePath(self.current)
         self.clear()
         self.framenr = frmnr
         
@@ -187,6 +197,13 @@ class VariablesViewer(QListView):
                     itm.setOpen(1)
                 else:
                     self.openItems.append(itemPath)
+            
+            if self.current:
+                citm = self.findItem(self.curpathlist, 0)
+                if citm:
+                    self.setCurrentItem(citm)
+                    self.setSelected(citm, 1)
+                    self.ensureItemVisible(citm)
 
     def showVariable(self, vlist):
         """
@@ -200,6 +217,10 @@ class VariablesViewer(QListView):
                 <li>the variables value (string)</li>
                 </ul>
         """
+        self.current = self.selectedItem()
+        if self.current:
+            self.curpathlist = self.__buildTreePath(self.current)
+        
         if len(vlist):
             itm = self.findItem(vlist[0], 0)
             for (var, vtype, value) in vlist[1:]:
@@ -214,8 +235,15 @@ class VariablesViewer(QListView):
             if itm is not None and not itm.isOpen():
                 itm.setOpen(1)
         self.openItems = openItems[:]
+            
+        if self.current:
+            citm = self.findItem(self.curpathlist, 0)
+            if citm:
+                self.setCurrentItem(citm)
+                self.setSelected(citm, 1)
+                self.ensureItemVisible(citm)
 
-    def generateItem(self, parent, dvar, dvalue, dtype):
+    def generateItem(self, parent, dvar, dvalue, dtype, isSpecial = 0):
         """
         Private method used to generate a QListViewItem representing a variable.
         
@@ -223,39 +251,25 @@ class VariablesViewer(QListView):
         @param dvar variable name (string or QString)
         @param dvalue value string (string or QString)
         @param dtype type string (string or QString)
-        @return The item that was generated (QListViewItem or ClassNode).
+        @param isSpecial flag indicating that a special node should be generated (boolean)
+        @return The item that was generated (QListViewItem or SpecialVarNode).
         """
-        if self.rx_class.exactMatch(dvalue):
-            return ClassNode(parent, dvar, dvalue, dtype, self.framenr, self.scope)
+        if isSpecial and \
+           (self.dvar_rx_class1.exactMatch(dvar) or \
+            self.dvar_rx_class2.exactMatch(dvar)):
+            isSpecial = 0
+        
+        if self.rx_class2.exactMatch(dtype):
+            return SpecialVarNode(parent, dvar, dvalue, dtype[7:-1], self.framenr, self.scope)
+        elif self.rx_class.exactMatch(dvalue) or \
+             self.rx_class3.exactMatch(dvalue) or \
+             isSpecial:
+            return SpecialVarNode(parent, dvar, dvalue, dtype, self.framenr, self.scope)
         else:
             if isinstance(dvalue, str):
                 dvalue = QString.fromLocal8Bit( dvalue )
             return QListViewItem(parent, dvar, dvalue, dtype)
         
-    def evalValueString(self, value):
-        """
-        Private method to evaluate a string.
-        
-        @param value value string to be evaluated
-        @return the value of the string or None in case of problems
-        """
-        try:
-            return eval(value)
-        except SyntaxError:
-            try:
-                vv = value.replace(': <',': "<')\
-                    .replace('[<','["<')\
-                    .replace('{<','{"<')\
-                    .replace('(<','("<')\
-                    .replace('>, <','>", "<')\
-                    .replace('>,','>",')\
-                    .replace('>}','>"}')\
-                    .replace('>]','>"]')\
-                    .replace('>)','>")')
-                return eval(vv)
-            except SyntaxError:
-                return None
-        
     def __unicode(self, s):
         """
         Private method to convert a string to unicode.
@@ -296,34 +310,8 @@ class VariablesViewer(QListView):
             dvar = var
         dvtype = self.getDispType(vtype)
         
-        if vtype in ['list', 'tuple']:
-            itm = self.generateItem(parent, dvar, '', dvtype)
-            list = self.evalValueString(value)
-            if list:
-                format = "%%%dd" % int(log10(len(list))+1)
-                for i in range(len(list)):
-                    lvtype = self.getType(list[i])
-                    ldvtype = self.getDispType(lvtype)
-                    slist = self.__unicode(list[i])
-                    if lvtype in ['list', 'tuple', 'dict']:
-                        self.addItem(itm, lvtype, format % i, slist)
-                    else:
-                        self.generateItem(itm, format % i, slist, ldvtype)
-            
-        elif vtype == 'dict':
-            itm = self.generateItem(parent, dvar, '', dvtype)
-            dict = self.evalValueString(value)
-            if dict:
-                for key, val in dict.items():
-                    dvtype = self.getType(val)
-                    ddvtype = self.getDispType(dvtype)
-                    if dvtype in ['list', 'tuple', 'dict']:
-                        self.addItem(itm, dvtype, self.__unicode(key), self.__unicode(val))
-                    else:
-                        if type(key) is types.StringType:
-                            key = "'%s'" % key
-                        self.generateItem(itm, self.__unicode(key), self.__unicode(val), ddvtype)
-            
+        if vtype in ['list', 'Array', 'tuple', 'dict', 'Hash']:
+            itm = self.generateItem(parent, dvar, self.trUtf8("%1 items").arg(value), dvtype, 1)
         elif vtype in ['unicode', 'str']:
             try:
                 sval = eval(value)
@@ -336,31 +324,6 @@ class VariablesViewer(QListView):
             
         return itm
 
-    def getType(self, value):
-        """
-        Private method used to get the type of the value passed in.
-        
-        @param value the real value (any)
-        @return type (string).
-        """
-        if type(value) is types.StringType and \
-           value.startswith('<'):
-            if value.find('instance') > -1:
-                return 'instance'
-            elif value.find('object') > -1:
-                val = value.split()[0][1:]
-                return val
-            elif value.startswith('<function'):
-                return 'function'
-            
-        valtypestr = str(type(value))[1:-1]
-        if valtypestr.split(' ',1)[0] == 'class':
-            # handle new class type of python 2.2+
-            valtype = valtypestr[7:-1]
-        else:
-            valtype = valtypestr[6:-1]
-        return valtype
-
     def getDispType(self, vtype):
         """
         Private method used to get the display string for type vtype.
@@ -373,7 +336,12 @@ class VariablesViewer(QListView):
             i = ConfigVarTypeStrings.index(vtype)
             dvtype = self.trUtf8(ConfigVarTypeDispStrings[i])
         except ValueError:
-            dvtype = vtype
+            if vtype == 'classobj':
+                dvtype = self.trUtf8(\
+                    ConfigVarTypeDispStrings[ConfigVarTypeStrings.index('instance')]\
+                )
+            else:
+                dvtype = vtype
         return dvtype
 
     def contentsMouseDoubleClickEvent(self, mouseEvent):
@@ -394,6 +362,8 @@ class VariablesViewer(QListView):
             
         vtype = itm.text(2)
         name = unicode(itm.text(0))
+        if name[-2:] in ['[]', '{}', '()']:
+            name = name[:-2]
         
         par = itm.parent()
         nlist = [name]
@@ -401,7 +371,10 @@ class VariablesViewer(QListView):
         while par is not None:
             pname = unicode(par.text(0))
             if pname[-2:] in ['[]', '{}', '()']:
-                nlist[0] = '[%s]' % nlist[0]
+                if nlist[0].endswith("."):
+                    nlist[0] = '[%s].' % nlist[0][:-1]
+                else:
+                    nlist[0] = '[%s]' % nlist[0]
                 nlist.insert(0, pname[:-2])
             else:
                 nlist.insert(0, '%s.' % pname)
