@@ -16,6 +16,9 @@ import re
 
 from TemplatesListsStyle import *
 
+from Utilities import html_uencode
+from Utilities.ModuleParser import RB_SOURCE
+
 _signal = re.compile(r"""
     ^@signal [ \t]+ 
     (?P<SignalName1>
@@ -111,12 +114,22 @@ class ModuleDocument:
         classList = self.genClassListSection()
         functionList = self.genFunctionListSection()
         try:
-            modBody = moduleTemplate % { \
-                'Module' : self.module.name,
-                'ModuleDescription' : self.formatDescription(self.module.description),
-                'ClassList' : classList,
-                'FunctionList' : functionList,
-            }
+            if self.module.type == RB_SOURCE:
+                rbModulesList = self.genRbModulesListSection()
+                modBody = rbFileTemplate % { \
+                    'Module' : self.module.name,
+                    'ModuleDescription' : self.formatDescription(self.module.description),
+                    'ClassList' : classList,
+                    'RbModulesList' : rbModulesList,
+                    'FunctionList' : functionList,
+                }
+            else:
+                modBody = moduleTemplate % { \
+                    'Module' : self.module.name,
+                    'ModuleDescription' : self.formatDescription(self.module.description),
+                    'ClassList' : classList,
+                    'FunctionList' : functionList,
+                }
         except TagError, e:
             sys.stderr.write("Error in tags of description of module %s.\n" % \
                 self.module.name)
@@ -125,7 +138,11 @@ class ModuleDocument:
             
         classesSection = self.genClassesSection()
         functionsSection = self.genFunctionsSection()
-        return "%s%s%s" % (modBody, classesSection, functionsSection)
+        if self.module.type == RB_SOURCE:
+            rbModulesSection = self.genRbModulesSection()
+        else:
+            rbModulesSection = ""
+        return "%s%s%s%s" % (modBody, classesSection, rbModulesSection, functionsSection)
         
     def genListSection(self, names, dict, className=''):
         """
@@ -163,6 +180,23 @@ class ModuleDocument:
             'Entries' : s,
         }
         
+    def genRbModulesListSection(self):
+        """
+        Private method to generate the section listing all modules of the file (Ruby only).
+        
+        @return The modules list section. (string)
+        """
+        names = self.module.modules.keys()
+        names.sort()
+        if names:
+            self.empty = 0
+            s = self.genListSection(names, self.module.modules)
+        else:
+            s = listEntryNoneTemplate
+        return listTemplate % { \
+            'Entries' : s,
+        }
+        
     def genFunctionListSection(self):
         """
         Private method to generate the section listing all functions of the module.
@@ -190,20 +224,21 @@ class ModuleDocument:
         classNames.sort()
         classes = []
         for className in classNames:
-            supers = self.module.classes[className].super
+            _class = self.module.classes[className]
+            supers = _class.super
             if len(supers) > 0:
                 supers = ', '.join(supers)
             else:
-                supers = ''
+                supers = 'None'
                 
-            methList, methBodies = self.genMethodSection(className)
+            methList, methBodies = self.genMethodSection(_class, className)
             
             try:
                 clsBody = classTemplate % { \
                     'Anchor' : className,
-                    'Class' : self.module.classes[className].name,
+                    'Class' : _class.name,
                     'ClassSuper' : supers,
-                    'ClassDescription' : self.formatDescription(self.module.classes[className].description),
+                    'ClassDescription' : self.formatDescription(_class.description),
                     'MethodList' : methList,
                     'MethodDetails' : methBodies,
                 }
@@ -217,47 +252,49 @@ class ModuleDocument:
             
         return ''.join(classes)
         
-    def genMethodsListSection(self, names, dict, className=''):
+    def genMethodsListSection(self, names, dict, className, clsName):
         """
         Private method to generate the methods list section of a class.
         
         @param names The names to appear in the list. (list of strings)
         @param dict A dictionary containing all relevant information.
         @param className The class name containing the names.
+        @param clsName The visible class name containing the names.
         @return The list section. (string)
         """
         lst = []
-        clsName = self.module.classes[className].name
         try:
             lst.append(listEntryTemplate % { \
-                'Link' : "%s%s" % (className, '__init__'),
+                'Link' : "%s.%s" % (className, '__init__'),
                 'Name' : clsName,
                 'Description' : self.getShortDescription(dict['__init__'].description),
-                'Deprecated' : self.checkDeprecated(dict['__init__'].description) and listEntryDeprecatedTemplate or "",
+                'Deprecated' : self.checkDeprecated(dict['__init__'].description) and \
+                               listEntryDeprecatedTemplate or "",
             })
         except KeyError:
             pass
             
         for name in names:
             lst.append(listEntryTemplate % { \
-                'Link' : "%s%s" % (className, name),
+                'Link' : "%s.%s" % (className, name),
                 'Name' : dict[name].name,
                 'Description' : self.getShortDescription(dict[name].description),
-                'Deprecated' : self.checkDeprecated(dict[name].description) and listEntryDeprecatedTemplate or "",
+                'Deprecated' : self.checkDeprecated(dict[name].description) and \
+                               listEntryDeprecatedTemplate or "",
             })
         return ''.join(lst)
         
-    def genMethodSection(self, className):
+    def genMethodSection(self, obj, className):
         """
         Private method to generate the method details section.
         
-        @param classname Name of the class containing the method. (string)
-        @return The method details section. (string)
+        @param obj Reference to the object being formatted.
+        @param className Name of the class containing the method. (string)
+        @return The method list and method details section. (tuple of two string)
         """
         methList = []
         methBodies = []
-        _class = self.module.classes[className]
-        methods = _class.methods.keys()
+        methods = obj.methods.keys()
         methods.sort()
         
         # first do the constructor
@@ -266,14 +303,14 @@ class ModuleDocument:
             try:
                 methBody = constructorTemplate % { \
                     'Anchor' : className,
-                    'Class' : _class.name,
+                    'Class' : obj.name,
                     'Method' : '__init__',
-                    'MethodDescription' : self.formatDescription(_class.methods['__init__'].description),
-                    'Params' : ', '.join(_class.methods['__init__'].parameters[1:]),
+                    'MethodDescription' : self.formatDescription(obj.methods['__init__'].description),
+                    'Params' : ', '.join(obj.methods['__init__'].parameters[1:]),
                 }
             except TagError, e:
                 sys.stderr.write("Error in tags of description of method %s.%s.\n" % \
-                    (className, method))
+                    (className, '__init__'))
                 sys.stderr.write("%s\n" % e.args)
                 methBody = ""
             methBodies.append(methBody)
@@ -282,10 +319,10 @@ class ModuleDocument:
             try:
                 methBody = methodTemplate % { \
                     'Anchor' : className,
-                    'Class' : _class.name,
-                    'Method' : _class.methods[method].name,
-                    'MethodDescription' : self.formatDescription(_class.methods[method].description),
-                    'Params' : ', '.join(_class.methods[method].parameters[1:]),
+                    'Class' : obj.name,
+                    'Method' : obj.methods[method].name,
+                    'MethodDescription' : self.formatDescription(obj.methods[method].description),
+                    'Params' : ', '.join(obj.methods[method].parameters[1:]),
                 }
             except TagError, e:
                 sys.stderr.write("Error in tags of description of method %s.%s.\n" % \
@@ -294,12 +331,116 @@ class ModuleDocument:
                 methBody = ""
             methBodies.append(methBody)
             
-        methList = self.genMethodsListSection(methods, _class.methods, className)
+        methList = self.genMethodsListSection(methods, obj.methods, className, obj.name)
         
+        if not methList:
+            methList = listEntryNoneTemplate
         return listTemplate % { \
             'Entries' : methList,
             }, ''.join(methBodies)
         
+    def genRbModulesSection(self):
+        """
+        Private method to generate the document section with details about Ruby modules.
+        
+        @return The Ruby modules details section. (string)
+        """
+        rbModulesNames = self.module.modules.keys()
+        rbModulesNames.sort()
+        rbModules = []
+        for rbModuleName in rbModulesNames:
+            rbModule = self.module.modules[rbModuleName]
+            methList, methBodies = self.genMethodSection(rbModule, rbModuleName)
+            classList, classBodies = self.genRbModulesClassesSection(rbModule, rbModuleName)
+            
+            try:
+                rbmBody = rbModuleTemplate % { \
+                    'Anchor' : rbModuleName,
+                    'Module' : rbModule.name,
+                    'ModuleDescription' : self.formatDescription(rbModule.description),
+                    'ClassesList' : classList,
+                    'ClassesDetails' : classBodies,
+                    'FunctionsList' : methList,
+                    'FunctionsDetails' : methBodies,
+                }
+            except TagError, e:
+                sys.stderr.write("Error in tags of description of Ruby module %s.\n" % \
+                    rbModuleName)
+                sys.stderr.write("%s\n" % e.args)
+                rbmBody = ""
+            
+            rbModules.append(rbmBody)
+            
+        return ''.join(rbModules)
+
+    def genRbModulesClassesSection(self, obj, modName):
+        """
+        Private method to generate the Ruby module classes details section.
+        
+        @param obj Reference to the object being formatted.
+        @param modName Name of the Ruby module containing the classes. (string)
+        @return The classes list and classes details section. (tuple of two string)
+        """
+        classNames = obj.classes.keys()
+        classNames.sort()
+        classes = []
+        for className in classNames:
+            _class = obj.classes[className]
+            supers = _class.super
+            if len(supers) > 0:
+                supers = ', '.join(supers)
+            else:
+                supers = 'None'
+            
+            classname = "%s.%s" % (modName, className)
+            methList, methBodies = self.genMethodSection(_class, className)
+            
+            try:
+                clsBody = rbModulesClassTemplate % { \
+                    'Anchor' : className,
+                    'Class' : _class.name,
+                    'ClassSuper' : supers,
+                    'ClassDescription' : self.formatDescription(_class.description),
+                    'MethodList' : methList,
+                    'MethodDetails' : methBodies,
+                }
+            except TagError, e:
+                sys.stderr.write("Error in tags of description of class %s.\n" % \
+                    className)
+                sys.stderr.write("%s\n" % e.args)
+                clsBody = ""
+            
+            classes.append(clsBody)
+            
+            
+        classesList = self.genRbModulesClassesListSection(classNames, obj.classes, modName)
+        
+        if not classesList:
+            classesList = listEntryNoneTemplate
+        return listTemplate % { \
+            'Entries' : classesList,
+            }, ''.join(classes)
+        
+    def genRbModulesClassesListSection(self, names, dict, moduleName):
+        """
+        Private method to generate the classes list section of a Ruby module.
+        
+        @param names The names to appear in the list. (list of strings)
+        @param dict A dictionary containing all relevant information.
+        @param moduleName Name of the Ruby module containing the classes. (string)
+        @return The list section. (string)
+        """
+        lst = []
+        for name in names:
+            lst.append(listEntryTemplate % { \
+                'Link' : "%s.%s" % (moduleName, name),
+                'Name' : dict[name].name,
+                'Description' : self.getShortDescription(dict[name].description),
+                'Deprecated' : self.checkDeprecated(dict[name].description) and \
+                               listEntryDeprecatedTemplate or "",
+            })
+        return ''.join(lst)
+        
     def genFunctionsSection(self):
         """
         Private method to generate the document section with details about functions.
@@ -340,7 +481,7 @@ class ModuleDocument:
         dlist = desc.splitlines()
         for desc in dlist:
             if desc:
-                return desc
+                return html_uencode(desc)
         else:
             return ''
             
@@ -376,7 +517,7 @@ class ModuleDocument:
                 if line == '.':
                     linelist.append("")
                 else:
-                    linelist.append(line)
+                    linelist.append(html_uencode(line))
             else:
                 lst.append(paragraphTemplate % { \
                     'Lines' : '\n'.join(linelist)
@@ -403,7 +544,7 @@ class ModuleDocument:
         for key in keys:
             lst.append(template % { \
                 'Name' : key,
-                'Description' : '\n'.join(dictionary[key]),
+                'Description' : html_uencode('\n'.join(dictionary[key])),
             })
         return ''.join(lst)
         
@@ -523,7 +664,7 @@ class ModuleDocument:
             parameterSect = ""
             
         if returns:
-            returnSect = returnsTemplate %'\n'.join(returns)
+            returnSect = returnsTemplate % html_uencode('\n'.join(returns))
         else:
             returnSect = ""
             
@@ -545,14 +686,14 @@ class ModuleDocument:
             
         if deprecated:
             deprecatedSect = deprecatedTemplate % { \
-                'Lines' : '\n'.join(deprecated),
+                'Lines' : html_uencode('\n'.join(deprecated)),
             }
         else:
             deprecatedSect = ""
         
         if authorInfo:
             authorInfoSect = authorInfoTemplate % { \
-                'Authors' : '\n'.join(authorInfo),
+                'Authors' : html_uencode('\n'.join(authorInfo)),
             }
         else:
             authorInfoSect = ""
