@@ -27,7 +27,7 @@ class ApplicationDiagram(QDialog):
     """
     allClasses = {}
     
-    def __init__(self,project,parent = None,name = None,modal = 0,fl = 0):
+    def __init__(self,project,parent = None,name = None,modal = 0,fl = 0, noModules=0):
         """
         Constructor
         
@@ -35,6 +35,7 @@ class ApplicationDiagram(QDialog):
         @param parent parent widget of the view (QWidget)
         @param name name of the view widget (QString or string)
         @param flags the window flags to be passed to the view widget
+        @keyparam noModules flag indicating, that no module names should be shown (boolean)
         """
         QDialog.__init__(self,parent,name,modal,fl)
 
@@ -66,7 +67,7 @@ class ApplicationDiagram(QDialog):
         self.connect(self.closeButton,SIGNAL("clicked()"),self,SLOT("close()"))
         
         self.project = project
-
+        self.noModules = noModules
 
     def languageChange(self):
         """
@@ -131,7 +132,7 @@ class ApplicationDiagram(QDialog):
         """
         Private method to build the modules shapes of the diagram.
         """
-        project = self.project.ppath.replace(os.sep, '.')
+        project = self.project.ppath.replace(os.sep, '.')[1:]
         packages = {}
         shapes = {}
         p = 10
@@ -243,7 +244,7 @@ class ApplicationDiagram(QDialog):
         """
         modules.sort()
         pm = PackageModel(name, modules)
-        pw = PackageWidget(self.canvas, pm, x, y)
+        pw = PackageWidget(self.canvas, pm, x, y, noModules=self.noModules)
         pw.show()
         rect = pw.rect()
         newSize = self.canvas.size()
