@@ -73,7 +73,7 @@ class ImportsDiagram(QDialog):
         self.connect(self.closeButton,SIGNAL("clicked()"),self,SLOT("close()"))
         
         self.packagePath = Utilities.normabspath(package)
-        self.package = self.packagePath.replace(os.sep, '.')
+        self.package = self.packagePath.replace(os.sep, '.')[1:]
 
 
     def languageChange(self):
@@ -140,7 +140,7 @@ class ImportsDiagram(QDialog):
         Private method to build the modules shapes of the diagram.
         """
         if not os.path.exists(os.path.join(self.packagePath, '__init__.py')):
-            ct = QCanvasText(self.trUtf8("The directory '%1' is not a package.")
+            ct = QCanvasText(self.trUtf8("The directory '%1' is not a Python package.")
                                  .arg(self.packagePath), self.canvas)
             ct.setX(10)
             ct.setY(10)
