@@ -26,7 +26,7 @@ class PackageDiagram(QDialog):
     """
     Class implementing a dialog showing a UML like class diagram of a package.
     """
-    def __init__(self,package,parent = None,name = None,modal = 0,fl = 0):
+    def __init__(self,package,parent = None,name = None,modal = 0,fl = 0, noAttrs=0):
         """
         Constructor
         
@@ -34,6 +34,7 @@ class PackageDiagram(QDialog):
         @param parent parent widget of the view (QWidget)
         @param name name of the view widget (QString or string)
         @param flags the window flags to be passed to the view widget
+        @keyparam noAttrs flag indicating, that no attributes should be shown (boolean)
         """
         QDialog.__init__(self,parent,name,modal,fl)
 
@@ -66,7 +67,7 @@ class PackageDiagram(QDialog):
         
         self.package = Utilities.normabspath(package)
         self.allClasses = {}
-
+        self.noAttrs = noAttrs
 
     def languageChange(self):
         """
@@ -107,9 +108,12 @@ class PackageDiagram(QDialog):
         
         @return dictionary of modules contained in the package.
         """
+        supportedExt = ['*.py', '*.ptl', '*.rb']
+        
         moduleDict = {}
-        modules = glob.glob(Utilities.normjoinpath(self.package,'*.py')) + \
-                  glob.glob(Utilities.normjoinpath(self.package,'*.ptl'))
+        modules = []
+        for ext in supportedExt:
+            modules.extend(glob.glob(Utilities.normjoinpath(self.package, ext)))
         tot = len(modules)
         try:
             prog = 0
@@ -121,8 +125,6 @@ class PackageDiagram(QDialog):
                 progress.setProgress(prog)
                 qApp.processEvents()
                 prog = prog + 1
-                if module.endswith("__init__.py"):
-                    continue
                 try: 
                     mod = readModule(module)
                 except ImportError:
@@ -142,7 +144,8 @@ class PackageDiagram(QDialog):
         
         The algorithm is borrowed from Boa Constructor.
         """
-        if not os.path.exists(os.path.join(self.package, '__init__.py')):
+        initlist = glob.glob(os.path.join(self.package, '__init__.*'))
+        if len(initlist) == 0:
             ct = QCanvasText(self.trUtf8("The directory '%1' is not a package.")
                                  .arg(self.package), self.canvas)
             ct.setX(10)
@@ -190,13 +193,21 @@ class PackageDiagram(QDialog):
                         cw = self.getCurrentShape(className.split('.')[-1])
                         if cw:
                             self.allClasses[className] = cw
-                    if cw and not (cw.external and module.classes.has_key(className)):
+                    if cw and cw.noAttrs != self.noAttrs:
+                        cw = None
+                    if cw and not (cw.external and \
+                                   (module.classes.has_key(className) or
+                                    module.modules.has_key(className))
+                                  ):
                         if className not in nodes:
                             nodes.append(className)
                     else:
                         if module.classes.has_key(className):
                             # this is a local class (defined in this module)
                             self.addLocalClass(className, module.classes[className], 10, 10)
+                        elif module.modules.has_key(className):
+                            # this is a local module (defined in this module)
+                            self.addLocalClass(className, module.modules[className], 10, 10, 1)
                         else:
                             self.addExternalClass(className, 10, 10)
                         nodes.append(className)
@@ -286,7 +297,7 @@ class PackageDiagram(QDialog):
                 x = x + rect.width() + whiteSpace
             y = y + currentHeight + verticalWhiteSpace
             
-    def addLocalClass(self, className, _class, x, y):
+    def addLocalClass(self, className, _class, x, y, isRbModule=0):
         """
         Private method to add a class defined in the module.
         
@@ -297,13 +308,17 @@ class PackageDiagram(QDialog):
         @param _class class to be shown (ModuleParser.Class)
         @param x x-coordinate (integer)
         @param y y-coordinate (integer)
+        @param isRbModule flag indicating a Ruby module (boolean)
         """
         meths = _class.methods.keys()
         meths.sort()
         attrs = _class.attributes[:]
         attrs.sort()
-        cl = ClassModel(_class.name, meths, attrs)
-        cw = ClassWidget(self.canvas, cl, 0, x, y)
+        name = _class.name
+        if isRbModule:
+            name = "%s (Module)" % name
+        cl = ClassModel(name, meths, attrs)
+        cw = ClassWidget(self.canvas, cl, 0, x, y, noAttrs=self.noAttrs)
         cw.show()
         self.allClasses[className] = cw
         rect = cw.rect()
@@ -326,7 +341,7 @@ class PackageDiagram(QDialog):
         @param y y-coordinate (integer)
         """
         cl = ClassModel(_class)
-        cw = ClassWidget(self.canvas, cl, 1, x, y)
+        cw = ClassWidget(self.canvas, cl, 1, x, y, noAttrs=self.noAttrs)
         cw.show()
         self.allClasses[_class] = cw
         rect = cw.rect()
