@@ -24,7 +24,7 @@ class UMLClassDiagram(QDialog):
     Class implementing a dialog showing a UML like class diagram.
     """
     
-    def __init__(self,file,parent = None,name = None,modal = 0,fl = 0):
+    def __init__(self,file,parent = None,name = None,modal = 0,fl = 0, noAttrs=0):
         """
         Constructor
         
@@ -32,6 +32,7 @@ class UMLClassDiagram(QDialog):
         @param parent parent widget of the view (QWidget)
         @param name name of the view widget (QString or string)
         @param flags the window flags to be passed to the view widget
+        @keyparam noAttrs flag indicating, that no attributes should be shown (boolean)
         """
         QDialog.__init__(self,parent,name,modal,fl)
 
@@ -63,7 +64,7 @@ class UMLClassDiagram(QDialog):
         self.connect(self.closeButton,SIGNAL("clicked()"),self,SLOT("close()"))
         
         self.file = file
-
+        self.noAttrs = noAttrs
 
     def languageChange(self):
         """
@@ -132,7 +133,12 @@ class UMLClassDiagram(QDialog):
                         _allClasses[className] = cw
                         if className not in _allModules[self.file]:
                             _allModules[self.file].append(className)
-                if cw and not (cw.external and module.classes.has_key(className)):
+                if cw and cw.noAttrs != self.noAttrs:
+                    cw = None
+                if cw and not (cw.external and \
+                               (module.classes.has_key(className) or
+                                module.modules.has_key(className))
+                              ):
                     cw.setCanvas(self.canvas)
                     cw.show()
                     if className not in nodes:
@@ -141,6 +147,9 @@ class UMLClassDiagram(QDialog):
                     if module.classes.has_key(className):
                         # this is a local class (defined in this module)
                         self.addLocalClass(className, module.classes[className], 10, 10)
+                    elif module.modules.has_key(className):
+                        # this is a local module (defined in this module)
+                        self.addLocalClass(className, module.modules[className], 10, 10, 1)
                     else:
                         self.addExternalClass(className, 10, 10)
                     nodes.append(className)
@@ -237,7 +246,7 @@ class UMLClassDiagram(QDialog):
                 x = x + rect.width() + whiteSpace
             y = y + currentHeight + verticalWhiteSpace
             
-    def addLocalClass(self, className, _class, x, y):
+    def addLocalClass(self, className, _class, x, y, isRbModule=0):
         """
         Private method to add a class defined in the module.
         
@@ -248,13 +257,17 @@ class UMLClassDiagram(QDialog):
         @param _class class to be shown (ModuleParser.Class)
         @param x x-coordinate (integer)
         @param y y-coordinate (integer)
+        @param isRbModule flag indicating a Ruby module (boolean)
         """
         meths = _class.methods.keys()
         meths.sort()
         attrs = _class.attributes[:]
         attrs.sort()
-        cl = ClassModel(_class.name, meths, attrs)
-        cw = ClassWidget(self.canvas, cl, 0, x, y)
+        name = _class.name
+        if isRbModule:
+            name = "%s (Module)" % name
+        cl = ClassModel(name, meths, attrs)
+        cw = ClassWidget(self.canvas, cl, 0, x, y, noAttrs=self.noAttrs)
         cw.show()
         _allClasses[className] = cw
         if _class.name not in _allModules[self.file]:
