@@ -54,8 +54,8 @@ class HelpBrowser(QTextBrowser):
             if webbrowser.isEmpty():
                 KQMessageBox.information(self,
                     self.trUtf8("Helpviewer"),
-                    self.trUtf8("""Currently no Web browser is selected.
-Please use the preferences dialog to specify one."""),
+                    self.trUtf8("""Currently no Web browser is selected."""
+                                """ Please use the preferences dialog to specify one."""),
                     self.trUtf8("&OK"),
                     None,
                     None,
