@@ -20,6 +20,7 @@ import Utilities
 from ConfigurationForm import ConfigurationForm
 from PreferencesLexer import PreferencesLexer
 from IconsPreviewDialog import IconsPreviewDialog
+import UI.PixmapCache
 
 from eric3config import getConfig
 
@@ -43,32 +44,76 @@ class ConfigurationDialog(ConfigurationForm):
         self.configList.setColumnWidth(1,0)
         
         # generate the list entries
+        
+        # editor pages
         itm = QListViewItem(self.configList, self.trUtf8("Editor"), "")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-editor.png"))
+        
         sitm = QListViewItem(itm, self.trUtf8("Highlighting Styles"), "editorSourcePage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-highlighting-styles.png"))
         sitm = QListViewItem(itm, self.trUtf8("Highlighter Association"), "editorHighlighterPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-highlighter-association.png"))
         sitm = QListViewItem(itm, self.trUtf8("General"), "editorGeneralPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-general.png"))
         sitm = QListViewItem(itm, self.trUtf8("Style"), "editorStylePage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-styles.png"))
         sitm = QListViewItem(itm, self.trUtf8("Colours"), "editorColoursPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-colours.png"))
         sitm = QListViewItem(itm, self.trUtf8("Properties"), "editorPropertiesPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-properties.png"))
         sitm = QListViewItem(itm, self.trUtf8("APIs"), "editorAPIsPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-api.png"))
         sitm = QListViewItem(itm, self.trUtf8("Autocompletion"), 
             "editorAutocompletionPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-autocompletion.png"))
         sitm = QListViewItem(itm, self.trUtf8("Calltips"), 
             "editorCalltipsPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-calltips.png"))
+        
         itm.setOpen(1)
+        
+        # debugger pages
+        itm = QListViewItem(self.configList, self.trUtf8("Debugger"), "")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-debugger.png"))
+        
+        sitm = QListViewItem(itm, self.trUtf8("General"), "debuggerGeneralPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-debugger.png"))
+        sitm = QListViewItem(itm, self.trUtf8("Python"), "debuggerPythonPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-pyDebugger.png"))
+        sitm = QListViewItem(itm, self.trUtf8("Ruby"), "debuggerRubyPage")
+        sitm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-rbDebugger.png"))
+        
+        itm.setOpen(1)
+        
+        # the other pages
         itm = QListViewItem(self.configList, self.trUtf8("Printer"), "printerPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-printer.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Project"), "projectPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-project.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Interface"), "interfacePage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-interface.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Viewmanager"), "viewmanagerPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-viewmanager.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Icons"), "iconsPage")
-        itm = QListViewItem(self.configList, self.trUtf8("Debugger"), "debuggerPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-icons.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Help"), "helpPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-help.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Qt"), "qtPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-qtlogo.png"))
         itm = QListViewItem(self.configList, self.trUtf8("CORBA"), "corbaPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-orbit.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Python"), "pythonPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-python.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Refactoring"), "refactoringPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-refactoring.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Email"), "emailPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-mail_generic.png"))
         itm = QListViewItem(self.configList, self.trUtf8("Application"), "applicationPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-application.png"))
+        itm = QListViewItem(self.configList, self.trUtf8("Version Control Systems"), "vcsPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("preferences-vcs.png"))
+        itm = QListViewItem(self.configList, self.trUtf8("Tasks Viewer"), "tasksPage")
+        itm.setPixmap(0, UI.PixmapCache.getPixmap("task.png"))
         
         ericPic = QPixmap(os.path.join(getConfig('ericPixDir'), 'eric.png'))
         self.emptyPagePixmap.setPixmap(ericPic)
@@ -115,6 +160,15 @@ class ConfigurationDialog(ConfigurationForm):
             self.lexers['Bash'] = PreferencesLexer('Bash', self)
         except ImportError:
             pass
+        # the following lexers are only supported for QScintilla > 1.4
+        try:
+            self.lexers['Ruby'] = PreferencesLexer('Ruby', self)
+        except ImportError:
+            pass
+        try:
+            self.lexers['Lua'] = PreferencesLexer('Lua', self)
+        except ImportError:
+            pass
         
         self.foldStyles = [
             QextScintilla.PlainFoldStyle,
@@ -130,8 +184,9 @@ class ConfigurationDialog(ConfigurationForm):
         self.initInterfacePage()
         self.initApplicationPage()
         self.initIconsPage()
-        self.initDebuggerPage()
+        self.initDebuggerPages()
         self.initVariousPages()
+        self.initHelpPage()
         
     def initEditorPages(self):
         """
@@ -144,7 +199,7 @@ class ConfigurationDialog(ConfigurationForm):
             self.lexerLanguageComboBox.insertItem(lang)
         self.handleLexerSelected(QString(''))
         
-        # init syntax highlighter page
+        # init lexer associations page
         for lang in languages:
             self.editorLexerCombo.insertItem(lang)
         lexerAssocs = Preferences.getEditorLexerAssocs()
@@ -265,7 +320,7 @@ class ConfigurationDialog(ConfigurationForm):
         # init properties page
         self.foldPythonCommentCheckBox.setChecked(Preferences.getEditor("PythonFoldComment"))
         self.foldPythonStringCheckBox.setChecked(Preferences.getEditor("PythonFoldString"))
-        self.badIndentationCheckBox.setChecked(Preferences.getEditor("PythonBadIndentation"))
+        self.pythonBadIndentationCheckBox.setChecked(Preferences.getEditor("PythonBadIndentation"))
         self.pythonAutoindentCheckBox.setChecked(Preferences.getEditor("PythonAutoIndent"))
         
         self.foldCppCommentCheckBox.setChecked(Preferences.getEditor("CppFoldComment"))
@@ -303,6 +358,22 @@ class ConfigurationDialog(ConfigurationForm):
             self.bashGroupBox.setEnabled(0)
         self.foldBashCommentCheckBox.setChecked(Preferences.getEditor("BashFoldComment"))
         
+        # enable the Ruby properties only for recent QScintilla versions
+        try:
+            self.lexers['Ruby'].getLexer().indentationWarning()
+            self.rubyGroupBox.setEnabled(1)
+        except (KeyError, AttributeError):
+            self.rubyGroupBox.setEnabled(0)
+        self.rubyBadIndentationCheckBox.setChecked(Preferences.getEditor("RubyBadIndentation"))
+        
+        # enable the Lua properties only for recent QScintilla versions
+        try:
+            self.lexers['Lua'].getLexer().foldCompact()
+            self.luaGroupBox.setEnabled(1)
+        except (KeyError, AttributeError):
+            self.luaGroupBox.setEnabled(0)
+        self.luaFoldCompactCheckBox.setChecked(Preferences.getEditor("LuaFoldCompact"))
+        
         # init api page
         self.apis = {}
         apiLanguages = [''] + Preferences.Prefs.editorAPIDefaults.keys()
@@ -413,29 +484,65 @@ class ConfigurationDialog(ConfigurationForm):
                 itm = QListViewItem(self.iconDirectoryList, lastItem, dir)
             lastItem = itm
         
-    def initDebuggerPage(self):
+    def initDebuggerPages(self):
         """
-        Private method to initialize the debugger page.
+        Private method to initialize the debugger pages.
         """
+        # general page
         self.remoteCheckBox.setChecked(Preferences.getDebugger("RemoteDbgEnabled"))
         self.hostLineEdit.setText(Preferences.getDebugger("RemoteHost"))
         self.execLineEdit.setText(Preferences.getDebugger("RemoteExecution"))
         self.passiveDbgCheckBox.setChecked(Preferences.getDebugger("PassiveDbgEnabled"))
         self.passiveDbgPortSpinBox.setValue(Preferences.getDebugger("PassiveDbgPort"))
+        self.debugEnvironReplaceCheckBox.setChecked(\
+            Preferences.getDebugger("DebugEnvironmentReplace"))
+        self.debugEnvironEdit.setText(Preferences.getDebugger("DebugEnvironment"))
+        self.automaticResetCheckBox.setChecked(Preferences.getDebugger("AutomaticReset"))
+        self.consoleDbgCheckBox.setChecked(Preferences.getDebugger("ConsoleDbgEnabled"))
+        self.consoleDbgEdit.setText(Preferences.getDebugger("ConsoleDbgCommand"))
+
+        # Python page
         self.customPyCheckBox.setChecked(Preferences.getDebugger("CustomPythonInterpreter"))
         self.interpreterEdit.setText(Preferences.getDebugger("PythonInterpreter"))
-        self.debugClientGroup.setButton(Preferences.getDebugger("DebugClientType"))
+        dct = Preferences.getDebugger("DebugClientType")
+        if dct == 1:
+            self.standardButton.setChecked(1)
+        elif dct == 2:
+            self.threadedButton.setChecked(1)
+        elif dct == 3:
+            self.noqtButton.setChecked(1)
+        else:
+            self.customButton.setChecked(1)
         self.debugClientEdit.setText(Preferences.getDebugger("DebugClient"))
-        self.automaticResetCheckBox.setChecked(Preferences.getDebugger("AutomaticReset"))
+        self.pyRedirectCheckBox.setChecked(Preferences.getDebugger("PythonRedirect"))
+        
+        # Ruby page
+        self.rubyInterpreterEdit.setText(Preferences.getDebugger("RubyInterpreter"))
+        self.rbRedirectCheckBox.setChecked(Preferences.getDebugger("RubyRedirect"))
+        
+    def initHelpPage(self):
+        """
+        Private method to initialize the help page.
+        """
+        hvId = Preferences.getHelp("HelpViewerType")
+        if hvId == 1:
+            self.helpBrowserButton.setChecked(1)
+        elif hvId == 2:
+            self.qtAssistantButton.setChecked(1)
+        elif hvId == 3:
+            self.webBrowserButton.setChecked(1)
+        else:
+            self.customViewerButton.setChecked(1)
+        self.customViewerEdit.setText(Preferences.getHelp("CustomViewer"))
+        self.webbrowserEdit.setText(Preferences.getHelp("Webbrowser"))
+        self.pdfviewerEdit.setText(Preferences.getHelp("Pdfviewer"))
+        self.pythonDocDirEdit.setText(Preferences.getHelp("PythonDocDir"))
+        self.qtDocDirEdit.setText(Preferences.getHelp("QtDocDir"))
         
     def initVariousPages(self):
         """
         Private method to initialize the various pages.
         """
-        self.assistantCheckBox.setChecked(Preferences.getHelp("UseQtAssistant"))
-        self.webbrowserEdit.setText(Preferences.getHelp("Webbrowser"))
-        self.pdfviewerEdit.setText(Preferences.getHelp("Pdfviewer"))
-        
         # init the shell settings
         self.stringEncodingComboBox.setCurrentText(Preferences.getSystem("StringEncoding"))
         self.shellLinenowidthSlider.setValue(\
@@ -470,6 +577,14 @@ class ConfigurationDialog(ConfigurationForm):
         self.mailUserEdit.setText(Preferences.getUser("MailServerUser"))
         self.mailPasswordEdit.setText(Preferences.getUser("MailServerPassword"))
         
+        # init the VCS settings
+        self.vcsAutoCloseCheckBox.setChecked(Preferences.getVCS("AutoClose"))
+        self.vcsAutoSaveCheckBox.setChecked(Preferences.getVCS("AutoSaveFiles"))
+        self.vcsAutoSaveProjectCheckBox.setChecked(Preferences.getVCS("AutoSaveProject"))
+        
+        # init the tasks settings
+        self.tasksMarkerEdit.setText(Preferences.getTasks("TasksMarkers"))
+        
     def initLanguageCombo(self):
         """
         Private method to initialize the language combobox of the interface page.
@@ -794,7 +909,7 @@ class ConfigurationDialog(ConfigurationForm):
         else:
             self.separateFileBrowserButton.setEnabled(1)
         
-    def handleInterpreterSelection(self):
+    def handlePythonInterpreterSelection(self):
         """
         Private slot to handle the Python interpreter selection.
         """
@@ -804,6 +919,16 @@ class ConfigurationDialog(ConfigurationForm):
         if not file.isNull():
             self.interpreterEdit.setText(QDir.convertSeparators(file))
         
+    def handleRubyInterpreterSelection(self):
+        """
+        Private slot to handle the Ruby interpreter selection.
+        """
+        file = KQFileDialog.getOpenFileName(self.rubyInterpreterEdit.text(),
+            None, self, None, self.trUtf8("Select Ruby interpreter for Debug Client"))
+            
+        if not file.isNull():
+            self.rubyInterpreterEdit.setText(QDir.convertSeparators(file))
+        
     def handleDebugClientSelection(self):
         """
         Private slot to handle the Debug Client selection.
@@ -897,6 +1022,16 @@ class ConfigurationDialog(ConfigurationForm):
         """
         self.exportQtDirCheckBox.setEnabled(not dir.isEmpty())
         
+    def handleCustomViewerSelection(self):
+        """
+        Private slot to handle the custom viewer selection.
+        """
+        file = KQFileDialog.getOpenFileName(self.customViewerEdit.text(),
+            None, self, None, self.trUtf8("Select Custom Viewer"))
+            
+        if not file.isNull():
+            self.customViewerEdit.setText(QDir.convertSeparators(file))
+        
     def handleWebbrowserSelection(self):
         """
         Private slot to handle the Web browser selection.
@@ -917,6 +1052,32 @@ class ConfigurationDialog(ConfigurationForm):
         if not file.isNull():
             self.pdfviewerEdit.setText(QDir.convertSeparators(file))
         
+    def handlePythonDocDirSelection(self):
+        """
+        Private slot to select the Python documentation directory.
+        """
+        dir = KQFileDialog.getExistingDirectory(\
+            self.pythonDocDirEdit.text(),
+            None, None,
+            self.trUtf8("Select Python documentation directory"),
+            1, 1)
+            
+        if not dir.isEmpty():
+            self.pythonDocDirEdit.setText(QDir.convertSeparators(dir))
+    
+    def handleQtDocDirSelection(self):
+        """
+        Private slot to select the Qt documentation directory.
+        """
+        dir = KQFileDialog.getExistingDirectory(\
+            self.qtDocDirEdit.text(),
+            None, None,
+            self.trUtf8("Select Qt documentation directory"),
+            1, 1)
+            
+        if not dir.isEmpty():
+            self.qtDocDirEdit.setText(QDir.convertSeparators(dir))
+    
     def handleIdlSelection(self):
         """
         Private slot to handle the IDL compiler selection.
@@ -1091,17 +1252,29 @@ class ConfigurationDialog(ConfigurationForm):
         Preferences.setIcons("Path", dirList)
         
         # save the debugger settings
+        # general page
         Preferences.setDebugger("RemoteDbgEnabled", self.remoteCheckBox.isChecked())
         Preferences.setDebugger("RemoteHost", self.hostLineEdit.text())
         Preferences.setDebugger("RemoteExecution", self.execLineEdit.text())
         Preferences.setDebugger("PassiveDbgEnabled", self.passiveDbgCheckBox.isChecked())
         Preferences.setDebugger("PassiveDbgPort", self.passiveDbgPortSpinBox.value())
+        Preferences.setDebugger("DebugEnvironmentReplace", 
+            self.debugEnvironReplaceCheckBox.isChecked())
+        Preferences.setDebugger("DebugEnvironment", self.debugEnvironEdit.text())
+        Preferences.setDebugger("AutomaticReset", self.automaticResetCheckBox.isChecked())
+        Preferences.setDebugger("ConsoleDbgEnabled", self.consoleDbgCheckBox.isChecked())
+        Preferences.setDebugger("ConsoleDbgCommand", self.consoleDbgEdit.text())
+        # Python page
         Preferences.setDebugger("CustomPythonInterpreter", self.customPyCheckBox.isChecked())
         Preferences.setDebugger("PythonInterpreter", self.interpreterEdit.text())
+        sel = self.debugClientGroup.selected()
         Preferences.setDebugger("DebugClientType", 
             self.debugClientGroup.id(self.debugClientGroup.selected()))
         Preferences.setDebugger("DebugClient", self.debugClientEdit.text())
-        Preferences.setDebugger("AutomaticReset", self.automaticResetCheckBox.isChecked())
+        Preferences.setDebugger("PythonRedirect", self.pyRedirectCheckBox.isChecked())
+        # Ruby page
+        Preferences.setDebugger("RubyInterpreter", self.rubyInterpreterEdit.text())
+        Preferences.setDebugger("RubyRedirect", self.rbRedirectCheckBox.isChecked())
         
         # save the editor settings of general page
         Preferences.setEditor("AutosaveInterval", self.autosaveSlider.value())
@@ -1167,7 +1340,7 @@ class ConfigurationDialog(ConfigurationForm):
         Preferences.setEditor("PythonFoldString",
             self.foldPythonStringCheckBox.isChecked())
         Preferences.setEditor("PythonBadIndentation",
-            self.badIndentationCheckBox.isChecked())
+            self.pythonBadIndentationCheckBox.isChecked())
         Preferences.setEditor("PythonAutoIndent",
             self.pythonAutoindentCheckBox.isChecked())
         if self.pythonAutoindentCheckBox.isChecked():
@@ -1203,6 +1376,12 @@ class ConfigurationDialog(ConfigurationForm):
         Preferences.setEditor("BashFoldComment",
             self.foldBashCommentCheckBox.isChecked())
             
+        Preferences.setEditor("RubyBadIndentation",
+            self.rubyBadIndentationCheckBox.isChecked())
+            
+        Preferences.setEditor("LuaFoldCompact",
+            self.luaFoldCompactCheckBox.isChecked())
+        
         # save the editor settings of api page
         lang = self.apiLanguageComboBox.currentText()
         self.apis[unicode(lang)] = self.getApisFromListBox()
@@ -1271,12 +1450,18 @@ class ConfigurationDialog(ConfigurationForm):
             self.projectAutoCompileFormsCheckBox.isChecked())
         
         # save the help settings
-        Preferences.setHelp("UseQtAssistant", 
-            self.assistantCheckBox.isChecked())
+        Preferences.setHelp("HelpViewerType", 
+            self.helpViewerGroup.id(self.helpViewerGroup.selected()))
+        Preferences.setHelp("CustomViewer",
+            self.customViewerEdit.text())
         Preferences.setHelp("Webbrowser",
             self.webbrowserEdit.text())
         Preferences.setHelp("Pdfviewer",
             self.pdfviewerEdit.text())
+        Preferences.setHelp("PythonDocDir",
+            self.pythonDocDirEdit.text())
+        Preferences.setHelp("QtDocDir",
+            self.qtDocDirEdit.text())
         
         # save the system settings
         Preferences.setSystem("StringEncoding",
@@ -1320,6 +1505,17 @@ class ConfigurationDialog(ConfigurationForm):
             self.mailUserEdit.text())
         Preferences.setUser("MailServerPassword",
             self.mailPasswordEdit.text())
+            
+        # write the VCS settings
+        Preferences.setVCS("AutoClose",
+            self.vcsAutoCloseCheckBox.isChecked())
+        Preferences.setVCS("AutoSaveFiles",
+            self.vcsAutoSaveCheckBox.isChecked())
+        Preferences.setVCS("AutoSaveProject",
+            self.vcsAutoSaveProjectCheckBox.isChecked())
+        
+        # write the tasks settings
+        Preferences.setTasks("TasksMarkers", self.tasksMarkerEdit.text())
         
         # write the lexer styles to disk
         for lexer in self.lexers.values():
