@@ -72,6 +72,14 @@ class PreferencesLexer(QextScintillaLexer):
             from QScintilla.LexerBash import LexerBash
             self.lex = LexerBash(self)
             self.lex.setAutoIndentStyle(0)
+        elif language == "Ruby":
+            from QScintilla.LexerRuby import LexerRuby
+            self.lex = LexerRuby(self)
+            self.lex.setAutoIndentStyle(0)
+        elif language == "Lua":
+            from QScintilla.LexerLua import LexerLua
+            self.lex = LexerLua(self)
+            self.lex.setAutoIndentStyle(0)
         else:
             raise RuntimeError, \
                 'Unsupported Lexer Language: %s' % language
