@@ -45,8 +45,15 @@ class Prefs:
         "AutomaticReset" : 0,
         "CustomPythonInterpreter" : 0,
         "PythonInterpreter" : "",
+        "RubyInterpreter" : "/usr/bin/ruby",
         "DebugClientType" : 1,      # this coresponds with the radio button id
-        "DebugClient" : ""
+        "DebugClient" : "",
+        "DebugEnvironmentReplace" : 0,
+        "DebugEnvironment" : "",
+        "PythonRedirect" : 1,
+        "RubyRedirect" : 1,
+        "ConsoleDbgEnabled" : 0,
+        "ConsoleDbgCommand" : "",
     }
     
     uiDefaults = {
@@ -66,16 +73,19 @@ class Prefs:
         "SingleApplicationMode" : 0,
         "CaptionShowsFilename" : 1,
         # the order in ViewProfiles is Project-Browser, File-Browser,
-        # Debug-Browser, Python-Shell, Log-Viewer
+        # Debug-Browser, Python-Shell, Log-Viewer, Task-Viewer
         "ViewProfiles" : {
-            "edit"  : [[1, 0, 0, 1, 1], "", 
+            "edit"  : [[1, 0, 0, 1, 1, 1], "", 
                 [[0,0,300,300], [620,0,300,300], [310,0,300,300],
-                 [310,350,400,100], [0,350,300,100]]],
-            "debug" : [[0, 0, 1, 1, 1], "",
+                 [310,350,400,100], [0,350,300,100], 
+                 [0,500,400,125]]],
+            "debug" : [[0, 0, 1, 1, 1, 1], "",
                 [[0,0,300,300], [620,0,300,300], [310,0,300,300],
-                 [310,350,400,100], [0,350,300,100]]],
+                 [310,350,400,100], [0,350,300,100], 
+                 [0,500,400,125]]],
         }
     }
+    viewProfilesLength = len(uiDefaults["ViewProfiles"]["edit"][2])
     
     iconsDefaults = {
         "Path" : QStringList(),
@@ -125,6 +135,12 @@ class Prefs:
         # Bash specifics
         "BashFoldComment" : 1,
         
+        # Ruby specifics
+        "RubyBadIndentation" : 1,
+        
+        # Lua specifics
+        "LuaFoldCompact" : 1,
+        
         "AutoIndentation" : 1,
         "BraceHighlighting" : 1,
         "CreateBackupFile" : 0,
@@ -248,6 +264,22 @@ class Prefs:
     except ImportError:
         pass
     
+    # the following languages are only supported for QScintilla > 1.4
+    try:
+        from qtext import QextScintillaLexerRuby
+        editorAPIDefaults["Ruby"] = QStringList()
+        editorLexerAssocDefaults["rb"] = "Ruby"
+        editorDefaultExtensions.append('rb')
+    except ImportError:
+        pass
+    try:
+        from qtext import QextScintillaLexerLua
+        editorAPIDefaults["Lua"] = QStringList()
+        editorLexerAssocDefaults["lua"] = "Lua"
+        editorDefaultExtensions.append('lua')
+    except ImportError:
+        pass
+    
     # defaults for the printer settings
     printerDefaults = {
         "PrinterName" : "",
@@ -272,9 +304,12 @@ class Prefs:
     
     # defaults for the help settings
     helpDefaults = {
-        "UseQtAssistant" : 0,
+        "HelpViewerType" : 1,      # this coresponds with the radio button id
         "Webbrowser" : "",
-        "Pdfviewer" : ""
+        "Pdfviewer" : "",
+        "CustomViewer" : "",
+        "PythonDocDir" : "",
+        "QtDocDir" : "",
     }
     
     # defaults for system settings
@@ -318,6 +353,19 @@ class Prefs:
         "MailServerPassword" : "",
     }
     
+    # defaults for vcs related stuff
+    vcsDefaults = {
+        "AutoClose" : 0,
+        "AutoSaveFiles" : 1,
+        "AutoSaveProject" : 1,
+    }
+    
+    #defaults for tasks related stuff
+    tasksDefaults = {
+        "TasksMarkers" : "FIX" + "ME: TO" + "DO:",  
+        # needed to keep it from being recognized as a task
+    }
+    
     # defaults for geometry
     geometryDefaults = {
         "MainGeometry"   : [0,0,0,0],
@@ -327,7 +375,8 @@ class Prefs:
         "SBV"            : [310,0,300,300,1],
         "LogViewer"      : [0,350,300,100,1],
         "Shell"          : [310,350,400,100,1],
-        "FileBrowser"    : [620,0,300,300,1]
+        "FileBrowser"    : [620,0,300,300,1],
+        "TaskViewer"     : [0,500,400,125],
     }
 
     # defaults for dock windows
@@ -360,13 +409,15 @@ def readPreferences(prefClass = Prefs):
     prefClass.debuggerPrefs = {}
     for key in "RemoteDbgEnabled", "PassiveDbgEnabled", \
                 "PassiveDbgPort", "CustomPythonInterpreter", \
-                "DebugClientType", "AutomaticReset":
+                "DebugClientType", "AutomaticReset", "DebugEnvironmentReplace", \
+                "PythonRedirect", "RubyRedirect", "ConsoleDbgEnabled":
         prefClass.debuggerPrefs[key], ok = \
             prefClass.settings.readNumEntry("/eric3/Debugger/" + key,
                 prefClass.debuggerDefaults[key])
                 
     for key in "RemoteHost", "RemoteExecution", "PythonInterpreter", \
-                "DebugClient":
+                "RubyInterpreter", "DebugClient", "DebugEnvironment", \
+                "ConsoleDbgCommand":
         prefClass.debuggerPrefs[key], ok = \
             prefClass.settings.readEntry("/eric3/Debugger/" + key,
                 prefClass.debuggerDefaults[key])
@@ -399,6 +450,13 @@ def readPreferences(prefClass = Prefs):
     v, ok = prefClass.settings.readEntry("/eric3/UI/ViewProfiles")
     if ok:
         prefClass.uiPrefs["ViewProfiles"] = eval(unicode(v))
+        for name in ["edit", "debug"]:
+            vpLength = len(prefClass.uiPrefs["ViewProfiles"][name][2])
+            if vpLength < prefClass.viewProfilesLength:
+                prefClass.uiPrefs["ViewProfiles"][name][0].extend(\
+                    prefClass.uiDefaults["ViewProfiles"][name][0][vpLength:])
+                prefClass.uiPrefs["ViewProfiles"][name][2].extend(\
+                    prefClass.uiDefaults["ViewProfiles"][name][2][vpLength:])
     else:
         prefClass.uiPrefs["ViewProfiles"] = prefClass.uiDefaults["ViewProfiles"]
     
@@ -470,11 +528,12 @@ def readPreferences(prefClass = Prefs):
     
     # read the entries for the help settings
     prefClass.helpPrefs = {}
-    for key in ["UseQtAssistant"]:
+    for key in ["HelpViewerType"]:
         prefClass.helpPrefs[key], ok = \
             prefClass.settings.readNumEntry("/eric3/Help/" + key,
                 prefClass.helpDefaults[key])
-    for key in ["Webbrowser", "Pdfviewer"]:
+    for key in ["Webbrowser", "Pdfviewer", "CustomViewer", \
+                "PythonDocDir", "QtDocDir"]:
         prefClass.helpPrefs[key], ok = \
             prefClass.settings.readEntry("/eric3/Help/" + key,
                 prefClass.helpDefaults[key])
@@ -556,6 +615,20 @@ def readPreferences(prefClass = Prefs):
             prefClass.userPrefs[key], ok = \
                 prefClass.settings.readEntry("/eric3/User/" + key,
                     prefClass.userDefaults[key])
+                    
+    # read the entries for the VCS settings
+    prefClass.vcsPrefs = {}
+    for key in prefClass.vcsDefaults.keys():
+        prefClass.vcsPrefs[key], ok = \
+            prefClass.settings.readNumEntry("/eric3/VCS/" + key,
+                prefClass.vcsDefaults[key])
+    
+    # read the entries for the tasks settings
+    prefClass.tasksPrefs = {}
+    for key in prefClass.tasksDefaults.keys():
+        prefClass.tasksPrefs[key], ok = \
+            prefClass.settings.readEntry("/eric3/Tasks/" + key,
+                prefClass.tasksDefaults[key])
     
 def savePreferences(prefClass = Prefs):
     """
@@ -682,7 +755,17 @@ def savePreferences(prefClass = Prefs):
     for key in prefClass.userPrefs.keys():
         prefClass.settings.writeEntry("/eric3/User/" + key,
                 prefClass.userPrefs[key])
-        
+                
+    # write the entries for the VCS settings
+    for key in prefClass.vcsPrefs.keys():
+        prefClass.settings.writeEntry("/eric3/VCS/" + key,
+                prefClass.vcsPrefs[key])
+                
+    # write the entries for the tasks settings
+    for key in prefClass.tasksPrefs.keys():
+        prefClass.settings.writeEntry("/eric3/Tasks/" + key,
+                prefClass.tasksPrefs[key])
+    
 def readToolbarSettings(mw, tbs, prefClass = Prefs):
     """
     Module function to read the toolbar settings from the central store.
@@ -1267,6 +1350,46 @@ def setUser(key, value, prefClass = Prefs):
     """
     prefClass.userPrefs[key] = value
     
+def getVCS(key, prefClass = Prefs):
+    """
+    Module function to retrieve the VCS related settings.
+    
+    @param key the key of the value to get
+    @param prefClass preferences class used as the storage area
+    @return the requested user setting
+    """
+    return prefClass.vcsPrefs[key]
+    
+def setVCS(key, value, prefClass = Prefs):
+    """
+    Module function to store the VCS related settings.
+    
+    @param key the key of the setting to be set
+    @param value the value to be set
+    @param prefClass preferences class used as the storage area
+    """
+    prefClass.vcsPrefs[key] = value
+    
+def getTasks(key, prefClass = Prefs):
+    """
+    Module function to retrieve the Tasks related settings.
+    
+    @param key the key of the value to get
+    @param prefClass preferences class used as the storage area
+    @return the requested user setting
+    """
+    return prefClass.tasksPrefs[key]
+    
+def setTasks(key, value, prefClass = Prefs):
+    """
+    Module function to store the Tasks related settings.
+    
+    @param key the key of the setting to be set
+    @param value the value to be set
+    @param prefClass preferences class used as the storage area
+    """
+    prefClass.tasksPrefs[key] = value
+    
 def getGeometry(key, prefClass = Prefs):
     """
     Module function to retrieve the display geometry.
