@@ -34,9 +34,15 @@ class AddDirectoryDialog(AddDirForm):
         self.targetDirEdit.setText(pro.ppath)
         self.ppath = pro.ppath
         self.handleFileTypeSelected(QString('(*.py)')) # enable all dialog elements
-        if filter == 'py':
-            filterList = QStringList.split(';;', self.trUtf8(\
-                            "Python Files (*.py);;Quixote PTL Files (*.ptl)"))
+        if filter == 'py':  # it is a source file
+            if pro.pdata["MIXEDLANGUAGE"][0]:
+                sourceExtensions = pro.sourceExtensions["Mixed"]
+                sourceFilters = pro.sourceFilters["Mixed"]
+            else:
+                sourceExtensions = pro.sourceExtensions[pro.pdata["PROGLANGUAGE"][0]]
+                sourceFilters = pro.sourceFilters[pro.pdata["PROGLANGUAGE"][0]]
+            filterList = QStringList.split(';;', QString(sourceFilters))
+            filter = sourceExtensions[0][1:]
         elif filter == 'ui':
             filterList = QStringList.split(';;', self.trUtf8(\
                             "Qt User-Interface Files (*.ui);;"))
@@ -48,10 +54,14 @@ class AddDirectoryDialog(AddDirForm):
                             "Other Files (*);;"))
             self.handleFileTypeSelected(QString('(*)'))
         else:
-            filterList = QStringList.split(';;', self.trUtf8(\
-                    "Python Files (*.py);;Quixote PTL Files (*.ptl);;"
+            if pro.pdata["MIXEDLANGUAGE"][0]:
+                filters = pro.sourceFilters["Mixed"]
+            else:
+                filters = pro.sourceFilters[pro.pdata["PROGLANGUAGE"][0]]
+            filters.append(self.trUtf8(\
                     "Qt User-Interface Files (*.ui);;Interface Files (*.idl);;"
                     "Other Files (*)"))
+            filterList = QStringList.split(';;', filters)
         self.filterComboBox.insertStringList(filterList)
         fi = '(*.%s)' % str(filter)
         index = 0
