@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002, 2003 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2005 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -11,6 +11,8 @@ import os
 
 from qt import *
 
+from KdeQt import KQFileDialog
+
 from AddFileForm import AddFileForm
 
 class AddFileDialog(AddFileForm):
@@ -33,12 +35,16 @@ class AddFileDialog(AddFileForm):
         self.targetDirEdit.setText(pro.ppath)
         self.filter = filter
         self.ppath = pro.ppath
+        if pro.pdata["MIXEDLANGUAGE"][0]:
+            self.sourceFilter = pro.sourceFilters["Mixed"]
+        else:
+            self.sourceFilter = pro.sourceFilters[pro.pdata["PROGLANGUAGE"][0]]
         
     def handleDirDialog(self):
         """
         Private slot to display a directory selection dialog.
         """
-        directory = QFileDialog.getExistingDirectory(self.targetDirEdit.text(),
+        directory = KQFileDialog.getExistingDirectory(self.targetDirEdit.text(),
             self, None, self.trUtf8("Select target directory"), 1)
             
         if not directory.isNull():
@@ -47,24 +53,34 @@ class AddFileDialog(AddFileForm):
     def handleFileDialog(self):
         """
         Private slot to display a file selection dialog.
+        
+        A filter of 'py' is synonym for source files.
         """
         dir = self.sourceFileEdit.text()
         if dir.isEmpty():
             dir = self.targetDirEdit.text()
         if self.filter is None:
-            dfilter = self.trUtf8(\
-            "Python Files (*.py);;Qt User-Interface Files (*.ui);;"
-            "Interface Files (*.idl);;Quixote PTL Files (*.ptl)")
+            dfilter = QString(self.sourceFilter)
+            dfilter.append(self.trUtf8(\
+                "Qt User-Interface Files (*.ui);;"
+                "Interface Files (*.idl);;"
+                "All Files (*)"
+            ))
         elif self.filter == 'ui':
             dfilter = self.trUtf8("Qt User-Interface Files (*.ui)")
         elif self.filter == 'py':
-            dfilter = self.trUtf8("Python Files (*.py);;"
-            "Quixote PTL Files (*.ptl);;All Files (*)")
+            dfilter = QString(self.sourceFilter)
+            dfilter.append(self.trUtf8(\
+                "All Files (*)"
+            ))
         elif self.filter == 'idl':
             dfilter = self.trUtf8("Interface Files (*.idl)")
+        elif self.filter == 'others':
+            dfilter = self.trUtf8("All Files (*)")
         else:
             return
-        fn = QFileDialog.getOpenFileName(dir,
+            
+        fn = KQFileDialog.getOpenFileName(dir,
             dfilter,
             self, None, self.trUtf8("Select source file"))
             
@@ -83,15 +99,16 @@ class AddFileDialog(AddFileForm):
         @param sfile the text of the source file line edit
         """
         if sfile.startsWith(self.ppath):
-            dir = os.path.dirname(str(sfile))
+            dir = os.path.dirname(unicode(sfile))
             self.targetDirEdit.setText(dir)
         
     def getData(self):
         """
         Public slot to retrieve the dialogs data.
         
-        @return tuple of two values (string string) giving the source file and
-            the target directory
+        @return tuple of three values (string, string, boolean) giving the 
+            source file, the target directory and a flag telling, whether
+            the file shall be added as source code
         """
-        return (str(self.sourceFileEdit.text()), 
-            str(self.targetDirEdit.text()))
+        return (unicode(self.sourceFileEdit.text()), 
+            unicode(self.targetDirEdit.text()), self.sourcecodeCheckBox.isChecked())
