@@ -308,6 +308,7 @@ class SearchReplaceDialog(SearchReplaceForm):
         self.replacetextCombo.insertStringList(self.replaceHistory)
         
         aw = self.viewmanager.activeWindow()
+        cline, cindex = aw.getCursorPosition()
         if self.wrapCheckBox.isChecked():
             if self.limitCheckBox.isChecked():
                 line = self.startLineSpinBox.value() - 1
@@ -343,9 +344,11 @@ class SearchReplaceDialog(SearchReplaceForm):
         
         wordWrap = self.wrapCheckBox.isChecked()
         self.wrapCheckBox.setChecked(0)
+        aw.beginUndoAction()
         while ok:
             aw.replace(rtxt)
             ok = self.findNext(ftxt)
+        aw.endUndoAction()
         if wordWrap:
             self.wrapCheckBox.setChecked(1)
         
