@@ -18,12 +18,12 @@ from KdeQt import KQMessageBox, KQInputDialog
 from QextScintillaCompat import QextScintillaCompat
 from Editor import Editor
 
-from LexerPython import LexerPython
-
 import Preferences
 import UI.PixmapCache
 from Utilities import toUnicode
 
+from Debugger.DebugClientCapabilities import HasShell, HasCompleter
+
 class Shell(QextScintillaCompat):
     """
     Module implementing a graphical Python shell.
@@ -54,15 +54,17 @@ class Shell(QextScintillaCompat):
 
         QWhatsThis.add(self,self.trUtf8(
 """<b>The Shell Window</b>"""
-"""<p>This is simply the Python interpreter running in a window. The"""
+"""<p>This is simply an interpreter running in a window. The"""
 """ interpreter is the one that is used to run the program being debugged."""
-""" This means that you can execute any Python command while the program"""
+""" This means that you can execute any command while the program"""
 """ being debugged is running.</p>"""
 """<p>You can use the cursor keys while entering commands. There is also a"""
 """ history of commands that can be recalled using the up and down cursor"""
 """ keys.</p>"""
-"""<p>The shell has two special commands. 'reset' kills the Python shell"""
+"""<p>The shell has some special commands. 'reset' kills the shell"""
 """ and starts a new one. 'clear' clears the display of the shell window."""
+""" 'start' is used to switch the shell language and must be followed by"""
+""" either 'Python' or 'Ruby'."""
 """ These commands are available through the context menu as well.</p>"""
 """<p>Pressing the Tab key after some text has been entered will show"""
 """ a list of possible commandline completions. The relevant entry may"""
@@ -80,6 +82,7 @@ class Shell(QextScintillaCompat):
         self.connect(dbs,PYSIGNAL('clientRawInput'),self.raw_input)
         self.connect(dbs,PYSIGNAL('clientBanner'),self.writeBanner)
         self.connect(dbs,PYSIGNAL('clientCompletionList'),self.showCompletions)
+        self.connect(dbs,PYSIGNAL('clientCapabilities'),self.handleClientCapabilities)
         self.dbs = dbs
 
         # Initialise instance variables.
@@ -89,7 +92,10 @@ class Shell(QextScintillaCompat):
         self.history = []
         self.histidx = -1
         self.inDragDrop = 0
+        self.lexer = None
 
+        self.clientType = ''
+        
         # Make sure we have prompts.
         if self.passive:
             sys.ps1 = unicode(self.trUtf8("Passive >>> "))
@@ -102,9 +108,13 @@ class Shell(QextScintillaCompat):
             sys.ps2
         except AttributeError:
             sys.ps2 = "... "
+        if self.passive:
+            self.getBanner()
 
-        # Display the banner.
-        self.getBanner()
+        # Create a little language context menu
+        self.lmenu = QPopupMenu()
+        self.lmenu.insertItem(self.trUtf8('Python'), self.handleStartPython)
+        self.lmenu.insertItem(self.trUtf8('Ruby'), self.handleStartRuby)
         
         # Create a little context menu
         self.menu = QPopupMenu(self)
@@ -115,6 +125,8 @@ class Shell(QextScintillaCompat):
         self.menu.insertItem(self.trUtf8('Reset'), self.handleReset)
         self.menu.insertItem(self.trUtf8('Reset and Clear'),
             self.handleResetAndClear)
+        self.menu.insertSeparator()
+        self.menu.insertItem(self.trUtf8('Start'), self.lmenu)
         
         self.setMargin0()
         self.setTextDisplay()
@@ -126,11 +138,23 @@ class Shell(QextScintillaCompat):
         
         self.setIcon(UI.PixmapCache.getPixmap("eric.png"))
         
-    def bindLexer(self):
+    def bindLexer(self, language = 'Python'):
         """
         Private slot to set the Python lexer.
         """
-        self.lexer = LexerPython(self)
+        if language == 'Python':
+            from LexerPython import LexerPython
+            self.lexer = LexerPython(self)
+        elif language == 'Ruby':
+            try:
+                from LexerRuby import LexerRuby
+                self.lexer = LexerRuby(self)
+            except ImportError:
+                self.lexer = None
+        
+        if self.lexer is None:
+            self.setLexer(None)
+            return
         
         # get the font for style 0 and set it as the default font
         key = '/eric3/Scintilla/%s/style0/font' % unicode(self.lexer.language())
@@ -143,10 +167,23 @@ class Shell(QextScintillaCompat):
         self.lexer.readSettings(Preferences.Prefs.settings, "/eric3/Scintilla")
         
         # now set the lexer properties
-        if Preferences.getEditor("PythonBadIndentation"):
-            self.lexer.setIndentationWarning(QextScintillaLexerPython.Inconsistent)
-        else:
-            self.lexer.setIndentationWarning(QextScintillaLexerPython.NoWarning)
+        if language == "Python":
+            from qtext import QextScintillaLexerPython
+            # Python lexer stuff
+            if Preferences.getEditor("PythonBadIndentation"):
+                self.lexer.setIndentationWarning(QextScintillaLexerPython.Inconsistent)
+            else:
+                self.lexer.setIndentationWarning(QextScintillaLexerPython.NoWarning)
+        elif language == "Ruby":
+            # Ruby lexer stuff
+            from qtext import QextScintillaLexerRuby
+            try:
+                if Preferences.getEditor("RubyBadIndentation"):
+                    self.lexer.setIndentationWarning(QextScintillaLexerRuby.Inconsistent)
+                else:
+                    self.lexer.setIndentationWarning(QextScintillaLexerRuby.NoWarning)
+            except AttributeError:
+                pass
         
     def setMargin0(self):
         """
@@ -223,7 +260,7 @@ class Shell(QextScintillaCompat):
             self.setAutoCompletionSource(QextScintilla.AcsDocument)
             self.acAPI = 0
         else:
-            if Preferences.getEditor("AutoCompletionSource") == QextScintilla.AcsDocument:
+            if Preferences.getEditor("AutoCompletionSource"):
                 self.setAutoCompletionSource(QextScintilla.AcsDocument)
             else:
                 self.setAutoCompletionSource(QextScintilla.AcsAPIs)
@@ -258,7 +295,22 @@ class Shell(QextScintillaCompat):
         self.inContinue = 0
         self.inRawMode = 0
         self.echoInput = 1
+        self.clientCapabilities = 0
 
+    def handleClientCapabilities(self, cap, clType):
+        """
+        Private slot to handle the reporting of the clients capabilities.
+        
+        @param cap client capabilities (integer)
+        @param clType type of the debug client (string)
+        """
+        self.clientCapabilities = cap
+        if clType != self.clientType:
+            self.clientType = clType
+            self.bindLexer(clType)
+            self.racEnabled = Preferences.getShell("RemoteAutoCompletionEnabled") and \
+                              (cap & HasCompleter)
+        
     def getBanner(self):
         """
         Private method to get the banner for the remote interpreter.
@@ -277,16 +329,18 @@ class Shell(QextScintillaCompat):
         
         @param version Python version string (string)
         @param platform platform of the remote interpreter (string)
-        @param dbgclient debug client type used (string)
+        @param dbgclient debug client variant used (string)
         """
+        self.clear()
         if self.passive:
             self.write('Passive Debug Mode')
         else:
             version = version.replace("#", unicode(self.trUtf8("No.")))
-            self.write(self.trUtf8('Python %1 on %2, %3\n')
+            self.write(self.trUtf8('%1 on %2, %3')
                         .arg(version)
                         .arg(platform)
                         .arg(dbgclient))
+        self.write('\n')
 
         self.write(sys.ps1)
         
@@ -464,21 +518,24 @@ class Shell(QextScintillaCompat):
                         self.history.append(QString(buf))
                         self.histidx = -1
                         cmd = buf
-##                        cmd = str(buf)
-                    if cmd == 'reset':
-                        self.dbs.startClient(0)
-                        self.clear()
+                    if cmd.startswith('start'):
+                        if not self.passive:
+                            cmdList = cmd.split(None, 1)
+                            if len(cmdList) < 2:
+                                self.dbs.startClient(0) # same as reset
+                            else:
+                                self.dbs.startClient(0, cmdList[1].capitalize())
+                            cmd = ''
+                    elif cmd == 'clear':
                         # Display the banner.
                         self.getBanner()
                         if not self.passive:
                             return
                         else:
                             cmd = ''
-                    elif cmd == 'clear':
-                        self.clear()
-                        # Display the banner.
-                        self.getBanner()
-                        if not self.passive:
+                    elif cmd == 'reset':
+                        self.dbs.startClient(0)
+                        if self.passive:
                             return
                         else:
                             cmd = ''
@@ -499,9 +556,9 @@ class Shell(QextScintillaCompat):
             if self.SendScintilla(QextScintilla.SCI_AUTOCACTIVE):
                 QextScintillaCompat.keyPressEvent(self, ev)
             else:
-                line = self.getCursorPosition()[0]
+                line, index = self.getCursorPosition()
                 buf = unicode(self.text(line)).replace(sys.ps1, "").replace(sys.ps2, "")
-                if self.inContinue and not buf.strip():
+                if self.inContinue and not buf[:index-len(sys.ps2)].strip():
                     QextScintillaCompat.keyPressEvent(self, ev)
                 elif self.racEnabled:
                     self.dbs.remoteCompletion(buf)
@@ -591,7 +648,6 @@ class Shell(QextScintillaCompat):
         """
         Private slot to handle the 'clear' context menu entry.
         """
-        self.clear()
         # Display the banner.
         self.getBanner()
         
@@ -608,16 +664,30 @@ class Shell(QextScintillaCompat):
         """
         self.dbs.startClient(0)
         
+    def handleStartPython(self):
+        """
+        Private slot to handle the 'Start Python' context menu entry.
+        """
+        self.dbs.startClient(0, 'Python')
+        
+    def handleStartRuby(self):
+        """
+        Private slot to handle the 'Start Ruby' context menu entry.
+        """
+        self.dbs.startClient(0, 'Ruby')
+        
     def handlePreferencesChanged(self):
         """
         Public slot to handle the preferencesChanged signal.
         """
-        # read the lexer settings
-        self.lexer.readSettings(Preferences.Prefs.settings, "/eric3/Scintilla")
-        if Preferences.getEditor("PythonAutoIndent"):
-            self.lexer.setAutoIndentStyle(0)
-        else:
-            self.lexer.setAutoIndentStyle(QextScintilla.AiMaintain)
+        if self.lexer is not None:
+            # read the lexer settings
+            self.lexer.readSettings(Preferences.Prefs.settings, "/eric3/Scintilla")
+            if self.lexer.language() == 'Python':
+                if Preferences.getEditor("PythonAutoIndent"):
+                    self.lexer.setAutoIndentStyle(0)
+                else:
+                    self.lexer.setAutoIndentStyle(QextScintilla.AiMaintain)
 
         # set margin 0 configuration
         self.setMargin0()
