@@ -375,9 +375,11 @@ try:
         """
         Class implementing an 'About Eric' dialog using the KDE About Dialog.
         """
-        def __init__(self):
+        def __init__(self, parent = None):
             """
             Constructor
+            
+            @param parent reference to the parent widget (QWidget)
             """
             KAboutDialog.__init__(self, KAboutDialog.AbtKDEStandard,
                 qApp.translate("AboutDialog", "About Eric"),
@@ -399,11 +401,13 @@ except ImportError:
         """
         Class implementing an 'About Eric' dialog.
         """
-        def __init__(self):
+        def __init__(self, parent = None):
             """
             Constructor
+            
+            @param parent reference to the parent widget (QWidget)
             """
-            AboutForm.__init__(self, None, None, 1, Qt.WDestructiveClose)
+            AboutForm.__init__(self, parent, None, 1, Qt.WDestructiveClose)
             
             self.ericLabel.setText(titleText)
             
