@@ -11,11 +11,15 @@ The class browsing part is based on IDLE's classbrowser.
 
 import sys
 import string
-import Utilities.pyclbr as pyclbr
 import os
 
 from qt import *
 
+from KdeQt import KQFileDialog
+
+import Utilities.ClassBrowsers
+import Utilities.ClassBrowsers.ClbrBaseClasses
+
 import UI.PixmapCache
 import Preferences
 
@@ -136,6 +140,16 @@ class BrowserDirectory(BrowserNode):
         """
         return self.dirName
 
+    def compare(self, item, col, ascending):
+        """
+        Private method to compare two items.
+        """
+        if issubclass(item.__class__, BrowserFile):
+            if Preferences.getUI("BrowsersListFoldersFirst"):
+                return ascending and -1 or 1
+        
+        return QListViewItem.compare(self, item, col, ascending)
+
 
 class BrowserSysPath(BrowserNode):
     """
@@ -207,7 +221,10 @@ class BrowserFile(BrowserNode):
         self.children = []
         self.fn = os.path.abspath(finfo)
         
-        self.pyFile = isPyFile
+        if not self.isRubyFile() and isPyFile:
+            self.pyFile = 1
+        else:
+            self.pyFile = 0
         
         if self.isPythonFile():
             if self.fileext == '.py':
@@ -217,6 +234,11 @@ class BrowserFile(BrowserNode):
             self.setExpandable(1)
             self.moduleName = os.path.basename(finfo)
             self.dirName = os.path.dirname(finfo)
+        elif self.isRubyFile():
+            self.setPixmap(0,UI.PixmapCache.getPixmap("fileRuby.png"))
+            self.setExpandable(1)
+            self.moduleName = os.path.basename(finfo)
+            self.dirName = os.path.dirname(finfo)
         elif self.isDesignerFile() or self.isDesignerHeaderFile():
             self.setPixmap(0,UI.PixmapCache.getPixmap("fileDesigner.png"))
         elif self.isLinguistFile():
@@ -225,6 +247,9 @@ class BrowserFile(BrowserNode):
             self.setPixmap(0,UI.PixmapCache.getPixmap("fileProject.png"))
         elif self.isIdlFile():
             self.setPixmap(0,UI.PixmapCache.getPixmap("fileIDL.png"))
+            self.setExpandable(1)
+            self.moduleName = os.path.basename(finfo)
+            self.dirName = os.path.dirname(finfo)
         elif self.isPixmapFile():
             self.setPixmap(0,UI.PixmapCache.getPixmap("filePixmap.png"))
         else:
@@ -246,6 +271,14 @@ class BrowserFile(BrowserNode):
         """
         return self.fileext in ['.py', '.ptl'] or self.pyFile
         
+    def isRubyFile(self):
+        """
+        Public method to check, if this file is a Ruby script.
+        
+        @return flag indicating a Ruby file (boolean)
+        """
+        return self.fileext == '.rb'
+        
     def isDesignerFile(self):
         """
         Public method to check, if this file is a Qt-Designer file.
@@ -302,22 +335,17 @@ class BrowserFile(BrowserNode):
         
         @param o flag indicating the open state
         """
-        if not self.isPythonFile():
+        if not self.isPythonFile() and \
+           not self.isRubyFile() and \
+           not self.isIdlFile():
             return
             
         if o:
             try:
-                dict = pyclbr.readmodule_ex(self.moduleName, [self.dirName])
-                pyclbr._modules.clear()
+                dict = Utilities.ClassBrowsers.readmodule(self.moduleName, [self.dirName])
             except ImportError:
                 return
                 
-            if dict.has_key('__all__'):
-                publics = dict['__all__'].identifiers[:]
-                del dict['__all__']
-            else:
-                publics = []
-                
             hideNonPublic = Preferences.getUI("BrowsersHideNonPublic")
             
             items = []
@@ -329,15 +357,9 @@ class BrowserFile(BrowserNode):
                 cl = dict[key]
                 try:
                     if cl.module == self.moduleName:
-                        if publics:
-                            isPublic = cl.name in publics and 2 or 0
-                        else:
-                            isPublic = None
-                        if hideNonPublic and \
-                            ((isPublic is None and cl.name.startswith('_')) or \
-                             (isPublic is not None and isPublic != 2)):
+                        if hideNonPublic and  not cl.isPublic():
                             continue
-                        node = BrowserClass(self, cl, last, self.fn, isPublic)
+                        node = BrowserClass(self, cl, last, self.fn)
                         self.children.append(node)
                         last = node
                 except AttributeError:
@@ -371,7 +393,7 @@ class BrowserClass(BrowserNode):
     """
     Class implementing a BrowserNode that represents a python class or function. 
     """
-    def __init__(self, parent, cl, after, filename, isPublic=None):
+    def __init__(self, parent, cl, after, filename):
         """
         Constructor
         
@@ -379,12 +401,6 @@ class BrowserClass(BrowserNode):
         @param cl Class object to be shown
         @param after sibling this node is positioned after
         @param filename filename of the file defining this class
-        @param isPublic flag indicating visibility if not None. It is one of
-            <ul>
-            <li>0 - private</li>
-            <li>1 - protected</li>
-            <li>2 - public</li>
-            </ul>
         """
         name = cl.name
         if hasattr(cl, 'super') and cl.super:
@@ -406,46 +422,36 @@ class BrowserClass(BrowserNode):
         self.file = filename
         self.children = []
         
-        self.isfunction = isinstance(self.cl, pyclbr.Function)
-        
+        self.isfunction = isinstance(self.cl, 
+                                     Utilities.ClassBrowsers.ClbrBaseClasses.Function)
+        self.ismodule = isinstance(self.cl, 
+                                   Utilities.ClassBrowsers.ClbrBaseClasses.Module)
         if self.isfunction:
-            if isPublic is not None:
-                if isPublic == 0:
-                    pix = UI.PixmapCache.getPixmap("method_private.png")
-                elif isPublic == 1:
-                    pix = UI.PixmapCache.getPixmap("method_protected.png")
-                else:
-                    pix = UI.PixmapCache.getPixmap("method.png")
+            if cl.isPrivate():
+                pix = UI.PixmapCache.getPixmap("method_private.png")
+            elif cl.isProtected():
+                pix = UI.PixmapCache.getPixmap("method_protected.png")
             else:
-                if name.startswith('__'):
-                    pix = UI.PixmapCache.getPixmap("method_private.png")
-                elif name.startswith('_'):
-                    pix = UI.PixmapCache.getPixmap("method_protected.png")
-                else:
-                    pix = UI.PixmapCache.getPixmap("method.png")
+                pix = UI.PixmapCache.getPixmap("method.png")
             self.setPixmap(0, pix)
             self.setText(0, "%s(%s)" % (name, ", ".join(self.cl.parameters)))
             # if no defaults are wanted
             # ... % (name, ", ".join([e.split('=')[0].strip() for e in self.cl.parameters]))
+        elif self.ismodule:
+            pix = UI.PixmapCache.getPixmap("module.png")
+            self.setPixmap(0, pix)
+            self.setText(0, name)
         else:
-            if isPublic is not None:
-                if isPublic == 0:
-                    pix = UI.PixmapCache.getPixmap("class_private.png")
-                elif isPublic == 1:
-                    pix = UI.PixmapCache.getPixmap("class_protected.png")
-                else:
-                    pix = UI.PixmapCache.getPixmap("class.png")
+            if cl.isPrivate():
+                pix = UI.PixmapCache.getPixmap("class_private.png")
+            elif cl.isProtected():
+                pix = UI.PixmapCache.getPixmap("class_protected.png")
             else:
-                if name.startswith('__'):
-                    pix = UI.PixmapCache.getPixmap("class_private.png")
-                elif name.startswith('_'):
-                    pix = UI.PixmapCache.getPixmap("class_protected.png")
-                else:
-                    pix = UI.PixmapCache.getPixmap("class.png")
+                pix = UI.PixmapCache.getPixmap("class.png")
             self.setPixmap(0, pix)
             self.setText(0, name)
         if self.cl and \
-           (self.cl.methods or self.cl.classes):
+           (self.cl.methods or self.cl.classes or self.cl.attributes):
             self.setExpandable(1)
                 
     def setOpen(self, o):
@@ -506,8 +512,7 @@ class BrowserClassAttributes(BrowserNode):
         """
         BrowserNode.__init__(self, parent, qApp.translate("BrowserClassAttributes", "Attributes"), after)
         
-        self.attributes = attributes[:]
-        self.attributes.sort()
+        self.attributes = attributes.copy()
         self.children = []
         
         self.setExpandable(1)
@@ -525,17 +530,20 @@ class BrowserClassAttributes(BrowserNode):
             hideNonPublic = Preferences.getUI("BrowsersHideNonPublic")
             
             last = None
-            for attrib in self.attributes:
-                if hideNonPublic and attrib.startswith('_'):
+            keys = self.attributes.keys()
+            keys.sort()
+            for key in keys:
+                attrib = self.attributes[key]
+                if hideNonPublic and not attrib.isPublic():
                     continue
-                if attrib.startswith('__'):
+                if attrib.isPrivate():
                     pix = UI.PixmapCache.getPixmap("attribute_private.png")
-                elif attrib.startswith('_'):
+                elif attrib.isProtected():
                     pix = UI.PixmapCache.getPixmap("attribute_protected.png")
                 else:
                     pix = UI.PixmapCache.getPixmap("attribute.png")
                     
-                node = BrowserNode(self, attrib, last)
+                node = BrowserNode(self, attrib.name, last)
                 node.setPixmap(0, pix)
                 self.children.append(node)
                 last = node
@@ -569,9 +577,9 @@ class BrowserMethod(BrowserNode):
         self.fn = fn
         self.file = filename
         self.children = []
-        if name.startswith('__'):
+        if fn.isPrivate():
             pix = UI.PixmapCache.getPixmap("method_private.png")
-        elif name.startswith('_'):
+        elif fn.isProtected():
             pix = UI.PixmapCache.getPixmap("method_protected.png")
         else:
             pix = UI.PixmapCache.getPixmap("method.png")
@@ -662,29 +670,32 @@ class Browser(QListView):
         self.setIcon(UI.PixmapCache.getPixmap("eric.png"))
         
         self.setRootIsDecorated(1)
-        self.setSorting(-1)
+        self.setSorting(0)
+        self.setShowSortIndicator(1)
         self.addColumn(self.trUtf8('Name'))
         self.setSelectionMode(QListView.Extended)
         self.selectedItemsFilter = [BrowserFile]
 
         self.progDir = None
-        self.children = []
-        last = None
+        self.last = None
         
         self.hideNonPublic = Preferences.getUI("BrowsersHideNonPublic")
 
-        node = BrowserSysPath(self,last)
-        last = node
-        self.children.append(node)
+        node = BrowserSysPath(self,self.last)
+        self.last = node
             
-        node = BrowserDirectory(self,unicode(QDir.convertSeparators(QDir.homeDirPath())),last)
-        last = node
-        self.children.append(node)
-
-        for d in QDir.drives():
-            node = BrowserDirectory(self,unicode(QDir.convertSeparators(d.absFilePath())),last)
-            last = node
-            self.children.append(node)
+        self.toplevelDirs = QStringList()
+        tdp, ok = Preferences.Prefs.settings.readListEntry('/eric3/Browser/ToplevelDirs')
+        if ok:
+            self.toplevelDirs = tdp
+        else:
+            self.toplevelDirs.append(QDir.convertSeparators(QDir.homeDirPath()))
+            for d in QDir.drives():
+                self.toplevelDirs.append(QDir.convertSeparators(d.absFilePath()))
+        
+        for d in self.toplevelDirs:
+            node = BrowserDirectory(self,unicode(d),self.last)
+            self.last = node
 
         self.connect(self,SIGNAL('contextMenuRequested(QListViewItem *, const QPoint &, int)'),
                      self.handleContextMenu)
@@ -744,8 +755,21 @@ class Browser(QListView):
         self.multiMenu = QPopupMenu(self)
         self.multiMenu.insertItem(qApp.translate('Browser','Open'), self.handleOpen)
         
+        # create the directory menu
+        self.dirMenu = QPopupMenu(self)
+        self.dirMenu.insertItem(qApp.translate('Browser','New toplevel directory...'), 
+            self.handleNewToplevelDir)
+        self.addAsTopLevelItem = self.dirMenu.insertItem(\
+            qApp.translate('Browser','Add as toplevel directory'),
+            self.handleAddAsToplevelDir)
+        self.removeFromToplevelItem = self.dirMenu.insertItem(\
+            qApp.translate('Browser','Remove from toplevel'),
+            self.handleRemoveToplevel)
+            
         # create the background menu
-        self.backMenu = None
+        self.backMenu = QPopupMenu(self)
+        self.backMenu.insertItem(qApp.translate('Browser','New toplevel directory...'), 
+            self.handleNewToplevelDir)
 
     def contentsMouseDoubleClickEvent(self, mouseEvent):
         """
@@ -767,7 +791,8 @@ class Browser(QListView):
         @param col the column of the mouse pointer (int)
         """
         try:
-            cnt = self.getSelectedItemsCount([BrowserFile, BrowserClass, BrowserMethod])
+            cnt = self.getSelectedItemsCount([BrowserDirectory, BrowserFile, 
+                                              BrowserClass, BrowserMethod])
             bfcnt = self.getSelectedItemsCount()
             if cnt > 1 and cnt == bfcnt:
                 self.multiMenu.popup(coord)
@@ -779,17 +804,19 @@ class Browser(QListView):
                         else:
                             self.pyMenu.setItemEnabled(self.unittestItem, 0)
                         self.pyMenu.popup(coord)
-                    elif itm.isDesignerFile() or \
-                            itm.isDesignerHeaderFile() or \
-                            itm.isLinguistFile() or \
-                            itm.isProjectFile() or \
-                            itm.isIdlFile():
-                        self.menu.popup(coord)
                     else:
-                        return
+                        self.menu.popup(coord)
                 elif isinstance(itm,BrowserClass) or \
                         isinstance(itm,BrowserMethod):
                     self.menu.popup(coord)
+                elif isinstance(itm,BrowserDirectory):
+                    if itm.parent() is None:
+                        self.dirMenu.setItemEnabled(self.removeFromToplevelItem, 1)
+                        self.dirMenu.setItemEnabled(self.addAsTopLevelItem, 0)
+                    else:
+                        self.dirMenu.setItemEnabled(self.removeFromToplevelItem, 0)
+                        self.dirMenu.setItemEnabled(self.addAsTopLevelItem, 1)
+                    self.dirMenu.popup(coord)
                 else:
                     self.backMenu.popup(coord)
             else:
@@ -816,8 +843,8 @@ class Browser(QListView):
         """
         hideNonPublic = Preferences.getUI("BrowsersHideNonPublic")
         if hideNonPublic != self.hideNonPublic:
-            self.refreshTree()
             self.hideNonPublic = hideNonPublic
+            self.refreshTree()
         
     def handleOpen(self):
         """
@@ -830,6 +857,8 @@ class Browser(QListView):
                 if isinstance(itm, BrowserFile):
                     if itm.isPythonFile():
                         self.emit(PYSIGNAL('pythonFile'),(itm.fileName(), -1))
+                    elif itm.isRubyFile():
+                        self.emit(PYSIGNAL('pythonFile'),(itm.fileName(),))
                     elif itm.isDesignerFile():
                         self.emit(PYSIGNAL('designerFile'),(itm.fileName(),))
                     elif itm.isDesignerHeaderFile():
@@ -842,6 +871,8 @@ class Browser(QListView):
                         self.emit(PYSIGNAL('pythonFile'),(itm.fileName(),))
                     elif itm.isPixmapFile():
                         self.emit(PYSIGNAL('pixmapFile'),(itm.fileName(),))
+                    else:
+                        self.emit(PYSIGNAL('pythonFile'),(itm.fileName(),))
                 elif isinstance(itm,BrowserClass):
                     self.emit(PYSIGNAL('pythonFile'),(itm.file, itm.cl.lineno))
                 elif isinstance(itm,BrowserMethod):
@@ -861,7 +892,43 @@ class Browser(QListView):
 
         if pyfn is not None:
             self.emit(PYSIGNAL('unittestOpen'),(pyfn,))
-            
+        
+    def handleNewToplevelDir(self):
+        """
+        Private slot to handle the New toplevel directory popup menu entry.
+        """
+        d = KQFileDialog.getExistingDirectory(\
+            None,
+            None, None,
+            self.trUtf8("New toplevel directory"),
+            1, 1)
+        if not d.isEmpty():
+            d = os.path.abspath(unicode(QDir.convertSeparators(d)))
+            if not self.toplevelDirs.contains(d):
+                node = BrowserDirectory(self, d, self.last)
+                self.last = node
+                self.toplevelDirs.append(node.fileName())
+        
+    def handleRemoveToplevel(self):
+        """
+        Private slot to handle the Remove from toplevel popup menu entry.
+        """
+        itm = self.currentItem()
+        self.toplevelDirs.remove(itm.fileName())
+        self.takeItem(itm)
+        del itm
+        
+    def handleAddAsToplevelDir(self):
+        """
+        Private slot to handle the Add as toplevel directory popup menu entry.
+        """
+        itm = self.currentItem()
+        d = itm.fileName()
+        if not self.toplevelDirs.contains(d):
+            node = BrowserDirectory(self, d, self.last)
+            self.last = node
+            self.toplevelDirs.append(node.fileName())
+        
     def handleProgramChange(self,fn):
         """
         Public slot to handle the programChange signal.
@@ -872,7 +939,7 @@ class Browser(QListView):
 
         self.progDir = BrowserDirectory(self,os.path.dirname(fn),None)
         
-    def wantedSelectedItem(self, itm, filter=None):
+    def wantedItem(self, itm, filter=None):
         """
         Private method to check type of a listview item.
         
@@ -897,7 +964,7 @@ class Browser(QListView):
         selectedItems = []
         itm = self.firstChild()
         while itm:
-            if itm.isSelected() and self.wantedSelectedItem(itm, filter):
+            if itm.isSelected() and self.wantedItem(itm, filter):
                 selectedItems.append(itm)
             itm = itm.itemBelow()
         return selectedItems
@@ -912,7 +979,29 @@ class Browser(QListView):
         count = 0
         itm = self.firstChild()
         while itm:
-            if itm.isSelected() and self.wantedSelectedItem(itm, filter):
+            if itm.isSelected() and self.wantedItem(itm, filter):
                 count += 1
             itm = itm.itemBelow()
         return count
+        
+    def getItemsFiltered(self, filter):
+        """
+        Public method to get the items of a specific type.
+        
+        @param filter list of class to check against
+        @return list of items (list of QListViewItem)
+        """
+        filteredItems = []
+        itm = self.firstChild()
+        while itm:
+            if self.wantedItem(itm, filter):
+                filteredItems.append(itm)
+            itm = itm.itemBelow()
+        return filteredItems
+        
+    def saveToplevelDirs(self):
+        """
+        Public slot to save the toplevel directories.
+        """
+        ok = Preferences.Prefs.settings.writeEntry('/eric3/Browser/ToplevelDirs', 
+                                                   self.toplevelDirs)
