@@ -60,9 +60,10 @@ class EmailDialog(EmailForm):
         else:
             msg = self.createSimpleMail()
             
-        self.sendmail(msg)
+        ok = self.sendmail(msg)
         
-        self.accept()
+        if ok:
+            self.accept()
         
     def createSimpleMail(self):
         """
@@ -70,21 +71,29 @@ class EmailDialog(EmailForm):
         
         @return string containing the mail message
         """
+        try:
+            import sipconfig
+            sip_version_str = sipconfig.Configuration().sip_version_str
+        except ImportError:
+            sip_version_str = "sip version not available"
+        
         if KdeQt.isKDE():
             msgtext = ("%s\r\n----\r\nVersion Numbers:\r\n  Python %s\r\n" + \
                 "  KDE %s\r\n  PyKDE %s\r\n  Qt %s\r\n" + \
-                "  PyQt %s\r\n  QScintilla %s\r\n  %s %s\r\n  Bicycle Repair Man %s\r\n" + \
-                "\r\nPlatform: %s\r\n%s\r\n") % (unicode(self.message.text()),
-                sys.version.split()[0],
+                "  PyQt %s\r\n  sip %s\r\n  QScintilla %s\r\n  %s %s\r\n" + \
+                "  Bicycle Repair Man %s\r\n\r\nPlatform: %s\r\n%s\r\n") % \
+                (unicode(self.message.text()), sys.version.split()[0],
                 str(KdeQt.kdeVersionString()), str(KdeQt.pyKdeVersionString()),
-                str(QT_VERSION_STR), str(PYQT_VERSION_STR), str(QSCINTILLA_VERSION_STR),
+                str(QT_VERSION_STR), str(PYQT_VERSION_STR), str(sip_version_str),
+                str(QSCINTILLA_VERSION_STR),
                 Program, Version, BRM_VERSION_STR, sys.platform, sys.version)
         else:
             msgtext = ("%s\r\n----\r\nVersion Numbers:\r\n  Python %s\r\n  Qt %s\r\n" + \
-                "  PyQt %s\r\n  QScintilla %s\r\n  %s %s\r\n  Bicycle Repair Man %s\r\n" + \
-                "\r\nPlatform: %s\r\n%s\r\n") % (unicode(self.message.text()),
-                sys.version.split()[0],
-                str(QT_VERSION_STR), str(PYQT_VERSION_STR), str(QSCINTILLA_VERSION_STR),
+                "  PyQt %s\r\n  sip %s\r\n  QScintilla %s\r\n  %s %s\r\n" + \
+                "  Bicycle Repair Man %s\r\n\r\nPlatform: %s\r\n%s\r\n") % \
+                (unicode(self.message.text()), sys.version.split()[0],
+                str(QT_VERSION_STR), str(PYQT_VERSION_STR), str(sip_version_str),
+                str(QSCINTILLA_VERSION_STR),
                 Program, Version, BRM_VERSION_STR, sys.platform, sys.version)
         
         msg = MIMEText(msgtext, _charset = sys.getdefaultencoding())
@@ -100,24 +109,32 @@ class EmailDialog(EmailForm):
         
         @return string containing the mail message
         """
+        try:
+            import sipconfig
+            sip_version_str = sipconfig.Configuration().sip_version_str
+        except ImportError:
+            sip_version_str = "sip version not available"
+        
         mpPreamble = ("This is a MIME-encoded message with attachments. "
             "If you see this message, your mail client is not "
             "capable of displaying the attachments.")
         if KdeQt.isKDE():
             msgtext = ("%s\r\n----\r\nVersion Numbers:\r\n  Python %s\r\n" + \
                 "  KDE %s\r\n  PyKDE %s\r\n  Qt %s\r\n" + \
-                "  PyQt %s\r\n  QScintilla %s\r\n  %s %s\r\n  Bicycle Repair Man %s\r\n" + \
-                "\r\nPlatform: %s\r\n%s\r\n") % (unicode(self.message.text()),
-                sys.version.split()[0],
+                "  PyQt %s\r\n  sip %s\r\n  QScintilla %s\r\n  %s %s\r\n" + \
+                "  Bicycle Repair Man %s\r\n\r\nPlatform: %s\r\n%s\r\n") % \
+                (unicode(self.message.text()), sys.version.split()[0],
                 str(KdeQt.kdeVersionString()), str(KdeQt.pyKdeVersionString()),
-                str(QT_VERSION_STR), str(PYQT_VERSION_STR), str(QSCINTILLA_VERSION_STR),
+                str(QT_VERSION_STR), str(PYQT_VERSION_STR), str(sip_version_str),
+                str(QSCINTILLA_VERSION_STR),
                 Program, Version, BRM_VERSION_STR, sys.platform, sys.version)
         else:
             msgtext = ("%s\r\n----\r\nVersion Numbers:\r\n  Python %s\r\n  Qt %s\r\n" + \
-                "  PyQt %s\r\n  QScintilla %s\r\n  %s %s\r\n  Bicycle Repair Man %s\r\n" + \
-                "\r\nPlatform: %s\r\n%s\r\n") % (unicode(self.message.text()),
-                sys.version.split()[0],
-                str(QT_VERSION_STR), str(PYQT_VERSION_STR), str(QSCINTILLA_VERSION_STR),
+                "  PyQt %s\r\n  sip %s\r\n  QScintilla %s\r\n  %s %s\r\n" + \
+                "  Bicycle Repair Man %s\r\n\r\nPlatform: %s\r\n%s\r\n") % \
+                (unicode(self.message.text()), sys.version.split()[0],
+                str(QT_VERSION_STR), str(PYQT_VERSION_STR), str(sip_version_str),
+                str(QSCINTILLA_VERSION_STR),
                 Program, Version, BRM_VERSION_STR, sys.platform, sys.version)
         
         # first part of multipart mail explains format
@@ -161,6 +178,7 @@ class EmailDialog(EmailForm):
         Private method to actually send the message.
         
         @param msg the message to be sent (string)
+        @return flag indicating success (boolean)
         """
         try:
             server = smtplib.SMTP(str(Preferences.getUser("MailServer")))
@@ -174,7 +192,7 @@ class EmailDialog(EmailForm):
                         QLineEdit.Password)
                     if not ok:
                         # abort
-                        return
+                        return 0
                 try:
                     server.login(str(Preferences.getUser("MailServerUser")),
                                  str(password))
@@ -188,9 +206,9 @@ class EmailDialog(EmailForm):
                         None,
                         0, -1)
                     if res == 0:
-                        self.sendmail(msg)
+                        return self.sendmail(msg)
                     else:
-                        return
+                        return 0
 
             result = server.sendmail(str(Preferences.getUser("Email")),
                 BugAddress, msg)
@@ -205,8 +223,11 @@ class EmailDialog(EmailForm):
                 None,
                 0, -1)
             if res == 0:
-                self.sendmail(msg)
-                
+                return self.sendmail(msg)
+            else:
+                return 0
+        return 1
+        
     def handleAdd(self):
         """
         Private slot to handle the Add... button.
