@@ -22,15 +22,16 @@ import imp
 import re
 import string
 
-__all__ = ["Module", "Class", "Function", "readModule"]
+__all__ = ["Module", "Class", "Function", "RbModule", "readModule"]
 
 TABWIDTH = 8
 
 PTL_SOURCE = 128
+RB_SOURCE = 129
 
-SUPPORTED_TYPES = [imp.PY_SOURCE, PTL_SOURCE]
+SUPPORTED_TYPES = [imp.PY_SOURCE, PTL_SOURCE, RB_SOURCE]
     
-_getnext = re.compile(r"""
+_py_getnext = re.compile(r"""
     (?P<String>
        \""" (?P<StringContents1>
                [^"\\]* (?:
@@ -144,6 +145,106 @@ _getnext = re.compile(r"""
     )
 """, re.VERBOSE | re.DOTALL | re.MULTILINE).search
 
+_rb_getnext = re.compile(r"""
+    (?P<Docstring>
+        =begin [ \t]+ edoc (?P<DocstringContents> .*? ) =end
+    )
+    
+|   (?P<String>
+        =begin .*? =end
+
+    |   " [^"\\\n]* (?: \\. [^"\\\n]*)* "
+
+    |   ' [^'\\\n]* (?: \\. [^'\\\n]*)* '
+    
+    |   <<-? ['"]? (?P<HereMarker> \w*? ) ['"]? [ \t]* $ .*? (?P=HereMarker)
+    )
+
+|   (?P<Comment>
+        ^
+        [ \t]* \#+ .*? $
+    )
+
+|   (?P<Method>
+        ^
+        (?P<MethodIndent> [ \t]* )
+        def [ \t]+
+        (?:
+            (?P<MethodName> [a-zA-Z_] [a-zA-Z0-9_?!=]* )
+        |
+            (?P<MethodName2> \w+ \. [a-zA-Z_] [a-zA-Z0-9_?!=]* )
+        )
+        [ \t]* 
+        (?: 
+            \( (?P<MethodSignature> (?: [^)] | \)[ \t]*,? )*? ) \)
+        )?
+        [ \t]*
+    )
+
+|   (?P<Class>
+        ^
+        (?P<ClassIndent> [ \t]* )
+        class
+        (?:
+            [ \t]+
+            (?P<ClassName> [A-Z] [a-zA-Z0-9_]* )
+            [ \t]*
+            (?P<ClassSupers> < [ \t]* [A-Z] [a-zA-Z0-9_]* )?
+        |
+            [ \t]* << [ \t]* 
+            (?P<ClassName2> [a-zA-Z_] [a-zA-Z0-9_]* )
+        )
+        [ \t]*
+    )
+
+|   (?P<ClassIgnored>
+        \(
+        [ \t]*
+        class
+        .*?
+        end
+        [ \t]*
+        \)
+    )
+
+|   (?P<Module>
+        ^
+        (?P<ModuleIndent> [ \t]* )
+        module [ \t]+
+        (?P<ModuleName> [A-Z] [a-zA-Z0-9_]* )
+        [ \t]*
+    )
+
+|   (?P<Attribute>
+        ^
+        (?P<AttributeIndent> [ \t]* )
+        (?P<AttributeName> (?: @ | @@ ) [a-zA-Z0-9_]* )
+        [ \t]* =
+    )
+
+|   (?P<Begin>
+            ^
+            [ \t]*
+            (?: if | unless | case | while | until | for | begin ) \b [^_]
+        |
+            [ \t]* do [ \t]* (?: \| .*? \| )? [ \t]* $
+    )
+
+|   (?P<End>
+        (?:
+            ^
+        |
+            ; 
+        )
+        [ \t]*
+        (?:
+            end [ \t]* $
+        |
+            end \b [^_]
+        )
+    )
+""", re.VERBOSE | re.DOTALL | re.MULTILINE).search
+
 _hashsub = re.compile(r"""^([ \t]*)#[ \t]?""", re.MULTILINE).sub
 
 _commentsub = re.compile(r"""#[^\n]*\n|#[^\n]*$""").sub
@@ -154,15 +255,18 @@ class Module:
     '''
     Class to represent a Python module.
     '''
-    def __init__(self, name, file=None):
+    def __init__(self, name, file=None, type=None):
         """
         Constructor
         
         @param name name of this module (string)
         @param file filename of file containing this module (string)
+        @param type type of this module
         """
         self.name = name
         self.file = file
+        self.modules = {}
+        self.modules_counts = {}
         self.classes = {}
         self.classes_counts = {}
         self.functions = {}
@@ -171,6 +275,13 @@ class Module:
         self.imports = []
         self.from_imports = {}
         self.package = '.'.join(name.split('.')[:-1])
+        self.type = type
+        if type in [imp.PY_SOURCE, PTL_SOURCE]:
+            self._getnext = _py_getnext
+        elif type == RB_SOURCE:
+            self._getnext = _rb_getnext
+        else:
+            self._getnext = None
         
     def _addclass(self, name, _class):
         """
@@ -186,6 +297,20 @@ class Module:
             self.classes_counts[name] = 0
         self.classes[name] = _class
         
+    def _addmodule(self, name, module):
+        """
+        Method to add information about a Ruby module.
+        
+        @param name name of module to be added (string)
+        @param module Module object to be added
+        """
+        if self.modules.has_key(name):
+            self.modules_counts[name] += 1
+            name = "%s_%d" % (name, self.modules_counts[name])
+        else:
+            self.modules_counts[name] = 0
+        self.modules[name] = module
+    
     def _addfunction(self, name, function):
         """
         Method to add information about a function.
@@ -212,6 +337,17 @@ class Module:
         """
         Method to scan the source text and retrieve the relevant information.
         
+        @param src the source text to be scanned (string)
+        """
+        if self.type in [imp.PY_SOURCE, PTL_SOURCE]:
+            self.__py_scan(src)
+        elif self.type == RB_SOURCE:
+            self.__rb_scan(src)
+        
+    def __py_scan(self, src):
+        """
+        Method to scan the source text of a Python module and retrieve the relevant information.
+        
         @param src the source text to be scanned (string)
         """
         # To avoid having to stop the regexp at each newline, instead
@@ -231,7 +367,7 @@ class Module:
         modulelevel = 1
         cur_obj = self
         while 1:
-            m = _getnext(src, i)
+            m = self._getnext(src, i)
             if not m:
                 break
             start, i = m.span()
@@ -403,7 +539,165 @@ class Module:
                 assert 0, "regexp _getnext found something unexpected"
             
             modulelevel = 0
+        
+    def __rb_scan(self, src):
+        """
+        Method to scan the source text of a Python module and retrieve the relevant information.
+        
+        @param src the source text to be scanned (string)
+        """
+        # To avoid having to stop the regexp at each newline, instead
+        # when we need a line number we simply string.count the number of
+        # newlines in the string since the last time we did this; i.e.,
+        #    lineno = lineno + \
+        #             string.count(src, '\n', last_lineno_pos, here)
+        #    last_lineno_pos = here
+        countnl = string.count
+        lineno, last_lineno_pos = 1, 0
+        classstack = [] # stack of (class, indent) pairs
+        indent = 0
+        i = 0
+        cur_obj = self
+        while 1:
+            m = self._getnext(src, i)
+            if not m:
+                break
+            start, i = m.span()
             
+            if m.start("Method") >= 0:
+                # found a method definition or function
+                thisindent = indent
+                indent += 1
+                meth_name = m.group("MethodName") or m.group("MethodName2")
+                meth_sig = m.group("MethodSignature")
+                meth_sig = meth_sig and meth_sig.replace('\\\n', '') or ''
+                lineno = lineno + \
+                         countnl(src, '\n',
+                                 last_lineno_pos, start)
+                last_lineno_pos = start
+                # close all classes/modules indented at least as much
+                while classstack and \
+                      classstack[-1][1] >= thisindent:
+                    del classstack[-1]
+                if classstack:
+                    csi = -1
+                    while csi >= -len(classstack):
+                        # nested defs are added to the class
+                        cur_class = classstack[csi][0]
+                        csi -= 1
+                        if cur_class is None:
+                            continue
+                            
+                        if isinstance(cur_class, Class) or isinstance(cur_class, RbModule):
+                            # it's a class/module method
+                            f = Function(None, meth_name,
+                                         None, lineno, meth_sig)
+                            cur_class._addmethod(meth_name, f)
+                            break
+                    else:
+                        # it's a nested function of a module function
+                        f = Function(self.name, meth_name, file, lineno, meth_sig)
+                        self._addfunction(meth_name, f)
+                else:
+                    # it's a function
+                    f = Function(self.name, meth_name, file, lineno, meth_sig)
+                    self._addfunction(meth_name, f)
+                cur_obj = f
+                classstack.append((None, thisindent)) # Marker for nested fns
+    
+            elif m.start("Docstring") >= 0:
+                contents = m.group("DocstringContents")
+                if contents is not None:
+                    contents = _hashsub(r"\1", contents)
+                if cur_obj:
+                    cur_obj._adddescription(contents)
+    
+            elif m.start("String") >= 0:
+                pass
+    
+            elif m.start("Comment") >= 0:
+                pass
+    
+            elif m.start("ClassIgnored") >= 0:
+                pass
+
+            elif m.start("Class") >= 0:
+                # we found a class definition
+                thisindent = indent
+                indent += 1
+                # close all classes/modules indented at least as much
+                while classstack and \
+                      classstack[-1][1] >= thisindent:
+                    del classstack[-1]
+                lineno = lineno + \
+                         countnl(src, '\n', last_lineno_pos, start)
+                last_lineno_pos = start
+                class_name = m.group("ClassName") or m.group("ClassName2")
+                inherit = m.group("ClassSupers")
+                if inherit:
+                    # the class inherits from other classes
+                    inherit = inherit[1:].strip()
+                    inherit = [_commentsub('', inherit)]
+                # remember this class
+                cur_class = Class(self.name, class_name, inherit,
+                                  file, lineno)
+                # add nested classes to the file
+                if classstack and isinstance(classstack[-1][0], RbModule):
+                    parent_obj = classstack[-1][0]
+                else:
+                    parent_obj = self
+                if parent_obj.classes.has_key(class_name):
+                    cur_class = parent_obj.classes[class_name]
+                else:
+                    parent_obj._addclass(class_name, cur_class)
+                cur_obj = cur_class
+                classstack.append((cur_class, thisindent))
+    
+            elif m.start("Module") >= 0:
+                # we found a module definition
+                thisindent = indent
+                indent += 1
+                # close all classes/modules indented at least as much
+                while classstack and \
+                      classstack[-1][1] >= thisindent:
+                    del classstack[-1]
+                lineno = lineno + \
+                         countnl(src, '\n', last_lineno_pos, start)
+                last_lineno_pos = start
+                module_name = m.group("ModuleName")
+                # remember this class
+                cur_class = RbModule(self.name, module_name,
+                                  file, lineno)
+                # add nested Ruby modules to the file
+                if self.modules.has_key(module_name):
+                    cur_class = self.modules[module_name]
+                else:
+                    self._addmodule(module_name, cur_class)
+                cur_obj = cur_class
+                classstack.append((cur_class, thisindent))
+    
+            elif m.start("Attribute") >= 0:
+                index = -1
+                while index >= -len(classstack):
+                    if classstack[index][0] is not None and \
+                       not isinstance(classstack[index][0], Function) and \
+                       not classstack[index][1] >= indent:
+                        classstack[index][0]._addattribute(m.group("AttributeName"))
+                        break
+                    else:
+                        index -= 1
+    
+            elif m.start("Begin") >= 0:
+                # a begin of a block we are not interested in
+                indent += 1
+    
+            elif m.start("End") >= 0:
+                # an end of a block
+                indent -= 1
+    
+            else:
+                assert 0, "regexp _getnext found something unexpected"
+        
     def createHierarchy(self):
         """
         Method to build the inheritance hierarchy for all classes of this module.
@@ -413,6 +707,8 @@ class Module:
         hierarchy =  {}
         for cls in self.classes.keys():
             self.assembleHierarchy(cls, self.classes, [cls], hierarchy)
+        for mod in self.modules.keys():
+            self.assembleHierarchy(mod, self.modules, [mod], hierarchy)
         return hierarchy
         
     def assembleHierarchy(self, name, classes, path, result):
@@ -532,6 +828,31 @@ class Class:
         """
         self.description = description
 
+class RbModule(Class):
+    '''
+    Class to represent a Ruby module.
+    '''
+    def __init__(self, module, name, file, lineno):
+        """
+        Constructor
+        
+        @param module name of module containing this class (string)
+        @param name name of the class (string)
+        @param file name of file containing this class (string)
+        @param lineno linenumber of the class definition (integer)
+        """
+        Class.__init__(self, module, name, None, file, lineno)
+        self.classes = {}
+        
+    def _addclass(self, name, _class):
+        """
+        Method to add information about a class.
+        
+        @param name name of class to be added (string)
+        @param _class Class object to be added
+        """
+        self.classes[name] = _class
+
 class Function:
     '''
     Class to represent a Python function or method.
@@ -586,15 +907,23 @@ def readModule(module, path=[], inpackage=0, basename=""):
         path = [os.path.dirname(module)]
         if module.lower().endswith(".py"):
             module = module[:-3]
-        if os.path.exists(os.path.join(path[0], "__init__.py")):
+        if os.path.exists(os.path.join(path[0], "__init__.py")) or \
+           os.path.exists(os.path.join(path[0], "__init__.rb")) or \
+           inpackage:
             if basename:
                 module = module.replace(basename, "")
-            modname = module.replace(os.sep, '.')
+            if os.path.isabs(module):
+                modname = os.path.splitdrive(module)[1][len(os.sep):]
+            else:
+                modname = module
+            modname = modname.replace(os.sep, '.')
             inpackage = 1
         else:
             modname = os.path.basename(module)
         if modname.lower().endswith(".ptl"):
             modname = modname[:-4]
+        elif modname.lower().endswith(".rb"):
+            modname = modname[:-3]
         module = os.path.basename(module)
 
     if _modules.has_key(modname):
@@ -603,7 +932,7 @@ def readModule(module, path=[], inpackage=0, basename=""):
 
     if module in sys.builtin_module_names:
         # this is a built-in module
-        _modules[modname] = Module(modname, None)
+        _modules[modname] = Module(modname, None, None)
         return _modules[modname]
 
     # search the path for the module
@@ -620,10 +949,10 @@ def readModule(module, path=[], inpackage=0, basename=""):
         # not Python source, can't do anything with this module
         if f:
             f.close()
-        _modules[modname] = Module(modname, None)
+        _modules[modname] = Module(modname, None, None)
         return _modules[modname]
 
-    mod = Module(modname, file)
+    mod = Module(modname, file, type)
     mod.scan(f.read())
     f.close()
     _modules[modname] = mod
@@ -654,13 +983,23 @@ def find_module(name, path):
         is a tuple of file suffix, file mode and file type)
     @exception ImportError The file or module wasn't found.
     """
+    # Quixote page template
     if name.lower().endswith('.ptl'):
         for p in path:      # only search in path
             if os.path.exists(os.path.join(p, name)):
                 pathname = os.path.join(p, name)
                 return (open(pathname), pathname, ('.ptl', 'r', PTL_SOURCE))
         raise ImportError
-        
+    
+    # Ruby source file
+    if name.lower().endswith('.rb'):
+        for p in path:      # only search in path
+            if os.path.exists(os.path.join(p, name)):
+                pathname = os.path.join(p, name)
+                return (open(pathname), pathname, ('.rb', 'r', RB_SOURCE))
+        raise ImportError
+    
+    # standard Python module file
     if name.lower().endswith('.py'):
         name = name[:-3]
         
@@ -694,6 +1033,8 @@ def resetParsedModule(module, basename=""):
             modname = os.path.basename(module)
         if modname.lower().endswith(".ptl"):
             modname = modname[:-4]
+        elif modname.lower().endswith(".rb"):
+            modname = modname[:-3]
         module = os.path.basename(module)
 
     if _modules.has_key(modname):
