@@ -144,7 +144,7 @@ _escape_map = {
 
 def escape_entities(m, map=_escape_map):
     """
-    Function to encode a html entities.
+    Function to encode html entities.
     
     @param m the match object
     @param map the map of entioties to encode
@@ -169,6 +169,33 @@ def html_encode(text, pattern=_escape):
     text = pattern.sub(escape_entities, text)
     return text.encode("ascii")
 
+_uescape = re.compile(eval(r'u"[\u0080-\uffff]"'))
+
+def escape_uentities(m):
+    """
+    Function to encode html entities.
+    
+    @param m the match object
+    @return the converted text (string)
+    """
+    char = m.group()
+    text = "&#%d;" % ord(char)
+    return text
+    
+def html_uencode(text, pattern=_uescape):
+    """
+    Function to correctly encode a unicode text for html.
+    
+    @param text text to be encoded (string)
+    @param pattern search pattern for text to be encoded (string)
+    @return the encoded text (string)
+    """
+    if not text:
+        return ""
+    text = unicode(text)
+    text = pattern.sub(escape_uentities, text)
+    return text.encode("ascii")
+
 def normcasepath(path):
     """
     Function returning a path, that is normalized with respect to its case and references.
