@@ -40,9 +40,11 @@ class CvsDiffDialog(LogForm):
         ))
         
         self.setWFlags(self.getWFlags() | Qt.WDestructiveClose)
-        self.process = QProcess(self)
+        self.process = QProcess()
         self.vcs = vcs
         
+        self.contents.setTextFormat(QTextEdit.LogText)
+        
         self.cAdded = QColor(190, 237, 190)
         self.cRemoved = QColor(237, 190, 190)
         self.cNormal = self.contents.paletteBackgroundColor()
