@@ -40,9 +40,11 @@ class CvsStatusDialog(LogForm):
         ))
         
         self.setWFlags(self.getWFlags() | Qt.WDestructiveClose)
-        self.process = QProcess(self)
+        self.process = QProcess()
         self.vcs = vcs
         
+        self.contents.setTextFormat(QTextEdit.LogText)
+        
         self.connect(self.process, SIGNAL('readyReadStdout()'),
             self.handleReadStdout)
         self.connect(self.process, SIGNAL('readyReadStderr()'),
