@@ -70,7 +70,7 @@ class Cvs(VersionControl):
         """
         self.versionStr = None
         
-        self.proc = QProcess('cvs', self)
+        self.proc = QProcess('cvs')
         self.proc.addArgument('-v')
         self.connect(self.proc, SIGNAL('processExited()'), self.vcsExistsProcExited)
         self.connect(self.proc, SIGNAL('readyReadStdout()'), self.vcsExistsReadyReadStdout)
@@ -84,7 +84,7 @@ class Cvs(VersionControl):
         @param noDialog flag indicating quiet operations (boolean)
         @return flag indicating an execution without errors (boolean)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('-d')
         proc.addArgument(vcsDir)
@@ -117,7 +117,7 @@ class Cvs(VersionControl):
         if msg.isEmpty():
             msg = QString('***')
             
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('-d')
         proc.addArgument(vcsDir)
@@ -149,7 +149,7 @@ class Cvs(VersionControl):
         @param tag tag of version to check out (string)
         @return flag indicating an execution without errors (boolean)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('-d')
         proc.addArgument(vcsDir)
@@ -180,7 +180,7 @@ class Cvs(VersionControl):
         @param tag tag of version to check out (string)
         @return flag indicating an execution without errors (boolean)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('-d')
         proc.addArgument(vcsDir)
@@ -219,7 +219,7 @@ class Cvs(VersionControl):
         if msg.isEmpty():
             msg = QString('***')
             
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('commit')
         self.addArguments(proc, self.options['commit'])
@@ -248,7 +248,7 @@ class Cvs(VersionControl):
         
         @param name file/directory name to be updated (string)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('update')
         self.addArguments(proc, self.options['update'])
@@ -286,7 +286,7 @@ class Cvs(VersionControl):
         @param isDir flag indicating name is a directory (boolean)
         @param binary flag indicating a binary add (boolean)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('add')
         self.addArguments(proc, self.options['add'])
@@ -357,7 +357,7 @@ class Cvs(VersionControl):
         
         @param path root directory of the tree to be added (string)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('add')
         self.addArguments(proc, self.options['add'])
@@ -417,7 +417,7 @@ class Cvs(VersionControl):
         @param project flag indicating deletion of a project tree (boolean)
         @return flag indicating successfull operation (boolean)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('remove')
         self.addArguments(proc, self.options['remove'])
@@ -425,7 +425,7 @@ class Cvs(VersionControl):
             dname = self.splitPathList(name)[0]
             for n in name:
                 if os.path.isdir(n):
-                    entries = utilities.direntries(n, 1)
+                    entries = Utilities.direntries(n, 1)
                     self.addArguments(proc, entries)
                 else:
                     proc.addArgument(n)
@@ -520,7 +520,7 @@ class Cvs(VersionControl):
         else:
             return
             
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('tag')
         self.addArguments(proc, self.options['tag'])
@@ -544,7 +544,7 @@ class Cvs(VersionControl):
         
         @param name file/directory name to be reverted (string)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('update')
         self.addArguments(proc, self.options['update'])
@@ -578,7 +578,7 @@ class Cvs(VersionControl):
         else:
             return
             
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('update')
         self.addArguments(proc, self.options['update'])
@@ -611,7 +611,7 @@ class Cvs(VersionControl):
         else:
             return
             
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('update')
         self.addArguments(proc, self.options['update'])
@@ -646,7 +646,7 @@ class Cvs(VersionControl):
             repdir = ':%s:%s@%s:%s' % (protocol, user, host, repository)
         else:
             repdir = ':%s:%s%s@%s:%s%s' % (protocol, user, passwd, host, port, repository)
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('-d')
         proc.addArgument(repdir)
@@ -668,7 +668,7 @@ class Cvs(VersionControl):
         @param repository repository name (string)
         """
         repdir = ':%s:%s@%s:%s%s' % (protocol, user, host, port, repository)
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('-d')
         proc.addArgument(repdir)
@@ -786,7 +786,7 @@ class Cvs(VersionControl):
             self.wdHistory.remove(wd)
             self.wdHistory.prepend(wd)
             
-            proc = QProcess('cvs', self)
+            proc = QProcess('cvs')
             self.addArguments(proc, commandList)
             proc.setWorkingDirectory(QDir(wd))
             
@@ -863,7 +863,7 @@ class Cvs(VersionControl):
         
         @param name file name(s) to be edited (string or list of strings)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('edit')
         proc.addArgument('-a')
@@ -890,7 +890,7 @@ class Cvs(VersionControl):
         
         @param name file name(s) to be unedited (string or list of strings)
         """
-        proc = QProcess('cvs', self)
+        proc = QProcess('cvs')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('unedit')
         if type(name) is types.ListType:
