@@ -40,9 +40,11 @@ class SvnDiffDialog(LogForm):
         ))
         
         self.setWFlags(self.getWFlags() | Qt.WDestructiveClose)
-        self.process = QProcess(self)
+        self.process = QProcess()
         self.vcs = vcs
         
+        self.contents.setTextFormat(QTextEdit.LogText)
+        
         self.cAdded = QColor(190, 237, 190)
         self.cRemoved = QColor(237, 190, 190)
         self.cNormal = self.contents.paletteBackgroundColor()
@@ -71,7 +73,7 @@ class SvnDiffDialog(LogForm):
         Public slot to start the svn diff command.
         
         @param fn filename to be diffed (string)
-        @param versions list of versions to be diffed (list of 2 QString or None)
+        @param versions list of versions to be diffed (list of up to 2 QString or None)
         """
         self.intercept = 0
         self.filename = fn
@@ -93,7 +95,10 @@ class SvnDiffDialog(LogForm):
             except:
                 pass
             self.process.addArgument('-r')
-            self.process.addArgument('%s:%s' % (str(versions[0]), str(versions[1])))
+            if len(versions) == 1:
+                self.process.addArgument(str(versions[0]))
+            else:
+                self.process.addArgument('%s:%s' % (str(versions[0]), str(versions[1])))
         if type(fn) is types.ListType:
             dname, fnames = self.vcs.splitPathList(fn)
             self.vcs.addArguments(self.process, fnames)
