@@ -8,11 +8,14 @@ Module implementing a dialog to show the output of the svn status command proces
 """
 
 import types
+import os
 
 from qt import *
 
 from StatusForm import StatusForm
 
+import Preferences
+
 class SvnStatusDialog(StatusForm):
     """
     Module implementing a dialog to show the output of the svn status command process.
@@ -27,11 +30,18 @@ class SvnStatusDialog(StatusForm):
         StatusForm.__init__(self, parent)
         
         self.setWFlags(self.getWFlags() | Qt.WDestructiveClose)
-        self.process = QProcess(self)
+        self.process = QProcess()
         self.vcs = vcs
         
         self.statusList.setSorting(9)
         
+        self.menu = QPopupMenu()
+        self.commitItem = self.menu.insertItem(self.trUtf8("Commit changes to repository..."),
+            self.handleCommit)
+        self.menu.setItemEnabled(self.commitItem, 0)
+        
+        self.connect(self.statusList,SIGNAL('contextMenuRequested(QListViewItem *, const QPoint &, int)'),
+                     self.handleContextMenu)
         self.connect(self.process, SIGNAL('readyReadStdout()'),
             self.handleReadStdout)
         self.connect(self.process, SIGNAL('readyReadStderr()'),
@@ -39,6 +49,11 @@ class SvnStatusDialog(StatusForm):
         self.connect(self.process, SIGNAL('processExited()'),
             self.handleProcessExited)
             
+        self.modifiedIndicators = QStringList()
+        self.modifiedIndicators.append(self.trUtf8('added'))
+        self.modifiedIndicators.append(self.trUtf8('deleted'))
+        self.modifiedIndicators.append(self.trUtf8('modified'))
+        
         self.status = {
             ' ' : self.trUtf8('normal'),
             'A' : self.trUtf8('added'),
@@ -74,7 +89,17 @@ class SvnStatusDialog(StatusForm):
         }
         
         self.rx_status = QRegExp('(.{8})\\s+([0-9]+)\\s+([0-9?]+)\\s+([\\w?]+)\\s+(.+)')
-            
+        
+    def handleContextMenu(self,itm,coord,col):
+        """
+        Private slot to show the context menu of the listview.
+        
+        @param itm the selected listview item (QListViewItem)
+        @param coord the position of the mouse pointer (QPoint)
+        @param col the column of the mouse pointer (int)
+        """
+        self.menu.popup(coord)
+        
     def closeEvent(self, e):
         """
         Private slot implementing a close event handler.
@@ -94,8 +119,18 @@ class SvnStatusDialog(StatusForm):
         @param fn -- filename(s) to show the status of (string or list of strings)
         """
         self.intercept = 0
+        self.args = fn
         
-        self.process.kill()
+        if self.process:
+            self.process.kill()
+        else:
+            self.process = QProcess()
+            self.connect(self.process, SIGNAL('readyReadStdout()'),
+                self.handleReadStdout)
+            self.connect(self.process, SIGNAL('readyReadStderr()'),
+                self.handleReadStderr)
+            self.connect(self.process, SIGNAL('processExited()'),
+                self.handleProcessExited)
         
         self.process.clearArguments()
         self.process.addArgument('svn')
@@ -111,12 +146,12 @@ class SvnStatusDialog(StatusForm):
             except:
                 pass
         if type(fn) is types.ListType:
-            dname, fnames = self.vcs.splitPathList(fn)
+            self.dname, fnames = self.vcs.splitPathList(fn)
             self.vcs.addArguments(self.process, fnames)
         else:
-            dname, fname = self.vcs.splitPath(fn)
+            self.dname, fname = self.vcs.splitPath(fn)
             self.process.addArgument(fname)
-        self.process.setWorkingDirectory(QDir(dname))
+        self.process.setWorkingDirectory(QDir(self.dname))
         
         self.process.start()
         self.setCaption(self.trUtf8('Subversion Status'))
@@ -129,11 +164,14 @@ class SvnStatusDialog(StatusForm):
             self.process.tryTerminate()
             QTimer.singleShot(2000, self.process, SLOT('kill()'))
             
-        self.cancelButton.setText(self.trUtf8('OK'))
+        self.cancelButton.setText(self.trUtf8('&OK'))
         self.cancelButton.setDefault(1)
         self.cancelButton.setFocus()
         
         self.inputGroup.setEnabled(0)
+        self.refreshButton.setEnabled(1)
+        
+        self.menu.setItemEnabled(self.commitItem, 1)
         
         self.process = None
         
@@ -159,7 +197,9 @@ class SvnStatusDialog(StatusForm):
         Private slot to handle the readyReadStdout signal.
         
         It reads the output of the process, formats it and inserts it into
-        the contents pane.
+        the contents pane.<br />
+        <b>Note</b>: If the order gets changed, the handleCommit and
+        getModifiedItems method need to be changed accordingly.
         """
         while self.process.canReadLineStdout():
             s = self.process.readLineStdout()
@@ -168,7 +208,7 @@ class SvnStatusDialog(StatusForm):
                 rev = self.rx_status.cap(2)
                 change = self.rx_status.cap(3)
                 author = self.rx_status.cap(4)
-                path = self.rx_status.cap(5)
+                path = self.rx_status.cap(5).stripWhiteSpace()
                 itm = QListViewItem(self.statusList,
                     self.status[flags[0]], self.propStatus[flags[1]],
                     self.locked[flags[2]], self.history[flags[3]],
@@ -236,3 +276,49 @@ class SvnStatusDialog(StatusForm):
             evt.accept()
             return
         StatusForm.keyPressEvent(self, evt)
+        
+    def handleCommit(self):
+        """
+        Private slot to handle the Commit context menu entry.
+        """
+        names = [os.path.join(self.dname, unicode(itm.text(9))) \
+                 for itm in self.getModifiedItems()]
+        if Preferences.getVCS("AutoSaveFiles"):
+            vm = qApp.mainWidget().getViewManager()
+            for name in names:
+                vm.saveEditor(name)
+        self.vcs.vcsCommit(names, '', parent=self)
+        self.refreshStatus()
+        
+    def getModifiedItems(self):
+        """
+        Private method to retrieve all emtries, that have a modified status.
+        
+        @return list of all items with a modified status
+        """
+        modifiedItems = []
+        itm = self.statusList.firstChild()
+        while itm:
+            if itm.isSelected() and \
+                    (self.modifiedIndicators.contains(itm.text(0)) or \
+                     self.modifiedIndicators.contains(itm.text(1))):
+                modifiedItems.append(itm)
+            itm = itm.itemBelow()
+        return modifiedItems
+
+    def refreshStatus(self):
+        """
+        Private slot to refresh the status display.
+        """
+        self.cancelButton.setText(self.trUtf8('&Cancel'))
+        self.cancelButton.setDefault(1)
+        self.cancelButton.setFocus()
+        
+        self.inputGroup.setEnabled(1)
+        self.refreshButton.setEnabled(0)
+        
+        self.menu.setItemEnabled(self.commitItem, 0)
+        
+        self.statusList.clear()
+        
+        self.start(self.args)
