@@ -21,6 +21,7 @@ from SvnDialog import SvnDialog
 from CommitDialog import SvnCommitDialog
 from LogDialog import SvnLogDialog
 from DiffDialog import SvnDiffDialog
+from RevisionSelectionDialog import SvnRevisionSelectionDialog
 from StatusDialog import SvnStatusDialog
 from TagDialog import SvnTagDialog
 from TagBranchListDialog import SvnTagBranchListDialog
@@ -84,7 +85,7 @@ class Subversion(VersionControl):
         """
         self.versionStr = None
         
-        self.proc = QProcess('svn', self)
+        self.proc = QProcess('svn')
         self.proc.addArgument('--version')
         self.connect(self.proc, SIGNAL('processExited()'), self.vcsExistsProcExited)
         self.connect(self.proc, SIGNAL('readyReadStdout()'), self.vcsExistsReadyReadStdout)
@@ -140,7 +141,7 @@ class Subversion(VersionControl):
         else:
             shutil.copytree(projectDir, os.path.join(tmpDir, project))
         
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('import')
         self.addArguments(proc, self.options['global'])
         proc.addArgument('-m')
@@ -193,7 +194,7 @@ class Subversion(VersionControl):
                 svnUrl = '%s/%s/%s' % (vcsDir, project, tag)
         else:
             svnUrl = '%s/%s' % (vcsDir, project)
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('checkout')
         self.addArguments(proc, self.options['global'])
         self.addArguments(proc, self.options['checkout'])
@@ -240,7 +241,7 @@ class Subversion(VersionControl):
                 svnUrl = '%s/%s/%s' % (vcsDir, project, tag)
         else:
             svnUrl = '%s/%s' % (vcsDir, project)
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('export')
         self.addArguments(proc, self.options['global'])
         proc.addArgument("--force")
@@ -253,20 +254,21 @@ class Subversion(VersionControl):
             dia.exec_loop()
         return dia.normalExit()
         
-    def vcsCommit(self, name, message, noDialog=0):
+    def vcsCommit(self, name, message, noDialog=0, parent=None):
         """
         Public method used to make the change of a file/directory permanent in the Subversion repository.
         
         @param name file/directory name to be committed (string or list of strings)
         @param message message for this operation (string)
         @param noDialog flag indicating quiet operations
+        @param parent reference to the parent object of the commit dialog (QWidget)
         @return flag indicating an execution without errors (boolean)
         """
         msg = QString(message)
         
         if not noDialog and msg.isEmpty():
             # call CommitDialog and get message from there
-            dlg = SvnCommitDialog()
+            dlg = SvnCommitDialog(parent)
             if dlg.exec_loop() == QDialog.Accepted:
                 msg = dlg.logMessage()
             else:
@@ -275,18 +277,12 @@ class Subversion(VersionControl):
         if msg.isEmpty():
             msg = QString('***')
             
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('commit')
         self.addArguments(proc, self.options['global'])
         self.addArguments(proc, self.options['commit'])
         proc.addArgument('-m')
         proc.addArgument(msg)
-        try:
-            if self.svnGetReposName(dname).startswith('http') or \
-               self.svnGetReposName(dname).startswith('svn'):
-                noDialog = 0
-        except:
-            noDialog = 0
         if type(name) is types.ListType:
             dname, fnames = self.splitPathList(name)
             self.addArguments(proc, fnames)
@@ -295,10 +291,17 @@ class Subversion(VersionControl):
             proc.addArgument(fname)
         proc.setWorkingDirectory(QDir(dname))
         
+        try:
+            if self.svnGetReposName(dname).startswith('http') or \
+               self.svnGetReposName(dname).startswith('svn'):
+                noDialog = 0
+        except:
+            noDialog = 0
+        
         if noDialog:
             return self.startSynchronizedProcess(proc)
         else:
-            dia = SvnDialog(self.trUtf8('Commiting changes to Subversion repository'))
+            dia = SvnDialog(self.trUtf8('Commiting changes to Subversion repository'), parent)
             res = dia.startProcess(proc)
             if res:
                 dia.exec_loop()
@@ -310,7 +313,7 @@ class Subversion(VersionControl):
         
         @param name file/directory name to be updated (string or list of strings)
         """
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('update')
         self.addArguments(proc, self.options['global'])
         self.addArguments(proc, self.options['update'])
@@ -334,7 +337,7 @@ class Subversion(VersionControl):
         @param name file/directory name to be added (string)
         @param isDir flag indicating name is a directory (boolean)
         """
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('add')
         self.addArguments(proc, self.options['global'])
         self.addArguments(proc, self.options['add'])
@@ -395,7 +398,7 @@ class Subversion(VersionControl):
         
         @param path root directory of the tree to be added (string or list of strings))
         """
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('add')
         self.addArguments(proc, self.options['global'])
         self.addArguments(proc, self.options['add'])
@@ -442,7 +445,7 @@ class Subversion(VersionControl):
         @param project flag indicating deletion of a project tree (boolean) (not needed)
         @return flag indicating successfull operation (boolean)
         """
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('delete')
         self.addArguments(proc, self.options['global'])
         self.addArguments(proc, self.options['remove'])
@@ -561,7 +564,7 @@ class Subversion(VersionControl):
         else:
             url = self.svnURL(unicode(tag))
             
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         if tagOp in [1, 2]:
             proc.addArgument('copy')
             self.addArguments(proc, self.options['global'])
@@ -590,7 +593,7 @@ class Subversion(VersionControl):
         
         @param name file/directory name to be reverted (string)
         """
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('revert')
         self.addArguments(proc, self.options['global'])
         if type(name) is types.ListType:
@@ -665,7 +668,7 @@ class Subversion(VersionControl):
             url = self.svnURL(unicode(tag))
             tn = url
             
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('switch')
         proc.addArgument(url)
         proc.addArgument(name)
@@ -698,7 +701,7 @@ class Subversion(VersionControl):
 
         rx_rev = QRegExp('\\d+|HEAD')
         
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('merge')
         self.addArguments(proc, self.options['global'])
         if rx_rev.exactMatch(urlrev1):
@@ -798,7 +801,7 @@ class Subversion(VersionControl):
         
         @param name directory name to be cleaned up (string)
         """
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('cleanup')
         self.addArguments(proc, self.options['global'])
         proc.addArgument(name)
@@ -827,7 +830,7 @@ class Subversion(VersionControl):
             self.wdHistory.remove(wd)
             self.wdHistory.prepend(wd)
             
-            proc = QProcess('svn', self)
+            proc = QProcess('svn')
             self.addArguments(proc, commandList)
             if not wd.isEmpty():
                 proc.setWorkingDirectory(QDir(wd))
@@ -953,7 +956,7 @@ class Subversion(VersionControl):
         
         @param name file/directory name to be resolved (string)
         """
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         if self.versionStr < "0.27.0":
             proc.addArgument('resolve')
         else:
@@ -985,7 +988,7 @@ class Subversion(VersionControl):
         if dlg.exec_loop() == QDialog.Accepted:
             target = dlg.getData()
             
-            proc = QProcess('svn', self)
+            proc = QProcess('svn')
             proc.addArgument('copy')
             self.addArguments(proc, self.options['global'])
             if rx_prot.exactMatch(target):
@@ -1019,7 +1022,7 @@ class Subversion(VersionControl):
         if dlg.exec_loop() == QDialog.Accepted:
             target = dlg.getData()
             
-            proc = QProcess('svn', self)
+            proc = QProcess('svn')
             proc.addArgument('move')
             self.addArguments(proc, self.options['global'])
             if rx_prot.exactMatch(target):
@@ -1038,7 +1041,7 @@ class Subversion(VersionControl):
                 if res and not rx_prot.exactMatch(target):
                     project.removeFile(name)
                     project.appendFile(target)
-                    project.renameMainScript(name, target) 
+                    project.renameMainScript(name, target)
         return res
         
     def svnListProps(self, name, recursive = 0):
@@ -1072,7 +1075,7 @@ class Subversion(VersionControl):
                     0, -1)
                 return
             
-            proc = QProcess('svn', self)
+            proc = QProcess('svn')
             proc.addArgument('propset')
             self.addArguments(proc, self.options['global'])
             if recursive:
@@ -1119,7 +1122,7 @@ class Subversion(VersionControl):
                 0, -1)
             return
             
-        proc = QProcess('svn', self)
+        proc = QProcess('svn')
         proc.addArgument('propdel')
         self.addArguments(proc, self.options['global'])
         if recursive:
@@ -1149,6 +1152,40 @@ class Subversion(VersionControl):
         self.tagbranchList = SvnTagBranchListDialog(self)
         self.tagbranchList.show()
         self.tagbranchList.start(path, tags)
+        
+    def svnExtendedDiff(self, name):
+        """
+        Public method used to view the difference of a file/directory to the Subversion repository.
+        
+        If name is a directory and is the project directory, all project files
+        are saved first. If name is a file (or list of files), which is/are being edited 
+        and has unsaved modification, they can be saved or the operation may be aborted.
+        
+        This method gives the chance to enter the revisions to be compared. An entry of 
+        0 for any specific revision is identical to the current revision (i.e the
+        revision as found in the working copy)
+        
+        @param name file/directory name to be diffed (string)
+        """
+        if type(name) is types.ListType:
+            names = name[:]
+        else:
+            names = [name]
+        for nam in names:
+            if os.path.isfile(nam):
+                editor = qApp.mainWidget().getViewManager().getOpenEditor(nam)
+                if editor and not editor.checkDirty() :
+                    return
+            else:
+                project = qApp.mainWidget().getProject()
+                if nam == project.ppath and not project.saveAllScripts():
+                    return
+        dlg = SvnRevisionSelectionDialog()
+        if dlg.exec_loop() == QDialog.Accepted:
+            revisions = dlg.getRevisions()
+            self.diff = SvnDiffDialog(self)
+            self.diff.show()
+            self.diff.start(name, revisions)
 
     ############################################################################
     ## Private Subversion specific methods are below.
