@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002, 2003 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2005 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -15,6 +15,8 @@ from ViewManager import ViewManager
 import QScintilla.Editor
 import UI.PixmapCache
 
+from UI.E3Action import E3Action
+
 class TabWidget(QTabWidget):
     """
     Class implementing a custimized TabWidget.
@@ -30,8 +32,69 @@ class TabWidget(QTabWidget):
         self.editors = []
         self.curIndex = 0
         
+        self.pixmapActive = UI.PixmapCache.getPixmap("led_green.png")
+        self.pixmapInactive = UI.PixmapCache.getPixmap("led_red.png")
+        self.indicator = QLabel(self)
+        self.indicator.setPixmap(self.pixmapInactive)
+        self.indicator.setFixedSize(self.pixmapInactive.size())
+        self.indicator.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
+        try:
+            self.setCornerWidget(self.indicator, Qt.TopRight)
+        except AttributeError:
+            pass
+        
         self.connect(self, SIGNAL("currentChanged(QWidget *)"), self.handleCurrentChanged)
         
+        self.initMenu()
+        self._tabBar = self.tabBar()
+        self._tabBar.installEventFilter(self)
+        self.contextMenuEditor = None
+        
+    def initMenu(self):
+        """
+        Private method to initialize the tab context menu.
+        """
+        self.menu = QPopupMenu(self)
+        self.menu.insertItem(QIconSet(UI.PixmapCache.getPixmap("close.png")),
+            self.trUtf8('Close'), self.handleContextClose)
+        self.menu.insertItem(self.trUtf8('Close All'), self.handleContextCloseAll)
+        self.menu.insertSeparator()
+        self.saveMenuId = \
+            self.menu.insertItem(QIconSet(UI.PixmapCache.getPixmap("fileSave.png")),
+            self.trUtf8('Save'), self.handleContextSave)
+        self.menu.insertItem(QIconSet(UI.PixmapCache.getPixmap("fileSaveAs.png")),
+            self.trUtf8('Save As...'), self.handleContextSaveAs)
+        self.menu.insertItem(QIconSet(UI.PixmapCache.getPixmap("fileSaveAll.png")),
+            self.trUtf8('Save All'), self.handleContextSaveAll)
+        self.projectMenuId = \
+            self.menu.insertItem(QIconSet(UI.PixmapCache.getPixmap("fileSaveProject.png")),
+                self.trUtf8('Save to Project'), self.handleContextSaveToProject)
+        self.menu.insertSeparator()
+        self.menu.insertItem(QIconSet(UI.PixmapCache.getPixmap("print.png")),
+            self.trUtf8('Print'), self.handleContextPrintFile)
+            
+        self.connect(self.menu, SIGNAL("aboutToShow()"), self.handleShowMenu)
+        
+    def handleShowMenu(self):
+        """
+        Private slot to handle the aboutToShow signal of the tab context menu.
+        """
+        if self.contextMenuEditor:
+            self.menu.setItemEnabled(self.saveMenuId, self.contextMenuEditor.isModified())
+        self.menu.setItemEnabled(self.projectMenuId, qApp.mainWidget().getProject().isOpen())
+    
+    def showIndicator(self, on):
+        """
+        Public slot to set the indicator on or off.
+        
+        @param on flag indicating the dtate of the indicator (boolean)
+        """
+        if on:
+            self.indicator.setPixmap(self.pixmapActive)
+        else:
+            self.indicator.setPixmap(self.pixmapInactive)
+        self.indicator.update()
+        
     def handleCurrentChanged(self):
         """
         Private slot called by the currentChanged signal.
@@ -130,13 +193,84 @@ class TabWidget(QTabWidget):
         """
         return len(self.editors) and 1 or 0
         
+    def handleContextClose(self):
+        """
+        Private method to close the selected tab.
+        """
+        if self.contextMenuEditor:
+            self.parent().handleCloseEditor(self.contextMenuEditor)
+        
+    def handleContextCloseAll(self):
+        """
+        Private method to close all tabs.
+        """
+        savedEditors = self.editors[:]
+        for editor in savedEditors:
+            self.parent().handleCloseEditor(editor)
+        
+    def handleContextSave(self):
+        """
+        Private method to save the selected tab.
+        """
+        if self.contextMenuEditor:
+            self.parent().saveEditorEd(self.contextMenuEditor)
+        
+    def handleContextSaveAs(self):
+        """
+        Private method to save the selected tab to a new file.
+        """
+        if self.contextMenuEditor:
+            self.parent().saveAsEditorEd(self.contextMenuEditor)
+        
+    def handleContextSaveAll(self):
+        """
+        Private method to save all tabs.
+        """
+        self.parent().saveEditorsList(self.editors)
+        
+    def handleContextSaveToProject(self):
+        """
+        Private method to save the selected tab to the current project.
+        """
+        if self.contextMenuEditor:
+            self.parent().saveEditorToProjectEd(self.contextMenuEditor)
+        
+    def handleContextPrintFile(self):
+        """
+        Private method to print the selected tab.
+        """
+        if self.contextMenuEditor:
+            self.parent().printEditor(self.contextMenuEditor)
+        
+    def eventFilter(self, object, event):
+        """
+        Private method called to filter an event of our tabbar.
+        
+        @param object object, that generated the event (QObject)
+        @param event the event, that was generated by object (QEvent)
+        @return flag indicating if event was filtered out
+        """
+        if event.type() == QEvent.MouseButtonPress:
+            if event.button() == Qt.RightButton:
+                tab = self._tabBar.selectTab(event.pos())
+                i = 0
+                self.contextMenuEditor = None
+                while i < self._tabBar.count():
+                    if self._tabBar.tabAt(i) == tab:
+                        self.contextMenuEditor = self.page(i)
+                        break
+                    i += 1
+                if self.contextMenuEditor:
+                    self.menu.exec_loop(self._tabBar.mapToGlobal(event.pos()))
+                    return 1
+            
+        return 0
+        
 class Tabview(QSplitter, ViewManager):
     """
     Class implementing a tabbed viewmanager class embedded in a splitter.
     
-    @signal lastEditorClosed emitted after the last editor window was closed
-    @signal editorOpened emitted after an editor window was opened
-    @signal editorSaved emitted after an editor window was saved
+    @signal changeCaption(string) emitted if a change of the caption is neccessary
     """
     def __init__(self,parent, ui, dbs):
         """
@@ -153,6 +287,7 @@ class Tabview(QSplitter, ViewManager):
         tw = TabWidget(self)
         self.tabWidgets.append(tw)
         self.currentTabWidget = tw
+        self.currentTabWidget.showIndicator(1)
         self.connect(tw, SIGNAL('currentChanged(QWidget*)'),
             self.handleCurrentChanged)
         tw.installEventFilter(self)
@@ -165,16 +300,18 @@ class Tabview(QSplitter, ViewManager):
         """
         ViewManager.initViewActions(self)
         
-        self.nextTabAct = QAction(self.trUtf8('Show next tab'), 
+        self.nextTabAct = E3Action(self.trUtf8('Show next tab'), 
                       self.trUtf8('Show next tab'), 
-                      QKeySequence(self.trUtf8('Ctrl+Alt+Tab')), self)
-        self.connect(self.nextTabAct, SIGNAL('activated()'), self.nextTab)
+                      QKeySequence(self.trUtf8('Ctrl+Alt+Tab')), 0,
+                      self, 'vm_view_next_tab')
+        self.nextTabAct.connectIt( SIGNAL('activated()'), self.nextTab)
         self.viewActions.append(self.nextTabAct)
         
-        self.prevTabAct = QAction(self.trUtf8('Show previous tab'), 
+        self.prevTabAct = E3Action(self.trUtf8('Show previous tab'), 
                       self.trUtf8('Show previous tab'), 
-                      QKeySequence(self.trUtf8('Shift+Ctrl+Alt+Tab')), self)
-        self.connect(self.prevTabAct, SIGNAL('activated()'), self.prevTab)
+                      QKeySequence(self.trUtf8('Shift+Ctrl+Alt+Tab')), 0,
+                      self, 'vm_view_previous_tab')
+        self.prevTabAct.connectIt( SIGNAL('activated()'), self.prevTab)
         self.viewActions.append(self.prevTabAct)
         
     def nextTab(self):
@@ -249,9 +386,18 @@ class Tabview(QSplitter, ViewManager):
         for i in range(self.tabWidgets.index(tw), -1, -1) + \
                  range(self.tabWidgets.index(tw) + 1, len(self.tabWidgets)):
             if self.tabWidgets[i].hasEditors():
+                self.currentTabWidget.showIndicator(0)
                 self.currentTabWidget = self.tabWidgets[i]
+                self.currentTabWidget.showIndicator(1)
                 self.activeWindow().setFocus()
                 break
+            
+        aw = self.activeWindow()
+        fn = aw and aw.getFileName() or None
+        if fn:
+            self.emit(PYSIGNAL('changeCaption'), (unicode(fn),))
+        else:
+            self.emit(PYSIGNAL('changeCaption'), ("",))
     
     def addView(self, win, fn=None):
         """
@@ -272,6 +418,10 @@ class Tabview(QSplitter, ViewManager):
             self.currentTabWidget.setTabToolTip(win, os.path.dirname(fn))
         self.currentTabWidget.showPage(win)
         win.setFocus()
+        if fn:
+            self.emit(PYSIGNAL('changeCaption'), (unicode(fn),))
+        else:
+            self.emit(PYSIGNAL('changeCaption'), ("",))
     
     def showView(self, win, fn=None):
         """
@@ -284,7 +434,9 @@ class Tabview(QSplitter, ViewManager):
         for tw in self.tabWidgets:
             if tw.hasEditor(win):
                 tw.showPage(win)
+                self.currentTabWidget.showIndicator(0)
                 self.currentTabWidget = tw
+                self.currentTabWidget.showIndicator(1)
                 break
         win.setFocus()
     
@@ -317,8 +469,11 @@ class Tabview(QSplitter, ViewManager):
         @param editor editor window to be changed
         @param newName new name to be shown (string or QString)
         """
-        self.currentTabWidget.changeTab(editor, os.path.basename(str(newName)))
-        self.currentTabWidget.setTabToolTip(editor, os.path.dirname(str(newName)))
+        self.currentTabWidget.changeTab(editor, 
+            os.path.basename(unicode(newName)))
+        self.currentTabWidget.setTabToolTip(editor, 
+            os.path.dirname(unicode(newName)))
+        self.emit(PYSIGNAL('changeCaption'), (unicode(newName),))
 
     def handleModificationStatusChanged(self, m, editor):
         """
@@ -332,12 +487,33 @@ class Tabview(QSplitter, ViewManager):
                 break
         if m:
             tw.setTabIconSet(editor, 
-                QIconSet(UI.PixmapCache.getPixmap("fileModified")))
+                QIconSet(UI.PixmapCache.getPixmap("fileModified.png")))
+        elif editor.hasSyntaxErrors():
+            tw.setTabIconSet(editor, 
+                QIconSet(UI.PixmapCache.getPixmap("syntaxError.png")))
         else:
             tw.setTabIconSet(editor, 
-                QIconSet(UI.PixmapCache.getPixmap("empty")))
+                QIconSet(UI.PixmapCache.getPixmap("empty.png")))
         self.checkActions(editor)
         
+    def handleSyntaxErrorToggled(self, editor):
+        """
+        Private slot to handle the syntaxerrorToggled signal.
+        
+        @param editor editor that sent the signal
+        """
+        for tw in self.tabWidgets:
+            if tw.hasEditor(editor):
+                break
+        if editor.hasSyntaxErrors():
+            tw.setTabIconSet(editor, 
+                QIconSet(UI.PixmapCache.getPixmap("syntaxError.png")))
+        else:
+            tw.setTabIconSet(editor, 
+                QIconSet(UI.PixmapCache.getPixmap("empty.png")))
+                
+        ViewManager.handleSyntaxErrorToggled(self, editor)
+        
     def addSplit(self):
         """
         Public method used to split the current view.
@@ -345,7 +521,9 @@ class Tabview(QSplitter, ViewManager):
         tw = TabWidget(self)
         tw.show()
         self.tabWidgets.append(tw)
+        self.currentTabWidget.showIndicator(0)
         self.currentTabWidget = self.tabWidgets[-1]
+        self.currentTabWidget.showIndicator(1)
         self.connect(tw, SIGNAL('currentChanged(QWidget*)'),
             self.handleCurrentChanged)
         tw.installEventFilter(self)
@@ -366,12 +544,16 @@ class Tabview(QSplitter, ViewManager):
             for editor in savedEditors:
                 res &= self.closeEditor(editor)
             if res:
-                i = self.tabWidgets.index(tw)
+                try:
+                    i = self.tabWidgets.index(tw)
+                except ValueError:
+                    return 1
                 if i == len(self.tabWidgets)-1:
                     i -= 1
                 self.tabWidgets.remove(tw)
                 tw.close(1)
                 self.currentTabWidget = self.tabWidgets[i]
+                self.currentTabWidget.showIndicator(1)
                 if len(self.tabWidgets) == 1:
                     self.splitRemoveAct.setEnabled(0)
                 return 1
@@ -395,6 +577,11 @@ class Tabview(QSplitter, ViewManager):
         """
         self.checkActions(editor)
         editor.setFocus()
+        fn = editor.getFileName()
+        if fn:
+            self.emit(PYSIGNAL('changeCaption'), (unicode(fn),))
+        else:
+            self.emit(PYSIGNAL('changeCaption'), ("",))
         
     def eventFilter(self, watched, event):
         """
@@ -404,7 +591,9 @@ class Tabview(QSplitter, ViewManager):
         @param event the event that occurred
         @return always 0
         """
-        if event.type() == QEvent.MouseButtonPress:
+        if event.type() == QEvent.MouseButtonPress and \
+           not event.button() == Qt.RightButton:
+            self.currentTabWidget.showIndicator(0)
             if isinstance(watched, QTabWidget):
                 self.currentTabWidget = watched
             elif isinstance(watched, QTabBar):
@@ -414,10 +603,16 @@ class Tabview(QSplitter, ViewManager):
                     if tw.hasEditor(watched):
                         self.currentTabWidget = tw
                         break
-                        
+            self.currentTabWidget.showIndicator(1)
+            
             aw = self.activeWindow()
             if aw is not None:
                 self.checkActions(aw)
                 aw.setFocus()
+                fn = aw.getFileName()
+                if fn:
+                    self.emit(PYSIGNAL('changeCaption'), (unicode(fn),))
+                else:
+                    self.emit(PYSIGNAL('changeCaption'), ("",))
             
         return 0
