@@ -4,7 +4,7 @@
 #
 
 """
-Module implementing the writer class for writing an XML project file..
+Module implementing the writer class for writing an XML project file.
 """
 
 import os
@@ -51,6 +51,13 @@ class ProjectWriter(XMLWriterBase):
         # add the main tag
         self._write('<Project version="%s">' % projectFileFormatVersion)
         
+        # do the programming language
+        self._write('  <ProgLanguage mixed="%d">%s</ProgLanguage>' % \
+            (self.pdata["MIXEDLANGUAGE"][0], self.pdata["PROGLANGUAGE"][0]))
+        
+        # do the UI type
+        self._write('  <UIType>%s</UIType>' % self.pdata["UITYPE"][0])
+        
         # do description
         if self.pdata["DESCRIPTION"]:
             self._write("  <Description>", 0)
@@ -58,12 +65,21 @@ class ProjectWriter(XMLWriterBase):
                 self._write(escape("%s" % d.strip()))
             self._write("%s</Description>" % escape(self.pdata["DESCRIPTION"][-1].strip()))
         
-        #do version, author and email
+        # do version, author and email
         for key in ["VERSION", "AUTHOR", "EMAIL"]:
             element = key.capitalize()
             if self.pdata[key]:
                 self._write("  <%s>%s</%s>" % (element, escape(self.pdata[key][0]), element))
             
+        # do the translation prefix
+        if self.pdata["TRANSLATIONPREFIX"]:
+            self._write("  <TranslationPrefix>")
+            parts = self.pdata["TRANSLATIONPREFIX"][0].split(os.sep)
+            for p in parts[:-1]:
+                self._write("    <Dir>%s</Dir>" % p)
+            self._write("    <Name>%s</Name>" % parts[-1])
+            self._write("  </TranslationPrefix>")
+        
         # do the sources
         self._write("  <Sources>")
         for name in self.pdata["SOURCES"]:
