@@ -22,6 +22,8 @@ from DocumentationTools.IndexGenerator import IndexGenerator
 from UI.Info import Version
 import Utilities
 
+supportedExtensions = [".py", ".ptl", ".rb"]    # list of supported filename extensions
+
 def usage():
     """
     Function to print some usage information.
@@ -95,6 +97,12 @@ def main():
     doIndex = 1
     noempty = 0
 
+    # Set the applications string encoding
+    try:
+        sys.setappdefaultencoding("utf-8")
+    except:
+        pass
+    
     for k, v in opts:
         if k in ["-p", "--prefix"]:
             prefix = v
@@ -155,14 +163,17 @@ def main():
             names = [arg]
     
         for filename in names:
-    
+            inpackage = 0
             if os.path.isdir(filename):
-                files = glob.glob(os.path.join(filename, Utilities.joinext("*", ".py"))) + \
-                        glob.glob(os.path.join(filename, Utilities.joinext("*", ".ptl")))
-                initFile = os.path.join(filename, Utilities.joinext("__init__", ".py"))
-                if initFile in files:
-                    files.remove(initFile)
-                    files.insert(0, initFile)
+                files = []
+                for ext in supportedExtensions:
+                    files.extend(glob.glob(os.path.join(filename,
+                                                        Utilities.joinext("*", ext))))
+                    initFile = os.path.join(filename, Utilities.joinext("__init__", ext))
+                    if initFile in files:
+                        inpackage = 1
+                        files.remove(initFile)
+                        files.insert(0, initFile)
             else:
                 if sys.platform == "win32" and glob.has_magic(filename):
                     files = glob.glob(filename)
@@ -172,7 +183,8 @@ def main():
             for file in files:
     
                 try:
-                    module = Utilities.ModuleParser.readModule(file, basename=basename)
+                    module = Utilities.ModuleParser.readModule(file, basename=basename, 
+                                                               inpackage=inpackage)
                     moduleDocument = ModuleDocument(module)
                     doc = moduleDocument.genDocument()
                 except IOError, v:
