@@ -53,7 +53,7 @@ def installTranslations():
     else:
         targetDir = getConfig('ericDir')
     
-    for fn in glob.glob(os.path.join('eric', '*.qm')):
+    for fn in glob.glob(os.path.join('eric', 'i18n', '*.qm')):
         shutil.copy2(fn, targetDir)
     
 def main(argv):
